/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * Common functions for Preview and File generation
 ********************************************************************/
#include "xrit2pic.h"
#include <malloc.h>

extern PREFER prefer;
extern GLOBINFO globinfo;

/***********************************************
 ***********************************************
 ** Common help-functions for presenting xrit info
 ***********************************************
 ***********************************************/


static int bpl2str(SEGMENT *segm,guint16 **str)
{
  FILE *fpi;
  int width=836;
  int height=682;
  int x,y,n;
  unsigned char c;
  if (!(fpi=open_xritimage(segm->pfn,NULL,segm->xh.xrit_frmt,segm->chan)))
  {
    segm->corrupt=TRUE;         /* cleared with new generation */
    return 1; 
  }

  width=segm->xh.nc;
  height=segm->xh.nl;
  segm->width=width;
  segm->height=height;

  *str=malloc(height*width*2);
  n=0;
  for (y=0; y < height; y++)
  {
    for (x=0; x<width; x++)
    {
      if (segm->xh.nb==1)
      {
        if (!(n%8))
          fread(&c,1,1,fpi);
        if (c&(0x80>>(x%8)))
          *((*str)+x+y*width)=0xff;
        else
          *((*str)+x+y*width)=0x00;
      }
      else
      {
        fread(&c,1,1,fpi);
        *((*str)+x+y*width)=c;
      }
      n++;
    }
  }

  fclose(fpi);

  return 0;
}

static void add_overlay_to_segm(SEGMENT *segm,PIC_CHARS *pc)
{
  if (!segm->ovlchnk)
  {
    segm->ovlchnk=calloc(segm->width*segm->height,sizeof(*segm->ovlchnk));
  }
  if ((pc) && (pc->orbittype==POLAR))
  {
    if (pc->gmap==normal)
    {
      polar_add_lonlat(segm,pc);
    }
//      else
//        polar_add_lonlat(segm,pc);
  }
  else
  {
    geo_add_lonlat(&prefer,segm,pc);
  }

/* Add overlay.*/
// Always do this (except for polar sats), so overlay may be enabled lateron in preview menu.
  if ((!pc) || (pc->orbittype!=POLAR))
  {
// oorspr.: alleen add_overlay als verandering. Gaat mis met cities, mark enz.
// Maakt het heel er traag voor HRV!
// test 'segm->loaded_ovl!=globinfo.pm.ol.overlaytype' gaat mis met mark nogui!
//    if ((segm) && (segm->loaded_ovl!=globinfo.pm.ol.overlaytype))
    {
      OVERLAYTYPE actual_overlay;
      actual_overlay=add_overlay(&prefer,globinfo.pm.ol.overlaytype,segm,pc);
      if (actual_overlay>0) globinfo.pm.ol.overlaytype=actual_overlay;
    }
  }
}

// matches with manage_aspect
static void correct_for_aspect(SEGMENT *segm)
{
  int wi,hi,wo,ho;
  int xi,yi,xo,yo;
  float xaspectcnt,yaspectcnt;
  float aspect_ratio=1.;
  guint16 *chnk;
  if ((!segm->chan->nc_orig) && (!segm->chan->nl_orig)) return; // no aspect correction
  if (segm->chan->nc_orig) wi=segm->chan->nc_orig; else wi=segm->chan->nc;
  if (segm->chan->nl_orig) hi=segm->chan->nl_orig; else hi=segm->chan->nl;
  wo=segm->chan->nc;
  ho=segm->chan->nl;
  if (wo>wi) aspect_ratio=(float)wi/(float)wo;
  if (ho>hi) aspect_ratio=(float)ho/(float)hi;

  chnk=malloc(wo*ho*sizeof(*chnk));
  yaspectcnt=0.;
  yo=0;
  for (yi=0; yi<hi; yi++)
  {
    int yrpt=1;
    if (aspect_ratio > 1.)
    {
      yaspectcnt= yaspectcnt+aspect_ratio-1.;
      if (yaspectcnt > 1.)
      {
        yrpt=(int)yaspectcnt;
        yaspectcnt-=yrpt;
        yrpt++;
      }
    }
    while (yrpt>0)
    {
      xaspectcnt=0;
      xo=0;
      for (xi=0; xi<wi; xi++)
      {
        int xrpt=1;
        if (aspect_ratio < 1.)
        {
          xaspectcnt= xaspectcnt+aspect_ratio;
          if (xaspectcnt > 1.)
          {
            xrpt=(int)xaspectcnt;
            xaspectcnt-=xrpt;
            xrpt++;
          }
        }
        while (xrpt>0)
        {
          if ((xo < wo) && (yo < ho))
            chnk[xo+yo*wo]=segm->chnk[xi+yi*wi];
          xo++;
          xrpt--;
        }
      }
      yo++;
      yrpt--;
    }
  }
  free_segmchnk(segm); // alos free's ovlchnk which isn't allocated yet anyway
  segm->chnk=chnk;
}

/***********************************************
 * Translate picture (wvt, jpeg, pgm, ...) into 
 *   segm->chnk.
 * polar: segm of all 5 channels loaded.
 ***********************************************/
int pic2str(SEGMENT *segm,PIC_CHARS *pc)
{
  int err=0;
  gboolean all_chan=TRUE;
  if (!segm) return Mis_segm;
  if (segm->corrupt) return Mis_segm;    /* corrupt cleared with new generation */
  if (!segm->chnk) // asumes all channels of this segm not loaded yet!
  {
    int nc=segm->chan->nc;
    int nl=segm->chan->nl;
    if (prefer.keep_unzipped)
    {
      unzip_segmentfile(segm);
    }
    // restore orig. sizes
    if (segm->chan->nc_orig)
      segm->chan->nc=segm->chan->nc_orig;
    if (segm->chan->nl_orig)
      segm->chan->nl=segm->chan->nl_orig;

    switch (segm->chan->ifrmt)
    {
      case 'j': err=jpg2str(segm,&segm->chnk);   break;
      case 'p': err=pgm2str(segm,&segm->chnk);   break;
      case 'w': err=wvt2str(segm,&segm->chnk);   break;
      case 'z': err=noaa2str(segm,all_chan,pc);  break;
      case 'Z': err=eps2str(segm,all_chan,pc);   break;
      case 'y': err=eps2str(segm,all_chan,pc);   break;
      case 'b': err=bpl2str(segm,&segm->chnk);   break;
      default : err=dump_data(segm,&segm->chnk); break; /* unknown format */
    }
    // back to adapted sizes
    segm->chan->nc=nc;
    segm->chan->nl=nl;

    // matches with manage_aspect
    if (globinfo.correct_aspect) correct_for_aspect(segm); 
    
    if ((segm->width==0) || (segm->height==0)) err=1;
    segm->loaded_ovl=0;

    if (!err)
    {
      add_overlay_to_segm(segm,pc);
    }
  }

  return err;
}
