/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/********************************************************************
 * int jpeg2avi(FILELIST *fl,char *mfno,long fps,int repeat_lastframe)
 * int add_frame(char *fn_movie,char *fn_frame,int max_frames,long fps,int repeat_lastframe)
 ********************************************************************/

#include "xrit2pic.h"
#include "avifrmt.h"
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>

#include <unistd.h>

/******************************************************************************
Opbouw AVI:
// rifsize = filesize-8 (=RIFF <riff_sz>)
// <AVI ><avi_hdr><movi hdr><db-hdr><frame1><db-hdr><frame2>.....<db-hdr><frameN><idx1-hdr><list1><list2>...<listN>
//   4             3x4       Nx8 +  jpg_totsize                                   2x4       4x4xN 

<db-hdr>= "00db"<fl->size> (tot. 8 bytes) (fl->size=jpeg size afgerond naar 4 bytes!)s

Relatie avi/FILELIST:
bytes		avi			betekenis
 --------------------
  0-  3		RIFF
  4-  7		  b0b1b2b3		# bytes in file
  8- 12		  'AVI ' 
 13- 15		  LIST
 16- 19		    d0000000            ? size=13?
 20- 23		    hdrl
 24- 27		    avih
 28- 31		      38000000		size: 32-87
 32- 35		      b0b1b2b3		# speed
 36- 39		      b0b1b2b3		# max_bytes_per_sec
 40- 43		      00000000		padding
 44- 47		      10000000		flags=AVIF_HASINDEX
 48- 51		      b0b1b2b3		nr frames
 52- 55		      00000000		init_frames
 56- 59		      01000000		streams
 60- 63		      00000000		buff_sz
 64- 67		      b0b1b2b3		width
 68- 71		      b0b1b2b3		height
 72- 87		      00....00		reserved[4]
 88- 91		    LIST
 92- 95		      84000000		size hdr: 96-227
 96- 99		      strl		type
100-103		      strh		
104-107		        30000000	size strh: 108-155
108-115		        vidsMJPG
116-127		        00..00
128-131		        b0b1b2b3	# speed zie 32- 35
132-155		        ..
156-159		      strf
160-163		        28000000	size: 164-203	
164-167		        28000000
168-171		        b0b1b2b3	width
172-175		        b0b1b2b3	height
176-179		        01001800
180-183		        MJPEG
184-187		        b0b1b2b3	width * height * 3
...
204-207		      LIST
208-211		        10000000	size: 212-227
212-219		        'odmldmlh'
220-223		        04000000
224-227		        05000000
		  'JUNK'
                    c0000000		size
                    'erpt'
                      40000000
                      b0b1b2b3		nr repeats end
228-231		  LIST
232-235		    b0b1b2b3		jpeg_size+8*frames + 4
236-239		    'movi'
-------------------------------------------------------
		      00db		uncompressed video frame 1
                        b0b1b2b3	fl->size
                        ffd8ffe00010	1e 6 bytes jpeg
		        AVI1		vervangt bytes 7...10 van JPEG
                        ......		rest jpeg
-------------------------------------------------------
		      00db		uncompressed video frame 2
                        b0b1b2b3	fl->size
                      (enz.)
-------------------------------------------------------
		  idx1			indextbl
                    b0b1b2b3		size: 16 * nr_indexpntr

  		    00db		
                      12000000
                      b0b1b2b3		offset (eerste=04)
                      b0b1b2b3		size
  		    00db		
                      12000000
                      b0b1b2b3		offset
                      b0b1b2b3		size
		....

 ******************************************************************************/
// store size of file 'fl->fn' in fl->size; start pos in fl->offset
static void get_1file_sz(FILELIST *fl)
{
  struct stat s;
  if (stat(fl->fn,&s)==-1)
  {
    fl->size=0;
    fl->offset=4;
  }
  else
  {
    fl->size=s.st_size;
    fl->size+=((4-(fl->size%4)) % 4); /* size file, 4-bytes rounded */

    if (fl->prev)
      fl->offset=fl->prev->offset+fl->prev->size+8;  // 8: extra bytes: "00db" + 4 bytes size
    else
      fl->offset=4;
  }
}

// write 4-bytes little endian
static void fwrite_quartet(FILE *fpo,unsigned long i)
{
  i=GUINT32_TO_LE(i);
  fwrite(&i,4,1,fpo);
}

// read 4-bytes little endian
static unsigned long fread_quartet(FILE *fp,unsigned long *i)
{
  int r;
  unsigned long tmp;
  r=fread(&tmp,4,1,fp);
  if (i)
    *i=GUINT32_FROM_LE(tmp);
  return r;
}

/*
copy 1 frame from avi.
Start with:
  "00db" <size_4bytes> <jpeg_6bytes> <AVI > <rest_jpeg>
("AVI " vervangt bytes 7...10 van JPEG!)
tot. size 1 frame = size_jpeg+8
*/
static void write_frame_cpy(FILE *fpi,FILELIST *fl,FILE *fpo,int hdrlen)
{
  char buff[512];
  int nrbytes=fl->size+8;
  int i;
  fseek(fpi,hdrlen+fl->offset-4,SEEK_SET);   // -4: voor  "00db"
  for (i=0; i<fl->size+8;)               // +8: incl. "00db"<file_size>
  {
    int nrrd;
    nrrd=MIN(nrbytes,512);
    fread(buff,1,nrrd,fpi);
    fwrite(buff,1,nrrd,fpo);
    nrbytes-=nrrd;
    i+=nrrd;
  }
}

/*
convert jpeg to 1 frame:
"00db" <size_4bytes> <jpeg_6bytes> <AVI > <rest_jpeg>
("AVI " vervangt bytes 7...10 van JPEG!)
tot. size 1 frame = size_jpeg+8
*/
static int write_frame(FILELIST *fl,FILE *fpo,int avi_width,int avi_height)
{
  FILE *fp;
  int nbw,nbr;
  char buff[512];
  int pic_width,pic_height;
  
  fprintf(fpo,"00db");                            // => "00db" (4 bytes)
  fwrite_quartet(fpo,fl->size);                   // => size   (4 bytes)

  /* Now copy jpeg file into RIFF file */
  if (!((fp = fopen(fl->fn, "rb")))) return -6;
  get_jpegsize_opened(fp,&pic_width,&pic_height);        // catch size

  if ((pic_width!=avi_width) || (pic_height!=avi_height))
  {
    fclose(fp);
    return -99;
  }

  nbw = 0;
  if ((nbr = fread(buff,1,6,fp)) != 6)
  {
    fclose(fp);
    return -7;
  }
  fwrite(buff, nbr, 1, fpo);                      // => 6 bytes van jpeg: ffd8 ffe0 0010
  nbw+=6;

  fread(buff,1, 4,fp);                            // skip 4 bytes van jpeg
  fwrite("AVI1", 4, 1, fpo);                      // => "AVI " (4 bytes)
  nbw+=4;

  while ((nbr = fread(buff, 1,512,fp)) > 0)       // rest jpeg
  {
    fwrite(buff, nbr, 1, fpo);
    nbw += nbr;
  }
  if (nbw < fl->size)
  {
    fwrite(buff, 1, fl->size-nbw, fpo);
  }
  fclose(fp);
  return 0;
}

// initialize avi header: LIST parts
static struct AVI_list_hdr init_avihdr(char *id,unsigned long size,char *type)
{
  struct AVI_list_hdr h;
  memcpy(h.id,id,4);
  h.sz=GUINT32_TO_LE(size);
  memcpy(h.type,type,4);
  return h;
}

// initialize avi header
static struct AVI_list_hdrl init_avihdrl(unsigned long jpg_sz,int frames,long us_per_frame,int width,int height)
{
  struct AVI_list_hdrl h;
  if (frames<1) frames=1;             // may never occur; prevent crash
  if (us_per_frame<1) us_per_frame=1;
  memset(&h,0,sizeof(h));
  /* struct AVI_list_hdr */
  h.list_hdr=init_avihdr("LIST",sizeof(struct AVI_list_hdrl) - 8,"hdrl");
  
  /* chunk avih */
  memcpy(h.avih_id,"avih",4);
  h.avih_sz=GUINT32_TO_LE(sizeof(struct AVI_avih));

  /* struct AVI_avih */
  h.avih.us_per_frame=GUINT32_TO_LE(us_per_frame);
  h.avih.max_bytes_per_sec=GUINT32_TO_LE(1000000 * (jpg_sz/frames) / us_per_frame);
  h.avih.flags=GUINT32_TO_LE(AVIF_HASINDEX);
  h.avih.tot_frames=GUINT32_TO_LE(frames);
  h.avih.streams=GUINT32_TO_LE(1);
  h.avih.width=GUINT32_TO_LE(width);
  h.avih.height=GUINT32_TO_LE(height);
     
  /* struct AVI_list_strl */
  h.strl.list_hdr=init_avihdr("LIST",GUINT32_TO_LE(sizeof(struct AVI_list_strl)-8),"strl");

  /* chunk strh */
  memcpy(h.strl.strh_id,"strh",4);
  h.strl.strh_sz=GUINT32_TO_LE(sizeof(struct AVI_strh)),

  /* struct AVI_strh */
  memcpy(h.strl.strh.type,"vids",4);
  memcpy(h.strl.strh.handler,"MJPG",4);
  h.strl.strh.scale=GUINT32_TO_LE(us_per_frame);
  h.strl.strh.rate=GUINT32_TO_LE(1000000);
  h.strl.strh.length=GUINT32_TO_LE(frames);
  
  /* chunk strf */
  memcpy(h.strl.strf_id,"strf",4);
  h.strl.strf_sz=sizeof(struct AVI_strf);

  /* struct AVI_strf */
  h.strl.strf.sz=GUINT32_TO_LE(sizeof(struct AVI_strf));
  h.strl.strf.width=GUINT32_TO_LE(width);
  h.strl.strf.height=GUINT32_TO_LE(height);
  h.strl.strf.planes_bit_cnt=GUINT32_TO_LE(1 + 24*256*256);
  memcpy(h.strl.strf.compression,"MJPG",4);
  h.strl.strf.image_sz=GUINT32_TO_LE(width * height * 3);

  /* struct AVI_list_odml */
  h.strl.list_odml.list_hdr=init_avihdr("LIST",16,"odml");
  memcpy(h.strl.list_odml.id,"dmlh",4);
  h.strl.list_odml.sz=GUINT32_TO_LE(4);
  h.strl.list_odml.frames=GUINT32_TO_LE(frames);
  return h;
}

static int read_chunkhdr(FILE *fp,unsigned long *size)
{
  char byte4[4];
  fread(byte4,1,4,fp);
  fread(size,1,4,fp);
  *size=GUINT32_FROM_LE(*size);
  if (!strncmp(byte4,"RIFF",4))   return 'R';
//  if (!strncmp(byte4,"AVI ",4)) return 'A';
  if (!strncmp(byte4,"LIST",4))   return 'L';
  if (!strncmp(byte4,"JUNK",4))   return 'J';
  return 0;
}

static int read_hdr(FILE *fp,long *frames,long *nr_rptend,long *jpg_sz,long *us_per_frame,int *width,int *height)
{
  struct AVI_list_hdrl hdrl;
  char byte4[4];
  char t;
  unsigned long size;
  unsigned long data;

  t=read_chunkhdr(fp,&size);                                      // "RIFF"
  if (t!='R') return -1;
  fread(byte4,1,4,fp);                                            // "AVI " 

  fread(&hdrl, sizeof(hdrl), 1, fp);
  *frames=GUINT32_FROM_LE(hdrl.avih.tot_frames);

  // expect possibly JUNK here
  t=read_chunkhdr(fp,&size);                                      // "JUNK or LIST"
  if ((t=='J') && (size>=4))
  {
    int n;
    for (n=0; n<size;)
    {
      fread(byte4,1,4,fp);  n+=4; 
      if ((!strncmp(byte4,"erpt",4)) && (n<size))
      {
        fread(byte4,1,4,fp);      n+=4;                           // size
        fread(&data,1,4,fp);  n+=4;
        if (nr_rptend) *nr_rptend=GUINT32_FROM_LE(data);
      }
    }
  }
  else
  {
    fseek(fp,-8,SEEK_CUR);
  }
  fread(byte4,1,4,fp);                                            // "LIST" 
  fread_quartet(fp,(unsigned long *)jpg_sz);
  *jpg_sz-=(8*(*frames) + 4);


  *us_per_frame=GUINT32_FROM_LE(hdrl.avih.us_per_frame);
  *width=GUINT32_FROM_LE(hdrl.avih.width);
  *height=GUINT32_FROM_LE(hdrl.avih.height);
  return 0;
}

// fl expected NULL
// nrframes=total amount index pointers, incl. repeated
// indextabel:
//   69 64 78 31 50 00 00 00 = idx1P...
// 
//   vast vast     offset      size
//   00db 12000000 04 00 00 00 18 64 1e 00
//   00db 12000000 24 64 1e 00 18 64 1e 00
//   ....
//   00db 12000000 84 90 79 00 18 64 1e 00
//
static int read_indextbl(FILE *fp,long nrframes,FILELIST **fl)
{
  FILELIST *fl1=NULL;
  char byte4[4];
  int i;
  // start offset-table is at 16*nrframes from end
  fseek(fp,(-16)*nrframes,SEEK_END);

  for (i=0; i<nrframes; i++)
  {
    unsigned long offset,size;
    fread(byte4,1,4,fp); if (strncmp(byte4,"00db",4)) return -1;
    fread_quartet(fp,NULL); // 18
    fread_quartet(fp,&offset);
    fread_quartet(fp,&size);


    // here fl1=previous added index item.
    // (offset altijd verschillend; zelfde offset voor repeated werkt niet!)
    if ((!fl1) || (fl1->offset!=offset)) // only add to list if different offset. Ignore repeats.
    {
      fl1=Create_Filelist(fl,NULL);
      fl1->offset=offset;
      fl1->size=size;
    }
  }
  return 0;
}

// 20 bytes
static void fwrite_erpt(FILE *fp,int erpt)
{
  fprintf(fp,"JUNK");
  fwrite_quartet(fp,12);
  fprintf(fp,"erpt");
  fwrite_quartet(fp,4);
  fwrite_quartet(fp,erpt);
}

// avi: <hdr> <frame1> .. <framen> <indextable>
// write the header
static void write_hdr(long frames,int erpt,long jpg_sz,long riff_sz,long us_per_frame,int width,int height,FILE *fpo)
{
  struct AVI_list_hdrl hdrl;
  
  hdrl=init_avihdrl(jpg_sz,frames,us_per_frame,width, height);

  if (erpt) riff_sz+=20;
  /* printing AVI riff hdr */
  fprintf(fpo,"RIFF");
  fwrite_quartet(fpo,riff_sz);
  fprintf(fpo,"AVI ");

  fwrite(&hdrl, sizeof(hdrl), 1, fpo);
  if (erpt) fwrite_erpt(fpo,erpt);
  /* list movi */
  fprintf(fpo,"LIST");
  fwrite_quartet(fpo,jpg_sz + 8*frames + 4);
  fprintf(fpo,"movi");
}

// write the indextable
// if fpo=NULL: just count nr. of index table entries
static int write_indextbl(FILELIST *fl,FILE *fpo)
{
  FILELIST *fl1;
  int nr_indexpntr=0;
  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->dont_use) continue;
    nr_indexpntr++;
    if (fpo)
    {
      fprintf(fpo,"00db");
      fwrite_quartet(fpo,18);
      fwrite_quartet(fpo,fl1->offset);
      fwrite_quartet(fpo,fl1->size);
    }
  }
  return nr_indexpntr;
}

// check filesize: extract from new to add jpeg size; must be euqal to what is in movie.
//   if equal to (width,height), so OK,  then return 0
static int check_picsize_file(char *fn,int width,int height)
{
  int pic_width,pic_height;
  get_jpegsize(fn,&pic_width,&pic_height);        // catch size
//printf("%d  %d  %d  %d\n",pic_width,width,pic_height,height);
  if ((pic_width!=width) || (pic_height!=height)) return 1;
  return 0;
}

// check size of all jpegs to add
static int check_sizes(FILELIST *fl,int width,int height)
{
  int err=0;
  for (; fl; fl=fl->next)
  {
    if (check_picsize_file(fl->fn,width,height))
    {
      fl->dont_use=TRUE;
      err++;
    }
    get_1file_sz(fl);
  }
  return err;
}

/*
 * AVI format:
 *   <avi_header>          write_hdr
 *   <frames>              write_frame
 *   <index_table>         write_indextbl
 */
static int fl2avi_pre(FILE *fpo,FILELIST *fl,int erpt,int uspfr,int width,int height)
{
  FILELIST *fl1;
  int nrtblentries=0;
  int nrframes=0;
  unsigned long jpg_totsize=0;
  unsigned long riff_size=0;

  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->dont_use) continue;
    nrtblentries++;

    if (!fl1->repeated)
    {
      nrframes++;
      jpg_totsize+=fl1->size;
    }
  }

// rifsize = filesize-8 (8=RIFF <riff_sz>)
// <AVI ><avi_hdr><movi hdr><db-hdr><frame1><db-hdr><frame2>.....<db-hdr><frameN><idx1-hdr><list1><list2>...<listN>
//   4             3x4       Nx8 +  jpg_totsize                                   2x4       4x4xN 
  riff_size = sizeof(struct AVI_list_hdrl) + 4 + 4 + jpg_totsize + 8*nrframes + 8 + 8 + 16*nrtblentries;
  write_hdr(nrtblentries, erpt,jpg_totsize,riff_size,uspfr, width, height,fpo);
  return nrframes;
}


static int fl2avi_post(FILE *fpo,FILELIST *fl)
{
  /* indices */
  int nr_indexpntr;
  nr_indexpntr=write_indextbl(fl,NULL);
  fprintf(fpo,"idx1");
  fwrite_quartet(fpo,16 * nr_indexpntr);
  write_indextbl(fl,fpo);
  return nr_indexpntr;
}


// combine all fl's into 1 movie file
// mfno: file of new avi file
// fps: frames per second
// repeat_lastframe: NOT USED HERE! 0: no repeat, 1:  last frame repeated 1x etc.
int jpeg2avi(FILELIST *fl,char *mfno,long fps,int repeat_lastframe)
{
  int nrframes,nrframes_act;
  int width=10;
  int height=10;
  long uspfr;
  FILELIST *fl1;

  FILE *fpo;
  if (!fl) return 0;

  // repeat this way doesn't work with mplayer! (index table with multiple pointers to same frame)
  repeat_lastframe=0;

  if (fps<=0) fps=1;
  
  uspfr=1000000/fps;
  // check that all pics in fl-chain have same size
  get_jpegsize(fl->fn,&width,&height); // catch size first
  if (check_sizes(fl,width,height)) return Avi_picsize; // check all fl's have same size; if not: error! <retval = <0)

  if (!(fpo=fopen(mfno,"wb"))) return Avi_write;
//  add_repeated(fl,0,repeat_lastframe);
  nrframes=fl2avi_pre(fpo,fl,0,uspfr,width,height);
  nrframes_act=0;
  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->dont_use) continue;
    if (fl1->repeated) continue;
    
    if ((write_frame(fl1,fpo,width,height))==0) nrframes_act++;
  }

  // write index table
  fl2avi_post(fpo,fl);

  fclose(fpo);
  if (nrframes_act!=nrframes)
  {
    return Avi_nrframes;
  }
  return nrframes;
}

// Mark files in table to remove
static void tag_frames_to_remove(FILELIST *fl,int nr_rmframes_begin, int nr_rmframes_end)
{
  FILELIST *fl1;

  // tag frames at end to remove
  for (fl1=fl; ((fl1) && (fl1->next)); fl1=fl1->next); // to end
  for (; ((nr_rmframes_end) && (fl1)); fl1=fl1->prev)
  {
    if (fl1->dont_use) continue;
    fl1->dont_use=TRUE;
    nr_rmframes_end--;
  }

  // tag frames at begin to remove
  for (fl1=fl; ((nr_rmframes_begin>0) && (fl1)); fl1=fl1->next)
  {
    if (fl1->dont_use) continue;
    fl1->dont_use=TRUE;
    nr_rmframes_begin--;
  }
}

// size of avi header
static int size_of_avihdr(FILE *fp)
{
  long nrframes,jpg_totsize,uspfr;
  int width,height;
  if (read_hdr(fp,&nrframes,NULL,&jpg_totsize,&uspfr,&width,&height)) return 0;
  return ftell(fp);
}

static int new_jpeg2avi(char *fn_movie,char *fn_frame,long fps)
{
  FILELIST *fl=NULL;
  int nrframes;
  Create_Filelist(&fl,fn_frame);         // create list of files (actual, 1 file=first frame)
  nrframes=jpeg2avi(fl,fn_movie,fps,0);  // generate avi (contains 1 frame)
  Remove_Filelist(fl);
  return (int)nrframes;
}

// Present movie: remove repeated frames, add new frame 
static int adapt_movie(char *fn_movie,FILELIST *fl,int erpt,long uspfr,int width,int height)
{
  FILE *fpi;
  FPFNTMP fpfntmp=clear_fpfntmp();
  FILELIST *fl1;
  int nrframes,nrframes_act;
  int p_offset;
  int hdrlen;
  if (!(fpi=fopen(fn_movie,"rb"))) return -1; // Open old movie. This file fits with prev. readed filelist 'fl'!

  fpfntmp=cropen();                           // create and open temp file for write
  if (!fpfntmp.fp)
  {
    rmclose(&fpfntmp);
    fclose(fpi);
    return -2;
  }    
  hdrlen=size_of_avihdr(fpi);                 // header contains size and table of frame-pointers
  hdrlen+=4;                                  // set to start first frame

  // write AVI header to new file (tfpo)
//  add_repeated(fl,0,repeat_lastframe);
  // nrframes=tot. # frames including repeated frames
  nrframes=fl2avi_pre(fpfntmp.fp,fl,erpt,uspfr,width,height);
  nrframes_act=0;

  // determine offset first frame to add
  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->new) continue;
    if (!fl1->dont_use) break;
  }

  // start copying frames from fpi to tpfo
  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->new) continue;
    if (fl1->dont_use) continue;
    if (fl1->repeated) continue;
    write_frame_cpy(fpi,fl1,fpfntmp.fp,hdrlen);
    nrframes_act++;
  }

  // calculate new offsets
  for (fl1=fl,p_offset=0; fl1; fl1=fl1->next)
  {
    if (fl1->dont_use) continue;
    if (p_offset)
      fl1->offset=p_offset+fl1->prev->size+8;  // 8: extra bytes: "00db" + 4 bytes size
    else
      fl1->offset=4;
    p_offset=fl1->offset;
  }

  // add new frames from jpeg
  for (fl1=fl; fl1; fl1=fl1->next)
  {
    if (fl1->new)
    {
      if ((write_frame(fl1,fpfntmp.fp,width,height))<0); //  err++; // add new frame to avi
    }
  }
  nrframes_act+=erpt;

  // finally write index table
  fl2avi_post(fpfntmp.fp,fl);

  fclose(fpi);
  fclose(fpfntmp.fp); fpfntmp.fp=NULL;
  // copy movie from temp-file to original one, overwriting it
  copy_file(fpfntmp.fn,fn_movie);
  rmclose(&fpfntmp);
  
  if (nrframes_act!=nrframes)
  {
//    return Avi_nrframes;  // error in nr. of expected frames
  }
  return nrframes;
}

// return: mi, containing:
//    repeated_at_end (from 'JUNK' part added at prev. generation to avi file)
//    nrframes: total nr. of  frames (inluding repeated frames)
static MOVIEINFO get_filelist_if_usable_avi(char *fn_movie,char *fn_frame)
{
  MOVIEINFO movieinfo;
  FILELIST *fl=NULL;
  FILE *fp;
  long uspfr;
  long jpg_totsize;
  int width, height;
  long nrframes;
  long erpt=0;
  
  memset(&movieinfo,0,sizeof(movieinfo));
  if (!(fp=fopen(fn_movie,"rb")))
  {
    return movieinfo;    // avi file doesn't exist (yet)
  }

  // read header avi file
  // nrframes=tot. amount of frames (including repeated frames)
  // erpt =  # repeats last frame
  if (read_hdr(fp,&nrframes,&erpt,&jpg_totsize,&uspfr,&width,&height))
  {
    fclose(fp); 
    return movieinfo;                              // wrong format; fn_movie unusable
  }
  // check if new frame has same size as frames in avi.
  if (check_picsize_file(fn_frame,width,height))
  {
    fclose(fp);
    return movieinfo;                              // different size; fn_movie unusable
  }

  // read index -> fl
  if (read_indextbl(fp,nrframes,&fl))
  { // wrong format; error
    Remove_Filelist(fl);
    fclose(fp); 
    return movieinfo;                              // wrong index table; fn_movie unusable
  }
  fclose(fp);
  movieinfo.fl=fl;
  movieinfo.width=width;
  movieinfo.height=height;
  movieinfo.nrframes=nrframes;
  movieinfo.repeated_at_end=erpt;
  movieinfo.uspfr=uspfr;
  return movieinfo;                                      // usable avi
}

int speed_minpsec2fps(SATLIST *sl,int min_p_sec)
{
  int fps;
  if (sl)
    fps=(sl->min_per_pic ? min_p_sec/sl->min_per_pic : 1);
  else
    fps=min_p_sec;
  fps=MAX(fps,1);
  return fps;
}

// Add frame from jpeg 'fn_frame' to existing movie.
// max_frames>0: limit to this amount
// repeat_lastframe: last frame added will be repeated (these repeats removed if a new frame is added)
//
// fn_movie        : name of existing movie to add frame to (if not exist: new movie started)
// fn_frame        : filename jpeg file to add
// max_frames      : limit to this amount of frames (0: mo limit)
// fps             : frames per second
// repeat_lastframe: last frame repeated to fake pauze at end of movie
//                   repeats are removed if next frame is added, so don't change 
//                     repeat_lastframe during process of making avi!
// Return nr. 'unique' frames
int add_frame(char *fn_movie,char *fn_frame,int max_frames,long fps,int pauze_at_end)
{
  MOVIEINFO mi;
  FILELIST *flnw;
  int i;
  int repeat_lastframe;
  int rm_at_end,rm_at_start;

  // get filelist if usable avi, else NULL
  mi=get_filelist_if_usable_avi(fn_movie,fn_frame); // filelist from old' movie file
  prlog("header avifile: width=%d  height=%d nrframes=%d  repeated_at_end=%d  usec per frame=%d\n",
    mi.width,mi.height,mi.nrframes,mi.repeated_at_end,mi.uspfr);

  // If no filelist from old avi file: start over again, with new avi file.
  if (!mi.fl)
  {
    char *fn=NULL;
    sprintfd(&fn,"%s_prev",fn_movie);
    rename(fn_movie,fn);
    freed(&fn);
    prlog("avi: new\n");
    return new_jpeg2avi(fn_movie,fn_frame,fps);
  }

  repeat_lastframe=pauze_at_end*fps;               // calc. frames to repeat from 'FramesPerSecond'
  // Tag frames to remove
  rm_at_end=mi.repeated_at_end;                    //  # repeated frames to remove from old avi
  if (rm_at_end>=mi.nrframes) rm_at_end=0;         // don't remove if frames to remove >- # frames present.

  // Determine # frames to remove from start;
  if (max_frames)                                  // limit frames specified
  {
    rm_at_start=mi.nrframes-rm_at_end-max_frames;  // remove to get fixed movie length
    rm_at_start+=1;                                // one frame will be added so remove 1 more at start
  }
  else
  {
    rm_at_start=0;                                 // unlimitedt movie, don't remove frames 
  }
  tag_frames_to_remove(mi.fl,rm_at_start,rm_at_end);
  prlog("rm_at_start=%d  rm_at_end=%d pauze_at_end=%d  fps=%d\n",rm_at_start,rm_at_end,pauze_at_end,fps);
  for (i=0; i<repeat_lastframe+1; i++)
  {
    flnw=Create_Filelist(&mi.fl,fn_frame);         // Add new file to list, and repeat for pauze if requested
    flnw->new=TRUE;
    get_1file_sz(flnw);                            // Determine size of new file to add
  }
  mi.nrframes=adapt_movie(fn_movie,mi.fl,repeat_lastframe,mi.uspfr,mi.width,mi.height);
  Remove_Filelist(mi.fl);
  prlog("nrframes=%d  repeat_lastframe=%d  unique frames=%d\n",mi.nrframes,repeat_lastframe,mi.nrframes-repeat_lastframe);
  return mi.nrframes-repeat_lastframe;
}
// vb. MSG:
//   rm_at_start=2, rm_at_end=12, pauze=3, fps=4
//   mi.nrframes=110 repeat_lastframe=12 unieke frames=98
//   rm_at_start=1, rm_at_end=12, pauze=3, fps=4
//   mi.nrframes=110 repeat_lastframe=12 unieke frames=98

