/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * File generation functions
 ********************************************************************/
#include "xrit2pic.h"
#include "xrit2pic_defines.h"
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
extern XRIT_DBASE dbase;
extern GLOBINFO globinfo;
extern PREFER prefer;
extern GtkCTree *globtree;
extern GROUP *globgrp;

/***********************************************
 ***********************************************
 ** File generation functions
 ***********************************************
 ***********************************************/



// Run a script each time a pic is completed.
// fn=file generated
// command: see preferences, tab 'Generation', field 'Command'
// E.g.: cp %s abc/%s
static int run_script(char *fn,GtkWidget *widget)
{
  char *cmd;
  int err=0;
  int i,n=0,len;

  // Count # strings
  for (i=0; i<strlen(prefer.run_cmd); i++)
    if (!strncmp(prefer.run_cmd+i,"%s",2)) n++;

  len=strlen(prefer.run_cmd)+n*strlen(fn);
  if (!len) return 1;
  len+=10;
  cmd=malloc(len);
  if (!cmd) return 1;
  switch(n)
  {
    case 0:  strncpy(cmd,prefer.run_cmd,len);              break;
    case 1:  snprintf(cmd,len,prefer.run_cmd,fn);          break;
    case 2:  snprintf(cmd,len,prefer.run_cmd,fn,fn);       break;
    case 3:  snprintf(cmd,len,prefer.run_cmd,fn,fn,fn);    break;
    case 4:  snprintf(cmd,len,prefer.run_cmd,fn,fn,fn,fn); break;
    default: strncpy(cmd,prefer.run_cmd,len);              break;
  }
  #if __GTK_WIN32__ == 1
    err=execcmd(cmd,NULL,NULL,TRUE);
  #else
    err=system(cmd);
  #endif
  if (widget)
  {
    if (err) Set_Entry(widget,LAB_INFO,"Failed: %s.",cmd);
    else     Set_Entry(widget,LAB_INFO,"Cmd: %s.",cmd);
  }
  free(cmd);
  return err;
}

#ifdef NAAR_SPAWN
int rerun_script(GtkWidget *widget,int what)
{
#if __GTK_WIN32__ == 0
  static pid_t pid=0;
  if (!*globinfo.rerun_cmd) return -1;
  if ((what==0) || (what==2))
  {
    if (pid>0) stop_prog(pid);
    pid=-1;
  }
  if ((what==0) || (what==1))
  {
    pid=spawn_prog(globinfo.rerun_cmd); // gaat mis als globinfo.rerun_cmd niet start?
  }
  if (widget)
  {
    if (pid<=0) Set_Entry(widget,LAB_INFO,"Failed: %s.",globinfo.rerun_cmd);
    else        Set_Entry(widget,LAB_INFO,"Cmd: %s.",globinfo.rerun_cmd);
  }
  return pid;
#else
  return 0;
#endif
}
#endif

static void determine_oformat(GROUP *grp_sel,PIC_CHARS *pc,GENMODES *modes)
{
  switch(modes->cformat)
  {
    case PIC_RAH:
      modes->image_oformat=PIC_RAH;           /* generate rah (AVHRR) */
    break;
    case PIC_JPEG:
      modes->image_oformat=PIC_JPEG;          /* generate jpeg */
    break;
    case PIC_CJPEG:
      modes->image_oformat=PIC_CJPEG;         /* generate jpeg without decompressing */
    break;
    case PIC_PGM:
      modes->image_oformat=PIC_PGM;           /* generate pgm */
    break;
    case PIC_PGM8:
      modes->image_oformat=PIC_PGM8;          /* generate pgm  8 bpp*/
    break;
    default:
      switch (pc->image_iformat)
      {
        case PIC_JPEG:
          modes->image_oformat=PIC_JPEG;       /* generate jpeg without decompressing */
        break;
#ifdef XXX
        case 'Z':
          modes->image_oformat=PIC_PGM;       /* generate ppm */
        break;
#endif
        case 'z': case 'Z': case 'y':     // 'z'=NOAA HRPT, 'y'=METOP HRPT, 'Z'=EPS METOP
          if ((grp_sel->gen_rah) && (!grp_sel->compose))
            modes->image_oformat=PIC_RAH;       /* generate rah */
          else
            modes->image_oformat=PIC_PGM;       /* generate ppm */
        break;
        case 't':
          modes->image_oformat=PIC_TIFF;       /* tiff */
        break;
        default:
          modes->image_oformat=PIC_PGM;         /* generate pgm */
        break;
      }
    break;
  }
  if ((pc->is_color) || (globinfo.dim3))
  {
    if (modes->image_oformat==PIC_CJPEG)
      modes->image_oformat=PIC_JPEG;
  }
}


/***********************************************
 * Generate a file in any format (JPEG, PGM, PPM)
 * from picture in any format (JPEG, Wavelet)
 * filename is generated, and available in *tfno
 ***********************************************/
int gen_picfile(GtkWidget *window,
                GROUP *grp_sel,          /* contains 1 picture to generate */
                PREFER *prefer,
                char **tfno,              /* output generated file, incl. path */
                gboolean keep_chnk,      /* keep chunk in mem? */
                gboolean show_progress,
                GENMODES *modes)
{
  int err=0;
  FILE *fpi;
  char *fn;
  char *fn1;
  PIC_CHARS pc1;
  PIC_CHARS *pc;
  DATATYPE dtype;
  CHANNEL *chan=NULL;
  if (!grp_sel) return -11;
  if (!grp_sel->chan) return -12;
  chan=grp_sel->chan;
  if (!chan->segm) chan=chan->next;
  if ((!chan) || (!chan->segm)) return -13;

  pc1=grp_sel->pc;
  pc=&pc1;

  dtype=chan->segm->xh.xrit_frmt;
  fn=chan->segm->pfn;
/*
    if (pc->pixel_shape)
    {
      if (pc->pixel_shape>1.) pc->o_height*=pc->pixel_shape;
      if (pc->pixel_shape<1.) pc->o_width/=pc->pixel_shape;
    }
*/
// adapt_lum betekent ook dat zoom-factor wordt meegenomen!
  if (modes->adapt_lum) zoom_size(modes,pc);
  if (modes->gen_film) adapt_filmsize(modes,pc);
  if (fn)
  {
    if (!(fpi=open_xritimage(fn,NULL,dtype,chan))) return Open_Rd;
    fclose(fpi);
  }
  // next needed to change pic-type (from menu) to vector type (if set as such)
  globinfo.pm.ol.overlaytype=change_ovltype(prefer,globinfo.pm.ol.overlaytype);

/* Determine output format */
  determine_oformat(grp_sel,pc,modes);


/* For avi film, files will be generated in temp dir. */
  if (modes->gen_film)
  {
    *tfno=prepare_filmgen(globinfo.dest_tmpdir,globinfo.dest_tmpsavdir,grp_sel,pc,modes);
  }
  else
  {
    *tfno=make_filename(globinfo.dest_dir,pc,NULL,modes,prefer->split_destdir);    /* generate in dest dir */
    if (modes->image_oformat=='t') strcatd(tfno,chan->segm->fn);
  }

/* strip path from filename; only for messages */
  fn1=strrchr(*tfno,DIR_SEPARATOR);
  if (fn1) fn1++; else fn1=*tfno;

  if (modes->test)
  {
    err=(pc->incomplete_pro? Mis_pro: pc->incomplete? Mis_segm: 0);
    return err;
  }

  // Generate pic
  switch(modes->image_oformat)
  {
    case PIC_RAH:
      if (overwrite(*tfno,modes->overwrite_mode))
      {
        if (raw16torah(grp_sel,*tfno,show_progress))
        {
          err=Aborted;
          if (window) Set_Entry(window,LAB_INFO,"Exist: %s.",fn1);
        }
      }
      else
      {
        err=Exist_fn;
        if (window) Set_Entry(window,LAB_INFO,"Exist: %s.",fn1);
      }

    break;
    case PIC_PGM: case PIC_PGM8: case PIC_JPEG:  /* jpeg, pgm8/16, for pic and movie */
    {
      gboolean do_generate=FALSE;
      if (modes->gen_film)
      {
        OVERWRMODE ovrwrmode=OVRWR_SKIP;
        switch(modes->filmframe_ovrwrmode)
        {
          case TMP_REPLACE: ovrwrmode=OVRWR_REPLACE; break;
          case TMP_REUSE:   ovrwrmode=OVRWR_SKIP;    break;
          case TMP_REMOVE:  ovrwrmode=OVRWR_REPLACE; break;
        }        
        if (overwrite(*tfno,ovrwrmode)) do_generate=TRUE;
      }
      else
      {
        if (overwrite_chngd(tfno,400,modes->overwrite_mode)) do_generate=TRUE;
      }
      if (do_generate)
      { /* film -> don't check if file exist */
        err=gen_a_picfile(grp_sel,pc,*tfno,keep_chnk,show_progress,modes);
        grp_sel->done=TRUE;
        if (window)
        {
          switch (err)
          {
            case Open_Wr : Set_Entry(window,LAB_INFO,"Can't open %s for write.",fn1); break;
            case Aborted : Set_Entry(window,LAB_INFO,"Aborted: %s.",fn1); break;
            case Mis_segm: Set_Entry(window,LAB_INFO,"Segments in %s missing.",fn1); break;
            case Mis_pro : Set_Entry(window,LAB_INFO,"Prologue for %s missing.",fn1); break;
            case 0:        Set_Entry(window,LAB_INFO,"Created: %s.",fn1); break;
            default:       Set_Entry(window,LAB_INFO,"?? Errmessage %d in %s.",err,fn1); break;
          }
        }
      }
      else if ((modes->gen_film) && (modes->filmframe_ovrwrmode==TMP_REUSE)) // 2=SKIP (reuse)
      {
        err=Exist_fn;
        grp_sel->done=TRUE;
      }
      else
      {
        err=Exist_fn;
        if (window) Set_Entry(window,LAB_INFO,"Exist: %s.",fn1);
      }
    }
    break;
    case PIC_CJPEG:                       /* jpeg, without decompressing */
      if (pc->is_color)
      {
        return JPG_Conc;
      }
   
      if (overwrite(*tfno,globinfo.overwrite_mode))
      {
        err=gen_jpgfile(grp_sel,pc,*tfno,show_progress);
        if (window) Set_Entry(window,LAB_INFO,"Created: %s.",fn1);
      }
      else
      {
        err=Exist_fn;
        if (window) Set_Entry(window,LAB_INFO,"Exist: %s.",fn1);
      }
    break;
    case PIC_TIFF:
      if (copy_file(chan->segm->pfn,*tfno))
      {
        if (window)
          Set_Entry(window,LAB_INFO,"Failed Copy of %s.",chan->segm->fn);
      }
      else
      {
        if (window)
          Set_Entry(window,LAB_INFO,"Copied: %s.",chan->segm->fn);
      }
    break;
  }
  if ((pc->incomplete) && (err==0)) err=Mis_segm;
  if ((pc->incomplete_pro) && (err==0)) err=Mis_pro;
  return err;
}

void handle_spaces(char *s,char dirsep) /* s='aap noot/wsat -i' */
{
  char *p;
  p=strrchr(s,dirsep);                   /* p='/wsat -i' */
  if (!p) return;
  p=strchr(p,' ');                       /* p=' -i' */
  if (p) *p=0;                           /* p='. -i" ; s="aap noot/wsat' */
  if ((strchr(s,' ')) && (*s!='"'))
  {
    if (p) memmove(p+2,p,strlen(p+1)+2); /* p='...-i' */
    memmove(s+1,s,strlen(s)+1);          /* s=' aap noot/wsat' ; p='t..-i' */
    *s='"';                              /* s='"aap noot/wsat' */
    if (p)
    {
      *(s+strlen(s))='"';                /* s='"aap noot/wsat"' ; p='t".-i' */
      p+=2; *p=' ';                      /* s='"aap noot/wsat" -i" */
    }
    else
    {
      strcat(s,"\"");
    }
  }
  else if (p) *p=' ';
}

#include <stdlib.h>
char *launch_viewer(PREFER *prefer,PIC_CHARS *pc,char *tfno,GENMODES *gmode)
{
#if __GTK_WIN32__ == 1
  const char nwproc=0;
#else
  const char nwproc='&';
#endif
  char *cmd=NULL;


  handle_spaces(prefer->prog_jpg,DIR_SEPARATOR);     /* determine launcher JPEG */
  handle_spaces(prefer->prog_pgm,DIR_SEPARATOR);     /* determine launcher PRM */

  switch(gmode->image_oformat)
  {
    case 't':                                        /* launch TIFF viewer */
      sprintfd(&cmd,"%s %s %s %c",
                  prefer->prog_jpg,tfno,(pc->scan_dir=='s'? prefer->ud_jpg : ""),nwproc);
    case 'J':                                        /* launch JPEG viewer */
      sprintfd(&cmd,"%s %s %s %c",
                  prefer->prog_jpg,tfno,(pc->scan_dir=='s'? prefer->ud_jpg : ""),nwproc);
    break;
    case 'j':                                        /* launch JPEG viewer */
      sprintfd(&cmd,"%s %s %s %c",
                  prefer->prog_jpg,tfno,(pc->scan_dir=='s' && !prefer->scandir_correct? prefer->ud_jpg : ""),nwproc);

    break;
    case 'p': case 'P':                              /* launch PGM viewer */
      sprintfd(&cmd,"%s %s %s %c",
                prefer->prog_pgm,tfno,
                 (pc->scan_dir=='s' && !prefer->scandir_correct? prefer->ud_pgm : ""),nwproc);

    break;
    default:
    break;
  }
  if (!cmd) return NULL;
  if (system(cmd)) return cmd;                                      /* Launch viewer */
  freed(&cmd);
  return NULL; 
}


static int gen_avi_extern(char *mfno,PREFER *prefer)
{
  int res=0;
  char *cmd=NULL;
  sprintfd(&cmd,prefer->prog_genmovie,globinfo.dest_tmpdir,"*.jpg","",mfno);
//  strcat(cmd," ");
//  strcat(cmd,prefer->opt_speed);
  if (system(cmd)) res=-1;
  freed(&cmd);
  return res;
}

int gen_avi(char *mfno,PREFER *prefer,FILELIST *fl)
{
  int res=0;
  if (prefer->extern_moviegen)
  {
    res=gen_avi_extern(mfno,prefer);
  }
  else
  {
    res=jpeg2avi(fl,mfno,prefer->speed,0);
  }
  return res;
}

int gen_cpgm(char *mfno,PREFER *prefer)
{
  DIR *directory;
  struct dirent *dirEntry;
  FILE *fpi,*fpo;
  int c;
  if (!(fpo=fopen(mfno,"wb"))) return -2;

/* opendir(): For Windows path MUST end with '.' */
  {
    char *dir=NULL;
    sprintfd(&dir,"%s.",globinfo.dest_tmpdir);
    if (!(directory = opendir(dir)))
    {
      fclose(fpo);
      return -2;
    }
    freed(&dir);
  }

  if (directory) while ((dirEntry = readdir(directory)))
  {
    char *fn=NULL;
    sprintfd(&fn,"%s%s",globinfo.dest_tmpdir,dirEntry->d_name);
    if (*dirEntry->d_name != '.')
    {
      if (!(fpi=fopen(fn,"rb"))) continue;
      while ((c=fgetc(fpi)) != EOF) fputc(c,fpo);
      fclose(fpi);
    }
    freed(&fn);      
  }
  closedir(directory);
  fclose(fpo);

  return 0;
}

static void Delete_Use_List(GROUP *igrp,GtkCTree *ctree,GtkWidget *widget)
{
  GROUP *grp,*grpnext;
  CHANNEL *chan,*channext;
  for (grp=igrp; grp; grp=grpnext)
  {
    char sat[10];
    grpnext=grp->next;

/* Distinquish between MSG1 and 2,3,4. 
   If MSG-2 operational: Don't dist. between 1 and 2!
*/
    if (!strncmp(grp->sat_src,"MSG1",4))
      sprintf(sat,"%c%s",grp->h_or_l,"MSG1");
    else if (!strncmp(grp->sat_src,"MSG2",4))
      sprintf(sat,"%c%s",grp->h_or_l,"MSG2");
    else if (!strncmp(grp->sat_src,"MSG3",4))
      sprintf(sat,"%c%s",grp->h_or_l,"MSG3");
    else if (!strncmp(grp->sat_src,"MSG4",4))
      sprintf(sat,"%c%s",grp->h_or_l,"MSG4");
    else
      strncpy(sat,grp->sat_src,10);
    
    for (chan=grp->chan; chan; chan=channext)
    {
      channext=chan->next;
      if (globinfo.satlist->enable_remove)
      {
        if (sat_to_remove(sat,chan->chan_name,0))
        {
          Set_Entry(widget,LAB_INFO,"Removed after: %s",chan->chan_name);
          Delete_Chan(chan,ctree);
        }
      }
    }
    if (!grp->chan)
    {
      if (globgrp==grp)
        globgrp=grp->next;
      Delete_Grp(grp,ctree);
    }
  }
}

// Determine last group in grp-list with satellite to process/
// Earlier groups in this list, not processed yet, will be skipped.
// This is to prevent pile up of items to process, because of data coming in too fast / PC too slow.
// This will also cause skipping of frames in a created movie.
// Return: nr. of skipped items.
static int get_last_selected_new(GROUP *igrp,GENMODES *gmode)
{
  GROUP *grp,*g_last=NULL,*g;
  int n=0;
  for (grp=igrp; grp ; grp=grp->next)
  {
    char *satname;
    if (!grp->new) continue;    // already processed
    satname=construct_satname(grp);
    if (!strcmpwild(satname,globinfo.pr.lv_sat))
    {                                           // This sat equals selected sat to process
      g_last=grp;                               // g_last is the last group in the grp-list containing selected sat
    }
  }

  for (g=igrp; g ; g=g->next)
  {
    if (g!=g_last)                              // Not the last group of selected sat
    {
      if (g->new)                               // not processed yet!
      {
        prlog("Skip [%02d:%02d]\n",g->grp_tm.tm_hour,g->grp_tm.tm_min);
        n++;
        g->new=FALSE;                           // Set to false, will not be processed...
      }
    }
  }
//  prlog("Skipped: %d\n",n);

  return n;
}

static char *get_grpname_selected()
{
  SATLIST *sl;
  sl=Get_Satlist_Item(globinfo.satlist,SATLIST_NAME,globinfo.pr.lv_sat);
  if (!sl) return NULL;
  return sl->grp_name;
}

char *create_aviname(int area_nr)
{
  char *fn=NULL;
  char *aviname=get_grpname_selected();
  if (!aviname) aviname="geo";
  if (area_nr)
    sprintfd(&fn,"%s_z%d.avi",aviname,area_nr);
  else
    sprintfd(&fn,"%s.avi",aviname);
  return fn;
}

#define STD_FILMNAME "geo.avi"
/* Auto translate pic which is also selected as 'live' */
void auto_translate(GENMODES *gmode,GROUP *grp,PREFER *pref,GtkWidget *widget)
{
  int n;
// Volgende regel: waarom? Geeft problemen als data te vlug binnen komt.
  if ((n=get_last_selected_new(grp,gmode))) // reset all new flags except last selected
  {
    Set_Entry(widget,LAB_RECINFO,"Skipped %d items, processing too slow...",n);
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }
  for (; grp; grp=grp->next)    // go through all groups
  {
    CHANNEL *chan;
//    prlog("  group: new=%d [%02d:%02d]\n",grp->new,grp->grp_tm.tm_hour,grp->grp_tm.tm_min);
    if (!grp->new) continue;    // already processed
//    prlog("  group: [%02d:%02d]\n",grp->grp_tm.tm_hour,grp->grp_tm.tm_min);

    // set flag to item to generate
    if (gmode->spm.compose)          
    {
      grp->selected=TRUE;      // select whole group
      chan=grp->chan;          // take first chan, maybe this isn't used!
    }
    else   // one channel used
    {
      for (chan=grp->chan; chan; chan=chan->next)
        chan->r=chan->g=chan->b=0;      // 'reset' all chans

      for (chan=grp->chan; chan; chan=chan->next)
      {
        if (contributes_to_pic(chan,globinfo.pr.lv_sat,globinfo.pr.lv_chan,gmode->essential_only))
        {
          chan->selected=TRUE;        // this single channel selected
          chan->r=chan->g=chan->b=1;
//          prlog("  Chan %s to use, [%02d:%02d]\n",chan->chan_name,grp->grp_tm.tm_hour,grp->grp_tm.tm_min);
          break;
        }
      }
    }

// If either chan=selected or composed then 'contributes_to_pic'.
// No wild-char!
    if (contributes_to_pic(chan,globinfo.pr.lv_sat,globinfo.pr.lv_chan,gmode->essential_only))
    {
      if (picture_complete(chan,NULL,gmode))
      {                 // all segments and channels available to generate the pic
        int err=0;
        int nrfr=0;
        char *tfno=NULL;
        char *p;
        prlog("    Complete! [%02d:%02d]\n",grp->grp_tm.tm_hour,grp->grp_tm.tm_min);
        if (widget)
        {
          Set_Entry(widget,LAB_INFO,"Translating...");
          Set_Entry(widget,LAB_RECINFO,"Translating...");
        }
        grp->compose=gmode->spm.compose;
        gmode->for_live=TRUE;
        pic_info(grp,widget,gmode);
        gmode->adapt_lum=FALSE;
        gmode->spm.invert=globinfo.pr.lv_clr.invert;
        gmode->spm.inv_ir=globinfo.invert_ir;
        gmode->spm.ol=globinfo.pr.lv_ol;
        gmode->spm.ol.ol_lum=0xff;
        if (gmode->gen_film)
          gmode->jpeg_quality=globinfo.movie_quality;
        else
          gmode->jpeg_quality=globinfo.jpeg_quality;
        gmode->gamma=globinfo.pr.lvg_gamma;
        gmode->adapt_lum=TRUE;
        geoarea2xy(grp,gmode);

        if (!(err=gen_picfile(NULL,grp,pref,&tfno,FALSE,FALSE,gmode)))
        {                           // pic-file generated
          SATLIST *sl=Get_Satlist_Item(globinfo.satlist,SATLIST_NAME,globinfo.pr.lv_sat);
          int movie_max_nrframes=0;
          char *mfno=NULL;
          prlog("      gen_picfile OK!\n");
          if (gmode->gen_film)
          {                         // if film: add new pic to avi
            char *mpfno=NULL;
            mfno=create_aviname(globinfo.area_nr);
            cat_pathfiled(&mpfno,prefer.dest_dir,mfno);

            if (pref->extern_moviegen)
            {
              gen_avi_extern(mpfno,pref);
            }
            else
            {
              int fps=speed_minpsec2fps(sl,globinfo.pr.lvg_movie_speed);
              if ((sl) && (sl->min_per_pic))
                movie_max_nrframes=globinfo.pr.lvg_movie_size_max_hr*60/sl->min_per_pic;
              else
                movie_max_nrframes=globinfo.pr.lvg_movie_size_max_hr;

              if (globinfo.pr.lvg_movie_start_utc_hr>=0)
                if ((grp->grp_tm.tm_hour==globinfo.pr.lvg_movie_start_utc_hr) && (grp->grp_tm.tm_min==0))
                  movie_max_nrframes=1;
              nrfr=add_frame(mpfno,tfno,movie_max_nrframes,fps,globinfo.pr.lvg_movie_rpt_lastfrm);
            } 
            freed(&mpfno);
          }

          // rest is for messages only
          if (widget)
          {
            if (gmode->gen_film)
            {
              char str[100];
              char *id="";
               if (strlen(grp->grp_time)-1-4>0)
                 id=grp->grp_time+strlen(grp->grp_time)-4; // set to last part = time
              if (movie_max_nrframes)
                snprintf(str,100,"Added frame '%s' to: %s (now %d frames, limit: %d)",
                 id,mfno,nrfr,movie_max_nrframes);
              else
                snprintf(str,100,"Added frame '%s' to: %s (now %d frames, no limit)",
                 id,mfno,nrfr);

              Set_Entry(widget,LAB_INFO,"%s",str);
              Set_Entry(widget,LAB_RECINFO,"%s",str);
            }
            else
            {
              if (tfno)
              {
                if (!(p=strrchr(tfno,DIR_SEPARATOR))) p=tfno; else p++;
              }
              else
              {
                p="????";
              }

              Set_Entry(widget,LAB_INFO,"Created: %s.",p);
              Set_Entry(widget,LAB_RECINFO,"Created %s.",p);
            }
          }
          freed(&mfno);

          if (gmode->run_script)
          {                      // run user-script
            run_script(tfno,widget);
          }
          #ifndef __NOGUI__
          if (gmode->rerun_script)
          {                      // run user-script
            grerun_script(widget,0);
          }
          #endif
        }
        else               // problem with generating this pic-file: skip
        {
          prlog("      Err genpicfile: %s\n",report_genpicerr_1line(err));
          if (tfno)
          {
            if (!(p=strrchr(tfno,DIR_SEPARATOR))) p=tfno; else p++;
          }
          else
          {
            p="???";
          }
          Set_Entry(widget,LAB_INFO,"(Skipped: %s; %s (%d)",p,report_genpicerr_1line(err),err);
          Set_Entry(widget,LAB_RECINFO,"(Skipped: %s; %s (%d)",p,report_genpicerr_1line(err),err);
        }

        free_chnks(grp);
        prlog("Ready with this pic: [%02d:%02d]\n",grp->grp_tm.tm_hour,grp->grp_tm.tm_min);
        grp->new=FALSE;
        grp->done=TRUE;  // ook al wat gedaan in gen_picfile
       
        if (globinfo.rm_raw)
        {                        // remove raw file immediately after use
          if (dbase.grp_last)
          {
            Delete_Use_List(dbase.grp_last->next,dbase.main_tree,widget);
          }
          else
          {
            Delete_Use_List(globgrp,dbase.main_tree,widget);
          }
//          Set_Entry(widget,LAB_RECINFO,"Deleted raw data: %d.",n);

          break;      // (from for (; grp...) Done; grp is destroyed, so MUST stop!!!
        }

        if ((gmode->gen_film) && (globinfo.pr.lvg_movie_rmjpeg))
        {
          remove(tfno);
        }
        freed(&tfno);

        break;        // complete pic found, so stop.
      }
    }
  }
}

