#include "xrit2pic.h"
#include "xrit2pic_defines.h"
extern GLOBINFO globinfo;
extern PREFER prefer;
static void SWAP(int *a,int *b)
{
  int t;
  t=*a; *a=*b; *b=t;
}

/*******************************************************************
 * Convert 'Europe' boundaries for geo-mappings.
 * fix_... = size-spec "Europe"
 * pp: based on x/y
 ********************************************************************/
static void conv_bnd(struct picpart *pp,PIC_CHARS *pc)
{
  int x1,x2;
  int y1,y2;
  y1=pp->fix_hoffset;      //deze y komt overeen met start-lat.
  y2=pp->fix_hoffset+pp->fix_height;
  x1=pp->fix_woffset;
  x2=pp->fix_woffset+pp->fix_width;
  fxy2dxy(x2,y2,&x2,&y2,pc);
  fxy2dxy(x1,y1,&x1,&y1,pc);
  pp->fix_height=y2-y1;
  pp->fix_hoffset=y1;     // dit wordt start-y die via dxy2fxy start-y in file geeft.
  pp->fix_width=x2-x1;
  pp->fix_woffset=x1;
  return;
}

/* Determine size pic in file 
 * Problem with HRV and movies: pc->width changes between frames!
 *
 * rgbi->width/height    = size drawable
 * rgbpi->pwidth/pheight = size array filled with pic (normally size of pic itself)
 * pc->o_width/o_height  = size generated pic.
 *   Differs from org. pic if:
 *     avhrr: doubled width to do linearizing
 *     HRV: width depends on shift north/south
 *     correction is done if pixel isn't square
 * HRV only:
 *   pc->org_width         = pc->width or pc->height (HRV full width)
*/
void size_pic(GROUP *grp_sel,GENMODES *gmode)
{
  PIC_CHARS *pc=&grp_sel->pc;
  CHANNEL *hrvchan=NULL;
  int width;
  struct picpart pp_norm;
  struct picpart pp_hrv;
  struct point delta;

  pp_norm=gmode->geoarea.pp_norm;
  pp_hrv=gmode->geoarea.pp_hrv;
  delta=gmode->geoarea.delta;
  width=pc->width;
  if (Chan_Contributes(grp_sel,"HRV"))  // contains HRV
  {
    pc->org_width=pc->height;                  // originally square...
    hrvchan=Get_Chan(grp_sel->chan,"HRV");
  }
  else
  {
    pc->org_width=pc->width;
  }
/* Actual boundaries globe */
  if (pc->scan_dir=='s')
  {
    pc->x_w=pc->width-1;
    pc->y_n=pc->height-1;
    pc->x_e=0;
    pc->y_s=0;
  }
  else
  {
    pc->x_w=0;
    pc->y_n=0;
    pc->x_e=pc->width-1;
    pc->y_s=pc->height-1;
  }
  if (pc->y_s > pc->x_e) pc->y_s=pc->x_e;  // correction e.g. GOES
  
  if (gmode->gen_film)
  {
    if (gmode->frame_minwidth) width=gmode->frame_minwidth;
  }

  if (!pp_norm.fix_width)
  {
    if (pp_norm.pct_width)
    {
      pp_norm.fix_width=width*pp_norm.pct_width/100;
      pp_norm.fix_woffset=width*pp_norm.pct_woffset/100;
    }
    else
    {
      pp_norm.fix_width=width*22/100;
      pp_norm.fix_woffset=width*36/100;
    }
  }

  if (!pp_norm.fix_height)
  {
    if (pp_norm.pct_height)
    {
      pp_norm.fix_height=pc->height*pp_norm.pct_height/100;
      pp_norm.fix_hoffset=pc->height*pp_norm.pct_hoffset/100;
    }
    else
    {
      pp_norm.fix_height=pc->height*17/100;
      pp_norm.fix_hoffset=pc->height*82/100;
    }
  }
  if ((pc->scan_dir=='n') && ((!delta.lon) || (!delta.lat)))
  {
    pp_norm.fix_hoffset=pc->height-pp_norm.fix_hoffset-pp_norm.fix_height;
    pp_norm.fix_woffset=pc->width-pp_norm.fix_woffset-pp_norm.fix_width;
  }
  conv_bnd(&pp_norm,pc);

  if (!pp_hrv.fix_width)
  {
    int hrvwidth=7600; // width;   // only O if 'europe' side is upper part
    if (pp_hrv.pct_width)
    {
      pp_hrv.fix_width=hrvwidth*pp_hrv.pct_width/100;
      pp_hrv.fix_woffset=hrvwidth*pp_hrv.pct_woffset/100;
    }
    else
    {
      pp_hrv.fix_width=hrvwidth*33/100;
      pp_hrv.fix_woffset=hrvwidth*51/100; // /2+100;
    }
  }
  if (!pp_hrv.fix_height)
  {
    if (pp_hrv.pct_height)
    {
      pp_hrv.fix_height=pc->height*pp_hrv.pct_height/100;
      pp_hrv.fix_hoffset=pc->height*pp_hrv.pct_hoffset/100;
    }
    else
    {
      pp_hrv.fix_height=pc->height*17/100;
      pp_hrv.fix_hoffset=pc->height*82/100;
    }
  }
  if ((pc->scan_dir=='n') && ((!delta.lon) || (!delta.lat)))
  {
    pp_hrv.fix_hoffset=pc->height-pp_hrv.fix_hoffset-pp_hrv.fix_height;
    pp_hrv.fix_woffset=pc->width-pp_hrv.fix_woffset-pp_hrv.fix_width;
  }
  conv_bnd(&pp_hrv,pc);

  if (IS_GEO(grp_sel))
  {
    if (hrvchan)                           // contains HRV
    {
      pc->gchar.shift_ypos   =hrvchan->shift_ypos;
      pc->gchar.upper_x_shift=hrvchan->upper_x_shift;
      pc->gchar.lower_x_shift=hrvchan->lower_x_shift;
    }
    else
    {
      if (grp_sel->chan)                   // expected always !=0
      {
        pc->gchar.shift_ypos   =grp_sel->chan->shift_ypos;
        pc->gchar.upper_x_shift=grp_sel->chan->upper_x_shift;
        pc->gchar.lower_x_shift=grp_sel->chan->lower_x_shift;
      }
    }
  }

  if (grp_sel->orbittype==GEO)
  {
    if (gmode->area_nr)
    {
      if (hrvchan)                             // contains HRV
      {
        pc->o_width=pp_hrv.fix_width;
        pc->o_xoffset=pp_hrv.fix_woffset;      // Offset from first pixel of line = east
        pc->o_height=pp_hrv.fix_height;
        pc->o_yoffset=pp_hrv.fix_hoffset;      // Offset from first line = south
      }
      else
      {
        pc->o_width=pp_norm.fix_width;
        pc->o_xoffset=pp_norm.fix_woffset;
        pc->o_height=pp_norm.fix_height;
        pc->o_yoffset=pp_norm.fix_hoffset;     // Offset from first line = south
      }
      pc->gmap=gmode->gmap;
    }
    else // no zoom
    {
      pc->o_width=width;
      pc->o_xoffset=0;
      pc->o_height=pc->height;
      pc->o_yoffset=0;
      pc->gmap=gmode->gmap;
    }
  }

  else if (grp_sel->orbittype==POLAR)
  {
    pc->o_width=width;
    pc->o_xoffset=0;
    pc->o_height=pc->height;
    pc->o_yoffset=0;
    pc->avhrr_lin=gmode->avhrr_lin;
    pc->avhrr_lin_dblx=gmode->avhrr_lin_dblx;
    if ((pc->avhrr_lin) && (pc->avhrr_lin_dblx)) pc->o_width*=2;
  }
  else
  {
    pc->o_width=width;
    pc->o_xoffset=0;
    pc->o_height=pc->height;
    pc->o_yoffset=0;
    pc->gmap=gmode->gmap;
  }
}


// convert lon,lat/dlon,dlat to zoom-info in pc (o_width,o_height/o_xoffset,o_yoffset)
void geoarea2xy(GROUP *grp,GENMODES *modes)
{
  int x1,x2,y1,y2,x,y;
  float lon1,lon2,lat1,lat2;
  modes->geoarea=prefer.geoarea[globinfo.area_nr-1];  // Boundaries channels
  PIC_CHARS *pc=&grp->pc;
  GEOCHAR gchar=pc->gchar;
  
  lon1=modes->geoarea.center.lon-modes->geoarea.delta.lon; // west lon
  lat1=modes->geoarea.center.lat+modes->geoarea.delta.lat; // north lat
  lon2=modes->geoarea.center.lon+modes->geoarea.delta.lon; // east lon
  lat2=modes->geoarea.center.lat-modes->geoarea.delta.lat; // south lat

  // *** determine x-west at area-center lat: x1
  if (lon1 <  pc->gchar.subsat_lon-85.)
  {
    /* Stretch to edge (glonlat2xy doesn't work near +/- 90 degrees) */
    if (pc->gchar.cfac < 0) x1=pc->width-1; else x1=0;
  }
  else
  {
    if (pc->gmap==normal)
    { // NIET glonlat2xy_xcorr!
      glonlat2xy(D2R(lon1),D2R(modes->geoarea.center.lat),&x1,&y,&gchar);  // x1=fpos gewenste lon1 bij center-lat
    }
    else
      glonlat2xy(D2R(lon1),D2R(lat1),&x1,&y1,&gchar);  // (x1,y1) bij (lon1,lat1)
  }
  // Now x1=offset ar center-lat (y not used)

  // *** determine x-east at area-center lat: x2
  if (lon2 > pc->gchar.subsat_lon+85.)
  {
    if (pc->gchar.cfac < 0)  x2=0; else x2=pc->width-1;
  }
  else
  {
    if (pc->gmap==normal)
      glonlat2xy(D2R(lon2),D2R(modes->geoarea.center.lat),&x2,&y,&gchar);  // x2=fpos gewenste lon2 bij center-lat
    else
      glonlat2xy(D2R(lon2),D2R(lat2),&x2,&y2,&gchar);  // (x2,y2) bij (lon2,lat2)
  }
  // Now x2=east position of center-lat (y not used)

  // *** determine y-north at area-center lon: y1
  if (pc->gmap==normal)
  {
    lat1=modes->geoarea.center.lat+modes->geoarea.delta.lat;
    if (lat1 > 85.)
    {
      if (pc->gchar.lfac < 0)   y1=pc->height-1; else y1=0;
    }
    else
    {
      glonlat2xy(D2R(modes->geoarea.center.lon),D2R(lat1),&x,&y1,&pc->gchar);  // y1=fpos gewenste lat1 bij center-lon
    }
    // Now y1=north position of center-lon (x not used)

    // *** determine y-south at area-center lon: y2
    lat2=modes->geoarea.center.lat-modes->geoarea.delta.lat;
    if (lat2 < -85.)
    {
    /* Stretch to edge (glonlat2xy doesn't work near +/- 90 degrees) */
      if (pc->gchar.lfac < 0) y2=0; else y2=pc->height-1; 
    }
    else
    {
      glonlat2xy(D2R(modes->geoarea.center.lon),D2R(lat2),&x,&y2,&pc->gchar);  // y2=fpos gewenste lat2 bij center-lon
    }
  }
  // Now y2=south position of center-lon (x not used)
  if (x1<x2) SWAP(&x1,&x2);
  if (y1<y2) SWAP(&y1,&y2);
  // (x1,y1) en (x2,y2) zijn nu de coord. in file die begin- en eindpunt af te beelden deel geven.
  // Zonder speciale mapping kunnen daaruit direct offset en b/h worden bepaald; wijst later via wy2py/wx2px direct naar filepos.
  // Met speciale mapping: dxy2fxy ertussen, dus x,y zodanig dat na dxy2fxy offset en w/b zdd dat naar juiste deel file.
  // -> x,y -> fxy2dxy
  modes->geoarea.pp_norm.fix_width=x1-x2;
  modes->geoarea.pp_norm.fix_woffset=x2;
  modes->geoarea.pp_norm.fix_height=y1-y2;
  modes->geoarea.pp_norm.fix_hoffset=y2;
  modes->geoarea.pp_hrv.fix_width=x1-x2;
  modes->geoarea.pp_hrv.fix_woffset=x2;
  modes->geoarea.pp_hrv.fix_height=y1-y2;
  modes->geoarea.pp_hrv.fix_hoffset=y2;
  size_pic(grp,modes);
}
//            wid  hei  xoff yoff
//norm: full: 3712,1392,   0,   0
//      euro: 1019, 630,1437, 706
//auto: full: 1856, 696,   0,   0
//      euro: 1520, 944,1610,2118

// new size: o_xoffset ... o_xoffset+o_width
// convert zx,zy/ox,oy to zoom-info in pc (o_width,o_height,o_xoffset,o_yoffset)
void zoom_size(GENMODES *gmode,PIC_CHARS *pc)
{
  if (!gmode->zx) return;
  if (!gmode->zy) return;
  if (pc->scan_dir=='n')
  {
    pc->o_xoffset+=gmode->ox;
    pc->o_yoffset+=gmode->oy;
  }
  else
  {
    pc->o_xoffset+=(pc->o_width*(1-1/gmode->zx)-gmode->ox);
    pc->o_yoffset+=(pc->o_height*(1-1/gmode->zy)-gmode->oy);
  }
  pc->o_width=pc->o_width/gmode->zx;     // o_width: already taken into account frx!
  pc->o_height=pc->o_height/gmode->zy;
}

void adapt_filmsize(GENMODES *gmode,PIC_CHARS *pc)
{
  gmode->dx=1;
  gmode->dy=1;
  /* Make sure size doesn't exceed predefined value */
  gmode->dx=pc->o_width/prefer.movie_resol + 1;   // increment x
  gmode->dy=pc->o_height/prefer.movie_resol + 1;  // increment y
  gmode->dx=MAX(gmode->dx,gmode->dy);           // dx>=dy
  gmode->dy=MAX(gmode->dx,gmode->dy);           // dy>=dx ==> dx=dy=max(org_dx,org_dy)
  pc->o_width/=gmode->dx;                       // new width output
  pc->o_height/=gmode->dy;                      // new height output
  pc->o_width=(pc->o_width/16)*16;              // make width multiple of 16 (needed for jpeg/avi?)

// niet doen! 
//  for (i=0; i<=NRANNO; i++)
//  {
//    gmode->spm.ol.annotxt[i].size=0; // 4/gmode->dx;
//  }  
}
