/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************
 *
 * GROUP *Load_List(GROUP *,GtkCTree *,GtkWidget *,char *path,gboolean clear,
 *                  LOCATION_SEGM,gboolean cont_update,GENMODES *)
 *
 * GROUP *Load_List_all(GROUP *,GtkCTree *,GtkWidget *,GENMODES *)
 ********************************************************************/

#include <stdlib.h>
#include <string.h>
#include "xrit2pic.h"
#include "xrit2pic_defines.h"
extern GLOBINFO globinfo;
extern XRIT_DBASE dbase;

static void msgnotify(GROUP *grp,int err)
{
#if __GTK_WIN32__ == 1
  const char nwproc=0;
#else
  const char nwproc='&';
#endif
  char cmd[100],tstr[20];
  strftime(tstr,20,"%Y %m %d %H %M",&grp->grp_tm);
  sprintf(cmd,"%s %s %s %c","msgnotify",(err? "MSGDATALOSS" : "MSGSCANEND"),tstr,nwproc);
  system(cmd);
}

static int check_msg(GROUP *grp)
{
  CHANNEL *c;
  int err=0;
  if (!grp) return -1;
  for (c=grp->chan; c; c=c->next)
    if (channel_complete(c)) err++;
  return err;
}


static LIST_FILES *remove_this(LIST_FILES *list_files,LIST_FILES *lf)
{
  if (!(remove(lf->pfn)))
  {
    if (globinfo.log) print_log(NULL,"Removed unknown: %s\n",(lf->fn? lf->fn : lf->pfn));
  }
  if (!lf->prev) list_files=lf->next;
  Destroy_Listitem(lf);
  return list_files;
}

// remove files if unknown or non-picture data file
static LIST_FILES *remove_file(LIST_FILES *list_files,gboolean rm_unk,gboolean rm_nonpic)
{
  LIST_FILES *lf,*lfnext;
  for (lf=list_files; lf; lf=lfnext)
  {
    lfnext=lf->next;
    /* Handle unknown files (not recognized by fn2xrit()) */
    if ((rm_unk) && (lf->unknown))
    {   
      list_files=remove_this(list_files,lf);
    }
    else if ((rm_nonpic) && ((lf->pxh.xrit_frmt==DWDSAT) || (lf->pxh.xrit_frmt==BUFR)))
    {   
      list_files=remove_this(list_files,lf);
    }
  }
  return list_files;
}



static SEGMENT *Find_Segmfromfile(GROUP *grp,char *fn)
{
  CHANNEL *chan;
  SEGMENT *segm;
  if (!fn) return NULL;
  for (; grp; grp=grp->next)
  {
    for (chan=grp->chan; chan; chan=chan->next)
    {
      for (segm=chan->segm; segm; segm=segm->next)
      {
        if (!strcmp(fn,segm->fn)) return segm;
      }
    }
  }
  return NULL;
}


/* remove lf from list */
static void extract_lf(LIST_FILES *lf)
{
  if (lf->next) lf->next->prev=lf->prev;
  if (lf->prev) lf->prev->next=lf->next;
  lf->prev=NULL;
  lf->next=NULL;
}

static LIST_FILES *add_lf_order(LIST_FILES *lf,LIST_FILES *lf2)
{
  LIST_FILES *lf1;
  if (!lf) return lf2;
  for (lf1=lf; lf1; lf1=lf1->next)
  {
    if (!strcmp(lf1->pxh.satsrc,lf2->pxh.satsrc))
    {
      struct tm tm;
      long sec;
      tm=lf1->pxh.time;
      sec=mktime(&tm);
      tm=lf2->pxh.time;
      sec=mktime(&tm)-sec;  // time_lf2 - time_lf1
      if (sec<0) break;     // time lf2 > time lf1
    }
  }

/* Add lf2 before lf1 */
  if (lf1)
  {
    if (lf1->prev) lf1->prev->next=lf2; else lf=lf2;
    lf2->prev=lf1->prev;
    lf2->next=lf1;
    lf1->prev=lf2;
  }
  else
  {
    for (lf1=lf; lf1->next; lf1=lf1->next);
    lf1->next=lf2;
    lf2->prev=lf1;
  }
  return lf;
}

static void update_chanstr(GROUP *grp,CHANNEL *chan,GtkCTree *ctree)
{
  int nr_chans=strlen(grp->chanstr);
  if ((nr_chans < MAXCHANSTR-1) && (chan->chan_name))
  {
    grp->chanstr[nr_chans]=chan->chan_name[0];
    grp->chanstr[nr_chans+1]=0;
  }

#ifndef __NOGUI__
  if ((ctree) && (grp->node))
    gtk_ctree_node_set_text(ctree,grp->node,5,grp->chanstr);
#endif
}

/******************************************************************
 * Update channel 1: add segments of chan2 to chan1
 *****************************************************************/
static void update_chan(GROUP *grp1,CHANNEL *chan1,CHANNEL *chan2,GtkCTree *ctree)
{
  SEGMENT *segm1,*segm2,*segm2next;

  // test for double segments (???kandat??); remove double
  for (segm2=chan2->segm; segm2; segm2=segm2next)     /* scan through segm. in new chan */
  {
    segm2next=segm2->next;
    /* Check if there are double segments */
    for (segm1=chan1->segm; segm1; segm1=segm1->next) /* scan through segm. in cur. chan */
    {
//      if (!segm1->xh) { unexp("segm1->xh=0: %s",chan1->chan_name); continue;  } /* shouldn't occur... */
//      if (!segm2->xh) { unexp("segm2->xh=0: %s",chan2->chan_name); continue;  } /* shouldn't occur... */
      if (segm1->xh.segment==segm2->xh.segment)  // segment already exist
      {                                          // segment will be removed
#ifdef DEBUG_KLAD
if (segm1->xh.segment) w_dbg("??? equal segment??? %s / %s, %d  pe=%d/%d\n",grp1->sat_src,
chan1->chan_name,segm1->xh.segment,chan1->pro,chan1->epi);
#endif

        if (!segm2->prev)                        // first segm of chan2?
          chan2->segm=segm2->next;               // Change link chan->segm
        Remove_Segm(segm2);                      // Now remove segm2
        segm2=NULL;
        chan2->nr_segm--;                        // correct nr new segments
        break;
      }
    }

    /* Update string of existing channels if current segment is number 1 */
    if ((segm2) && (segm2->xh.segment==chan2->seq_start))
    {
      update_chanstr(grp1,chan2,ctree);
    }
  }


/* remaining segments are new; add to segments in channel in main group */
  if (chan2->segm)                              /* new segments to add */
  {
    Add_Segm(&chan1->segm,chan2->segm);         /* connect remaining segms to end of segms of chan1. */
    for (segm1=chan2->segm; segm1; segm1=segm1->next)
    {
      segm1->chan=chan1;                        /* Change link from chan2 to chan1 */
    }
    chan1->nr_segm+=chan2->nr_segm;
    if (!chan1->seq_end)
    {
      chan1->seq_start=chan2->seq_start;
      chan1->seq_end=chan2->seq_end;
      chan1->nc=chan2->nc;
      chan1->nl=chan2->nl;
      chan1->nb=chan2->nb;
      chan1->ifrmt=chan2->ifrmt;
      chan1->scan_dir=chan2->scan_dir;
      chan1->lines_per_sec=chan2->lines_per_sec;
    }
    
    /* For AVHRR: Update seq_end */

    if (strchr("GAM",grp1->h_or_l)) chan1->seq_end+=chan2->nr_segm; /*METOP*/

/*
Kan niet sorteren, of alle segm. eerst verwijderen!
    chan1->segm=Sort_Segm(chan1->segm);
*/

    /* New segments still exist from chan2; use it to update gui-tree */
    for (segm1=chan2->segm; segm1; segm1=segm1->next)
    {
      Add_Segm_totree(ctree,grp1,chan1,segm1);
    }

    chan2->segm=NULL;           /* Now chan2 can be freed without loss of segm! */
    /* chan2 is removed lateron; has to deal with old group of this channel! */

#ifndef __NOGUI__
    if ((ctree) && (chan1->node))
    {
      GdkColor clr_chok={0,0,0x9999,0};  /* color channel item complete */
      GdkColor clr_chft={0,0xffff,0,0};  /* color channel item not complete */
      GdkColor clr_chan;
      char strrange[50];
      sprintf(strrange,"%d of %d ",chan1->nr_segm,chan1->seq_end-chan1->seq_start+1);
      gtk_ctree_node_set_text(ctree,chan1->node,6,strrange);
      gtk_ctree_node_set_text(ctree,chan1->node,7,strrange);
      if (chan1->nr_segm==chan1->seq_end-chan1->seq_start+1)
        clr_chan=clr_chok;
      else
        clr_chan=clr_chft;
      gtk_ctree_node_set_foreground(ctree,chan1->node,&clr_chan);
    }
#endif
  }
}

/* grp1 and grp2 have same id, so combine into 1 group. */
static gboolean  update_group(GROUP *grp1,GROUP *grp2,GtkCTree *ctree)
{
  CHANNEL *chan1,*chan2,*chan2next;
  gboolean last=FALSE;
/* Add segments part of channel which is already in main group */
  for (chan2=grp2->chan; chan2; chan2=chan2next)       /* scan through new channels (to add) */
  {
    chan2next=chan2->next;
    if (!chan2->chan_name) continue;
    for (chan1=grp1->chan; chan1; chan1=chan1->next)   /* scan through existing channels */
    {
      if (!chan1->chan_name) continue;
      if (!strcmp(chan1->chan_name,chan2->chan_name))  /* channel already exist */
      {
        update_chan(grp1,chan1,chan2,ctree);           /* add new items in chan2 to chan1 */
        if (!chan2->prev)                              /* first in group? */
          grp2->chan=chan2->next;                      /* Change link from grp to chan */
        Remove_Chan(chan2);                            /* Now remove channel */
        break;
      }
    }
  }

/* remaining channels are new; add to channels in main group */
  if (grp2->chan)                          /* new channel to add */
  {
    Add_Chan(&grp1->chan,grp2->chan);         /* connect remaining segms to end of segms of chan1. */

/* update channel overview with new channel */
    for (chan2=grp2->chan; chan2; chan2=chan2->next)
    {
      chan2->group=grp1;
      if (chan2->segm)
      {
        if (chan2->segm->xh.segment==chan2->seq_start)  /* Add channel code (column Format) */
        {
          update_chanstr(grp1,chan2,ctree);
        }
      }
      Add_Chan_totree(ctree,grp1,chan2);  /* update tree: add new channels or pro/epi */
    }

    grp2->chan=NULL;           /* Now grp2 can be freed without loss of chan! */
  }

  if (grp2->pro_epi)
  {
    if (!grp1->pro_epi)
    {
      grp1->pro_epi=grp2->pro_epi;
      grp2->pro_epi=NULL;
      grp1->pro_epi->group=grp1;
      Add_Chan_totree(ctree,grp1,grp1->pro_epi);  /* update tree: add new channels */
      if ((grp1->pro_epi) && (grp1->pro_epi->epi)) last=TRUE;
    }
    else
    {
      if (grp2->pro_epi->pro)
      {
        if (!grp1->pro_epi->pro)                   /* only add if not present yet */
        {
          grp1->pro_epi->pro=grp2->pro_epi->pro;
          grp2->pro_epi->pro=NULL;
          grp1->pro_epi->pro->chan=grp1->pro_epi;
          Add_Segm_totree(ctree,grp1,grp1->pro_epi,grp1->pro_epi->pro);
        }
      }
      if (grp2->pro_epi->epi)
      {
        last=TRUE;
        if (!grp1->pro_epi->epi)                   /* only add if not present yet */
        {
          grp1->pro_epi->epi=grp2->pro_epi->epi;
          grp2->pro_epi->epi=NULL;
          grp1->pro_epi->epi->chan=grp1->pro_epi;
          Add_Segm_totree(ctree,grp1,grp1->pro_epi,grp1->pro_epi->epi);
        }
      }
    }
  }
  return last;
}


/******************************************************
 * Update GROUP->CHAN->SEGM database with new items in filelist 'lf'
 * and add to ctree
 ******************************************************/
static GROUP *update_dbase(GtkCTree *ctree,GROUP *curgrp,LIST_FILES *lf,
                    LOCATION_SEGM where,
                    int progress_func())
{
  GROUP *nwgrp,*grp1,*grp2,*grp2next,*lastgrp=NULL;

  nwgrp=mk_dbase(lf,NULL,where,progress_func);     /* make new group-set of current files */
#ifndef __NOGUI__
  if (ctree) gtk_clist_freeze((GtkCList *)ctree); /* Freeze list for fast creation */
#endif
  
/* check each nwgrp if it contains items not yet in current group */
  for (grp1=curgrp; grp1; grp1=grp1->next)    /* scan through existing group */
  {
    for (grp2=nwgrp; grp2; grp2=grp2next)     /* scan through new group (items to add) */
    {
      grp2next=grp2->next;
      if (!strcmp(grp1->id,grp2->id))          /* channel belongs to already existing group */ 
      {
        if (update_group(grp1,grp2,ctree))         /* add new items in grp2 to grp1 */
        {
          lastgrp=grp1;
        }
/* grp2 done; remove it */
        if (!grp2->prev)
          nwgrp=grp2->next;
        Remove_Grp(grp2);
        break;
      }
    }
  }

/* remaining groups are new; add to main group; */
  if (nwgrp)
  {
    for (grp1=curgrp; grp1 && grp1->next; grp1=grp1->next);
    if (grp1)
    {
      grp1->next=nwgrp;
      nwgrp->prev=grp1;
    }
    globinfo.time_last_rec=nwgrp->grp_tm.tm_hour;
  }


  if ((globinfo.pr.msg_notify) && (lastgrp))
  {
    msgnotify(lastgrp,check_msg(lastgrp));
  }

#ifndef __NOGUI__
  if (ctree) gtk_clist_thaw((GtkCList *)ctree);
#endif
  return nwgrp;
}


static LIST_FILES *sort_avhrr_lf(LIST_FILES *lf,char type)
{
  LIST_FILES *lf1,*lf2,*lf2next,*lftmp;
  char *satname=NULL;
  lftmp=NULL;
  lf1=lf;
  while (lf1)
  {
    if (lf1->pxh.xrit_frmt!=type) { lf1=lf1->next; continue; } // no AVHRR
    satname=lf1->pxh.satsrc;
    for (lf2=lf1; lf2; lf2=lf2next)
    {
      lf2next=lf2->next;
      if (!strcmp(satname,lf2->pxh.satsrc))
      {
        if (!lf2->prev) lf=lf2->next;
        extract_lf(lf2);
        lftmp=add_lf_order(lftmp,lf2);
        lf1=lf;
      }
    }
  }
  if (!lf)
  {
    lf=lftmp;
  }
  else if (lftmp)
  {
    for (lf1=lf; ((lf1) && (lf1->next)); lf1=lf1->next);
    lf1->next=lftmp;
    lftmp->prev=lf1;
  }
  return lf;
}

#define DMIN_NEWORBIT 4   // more: treat as new orbit
/* !! Probleem als gedeelte in received en gedeelte in done! */

// voor avhrr-metop: min_segm =1 -> uit data of fnaam bepalen!
static void update_avhrr(LIST_FILES *lfi,char type)  /*NOAA*/
{
  LIST_FILES *lf,*lfa=NULL,*lfp=NULL;
  int min_segm=(type==METOP? 3 : 1);    // bepaal 

  int neighbours;
  for (lf=lfi; lf; lf=lf->next)
  {
    if (lf->pxh.xrit_frmt!=type) continue;  // no AVHRR
    if (!lfp)
    {
      lfa=lf;
      lf->pxh.seq_start=1;
      lf->pxh.seq_end=1;
      lf->pxh.segment=1;
      lfp=lf;
      continue;
    }

    neighbours=0;
    if (!strcmp(lfp->pxh.satsrc,lf->pxh.satsrc))
    {
      int dt=mktime_ntz(&lf->pxh.time)-mktime_ntz(&lfp->pxh.time);
      int dsegm=dt/(min_segm*60);
      if ((dsegm>0) && (dsegm<=DMIN_NEWORBIT)) neighbours=dsegm;
    }
    if (neighbours)
    {
      lf->pxh.segment=lfp->pxh.segment+neighbours;
      lfa->pxh.seq_start=1;
      lfa->pxh.seq_end=lf->pxh.segment;
      lfa->pxh.seq_no=lf->pxh.segment;
    }
    else                   // start new path
    {
      lfa=lf;              // 1e segment of group
      lf->pxh.segment=1;
      lf->pxh.seq_start=1; // toegevoegd 20-11-2011; nw metop-segment!
      lf->pxh.seq_end=1;
    }
    lfp=lf;
  }

  lfa=NULL;
  for (lf=lfi; lf; lf=lf->next)
  {
    if (lf->pxh.xrit_frmt!=type) continue;  // no AVHRR

    if (lf->pxh.segment==1)
    {
      lfa=lf;
    }
    else if (lfa)
    {

// Volgende geeft alle segm zelfde start-tijd. Waarom?
// Weggehaald ivm NOAA positieberekeningen! (2-12-06)
//      lf->pxh.time=lfa->pxh.time;
      lf->pxh.pic_id=lfa->pxh.pic_id; // make id's of segments 1 group the same
    }
  }  
}

/* Remove files if not in keep-list */
static LIST_FILES *filter_list(LIST_FILES *list_files,GROUP *grp,GtkWidget *wdgt)
{
  LIST_FILES *lf,*lfnext;
  Set_Entry(wdgt,LAB_RECINFO,"");
  for (lf=list_files; lf; lf=lfnext)
  {
    char tmp[10];
    lfnext=lf->next;
    if (lf->unknown) continue;    /* This is done by 'remove_unk" */

    // Distinquish between HMSG and LMSG  
    if (!strncmp(lf->pxh.src,"MSG1",4))
      sprintf(tmp,"%c%s",lf->pxh.hl,"MSG1");
    else if (!strncmp(lf->pxh.src,"MSG2",4))
      sprintf(tmp,"%c%s",lf->pxh.hl,"MSG2");
    else if (!strncmp(lf->pxh.src,"MSG3",4))
      sprintf(tmp,"%c%s",lf->pxh.hl,"MSG3");
    else if (!strncmp(lf->pxh.src,"MSG4",4))
      sprintf(tmp,"%c%s",lf->pxh.hl,"MSG4");
    else
      strcpy(tmp,lf->pxh.src);

    if (globinfo.satlist->enable_remove)
    {
      if (sat_to_remove(tmp,lf->pxh.chan,lf->pxh.special))
      {  // in list, so remove
  /* Check if this file already in grp, don't remove if so! */
        if ((lf->fn) && (!lf->is_dir) && (!Find_Segmfromfile(grp,lf->fn)))
        {
          if (!(remove(lf->pfn)))
          {
            if (globinfo.log) print_log(NULL,"Removed xrit: %s\n",(lf->fn? lf->fn : lf->pfn));
            if (lf->pxh.hl)
              Set_Entry(wdgt,LAB_RECINFO,"Removed %c %s-%d",
                        lf->pxh.hl,lf->pxh.chan,lf->pxh.segment);
            else
              Set_Entry(wdgt,LAB_RECINFO,"Removed %s",lf->fn);
          }
          if (!lf->prev) list_files=lf->next;
          Destroy_Listitem(lf);
        }
      }
    }
  }

  return list_files;
}



void add_grp_to_showlist(GROUP *grp,GtkWidget *widget);

/*******************************************************
 * Load xrit-files at location '*path' into tree.
 * clear=true  -> Clear prev. tree and start from scratch.
 * clear=false -> Read new items and add to existing tree.
 * where=in_done -> Assumed to read from done dir.
 *******************************************************/
GROUP *Load_List(GROUP *grp,GtkCTree *ctree,GtkWidget *widget,
                 char *path,gboolean clear,
                 LOCATION_SEGM where,
                 gboolean cont_update,GENMODES *gmode)
{
  GtkWidget *wnd=NULL;
  LIST_FILES *list_files=NULL;
  GROUP *nwgrp;
  gboolean show_progress=((!cont_update) && (widget));
#ifndef __NOGUI__
  GROUP *grp1;
  GtkWidget *widgeti=widget;
  wnd=Find_Parent_Window(widget);
#endif
  // clear dbase and gtklist
  if (clear)
  {
#ifndef __NOGUI__
    if (ctree) gtk_clist_clear(GTK_CLIST(ctree)); /* Clear list to start from scratch */
#endif
    Remove_Grps(grp);
    grp=NULL;
    Destroy_List(dbase.unknown,TRUE);              /* Remove unk list */
    dbase.unknown=NULL;
  }

  // message
  if (show_progress)
  {
    Set_Entry(widget,LAB_INFO,"Make filelist...");
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }

  // create list of files
  list_files=create_filelist(path);    /* Generate a list of all files/dirs */

  if (show_progress)
  {
    Set_Entry(widget,LAB_INFO,"Update filelist...");
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }
  
  // update list
  update_filelist(list_files,cont_update,NULL);   
  if (show_progress)
  {
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }
    
  if (!list_files)
  {
    if (wnd)
    {
      if (where == in_received)
      {
        Create_Message("Error","Can't open directory\n  %s.",path);
      }
    }
    if (show_progress)
    {
      Set_Entry(widget,LAB_INFO,"");
#ifndef __NOGUI__
      while (g_main_iteration(FALSE));
#endif
    }
    return 0;
  }

  // remove unneeded files
  if (cont_update)
  {                               /* Remove files if requested */
    if (globinfo.pic_only)
    {
      list_files=remove_file(list_files,TRUE,TRUE);
    }
    if (globinfo.satlist->enable_remove)
    {
      // voor unk alleen
      list_files=remove_file(list_files,globinfo.satlist->unk_sel,FALSE);
      if (!globinfo.gen_new)
        list_files=filter_list(list_files,grp,widget);
    }
    widget=NULL;                  /* Suppress progress windows */   
  }

  // sort&update avhrr
  if (show_progress)
  {
    Set_Entry(widget,LAB_INFO,"Update avhrr.........");
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }
  list_files=sort_avhrr_lf(list_files,NHRPT);
  update_avhrr(list_files,NHRPT);

  list_files=sort_avhrr_lf(list_files,MHRPT);
  update_avhrr(list_files,MHRPT);

  list_files=sort_avhrr_lf(list_files,METOP);
  update_avhrr(list_files,METOP);

  if (show_progress)
  {
    Set_Entry(widget,LAB_INFO,"");
#ifndef __NOGUI__
    while (g_main_iteration(FALSE));
#endif
  }
  
  // update/make database
  if (grp)
  {
    if (widget)
    {
      nwgrp=update_dbase(ctree,grp,list_files,where,(void *)progress_func); /* Update dbase */
    }
    else
      nwgrp=update_dbase(ctree,grp,list_files,where,NULL); /* Update dbase */

#ifndef __NOGUI__
      if (cont_update)    show_tree(ctree,nwgrp,FALSE,!cont_update);                /* Add new groups to tree */
      for (grp1=nwgrp; grp1; grp1=grp1->next)
        add_grp_to_showlist(grp1,widgeti);
#endif
  }
  else                                           /* Start building dbase and tree from scratch */
  {
    if (widget)
      nwgrp=mk_dbase(list_files,gmode,where,(void *)progress_func);
    else
      nwgrp=mk_dbase(list_files,gmode,where,NULL);
#ifndef __NOGUI__
    if (cont_update) show_tree(ctree,nwgrp,TRUE,!cont_update);
    for (grp1=nwgrp; grp1; grp1=grp1->next)
      add_grp_to_showlist(grp1,widget);
#endif
    grp=nwgrp;
  }
  Destroy_List(list_files,TRUE);                /* Remove list, but NOT pointers inside! */

  if (!cont_update)                             /* not in update mode: */
    grp=Sort_Grp(grp);                          /* sort received/done */

  return grp;
} 

GROUP *Load_List_all(GROUP *grp,GtkCTree *ctree,GtkWidget *widget,GENMODES *gmode)
{
  GROUP *grp1;
  grp1=Load_List(grp ,ctree,widget,globinfo.src_donedir,TRUE ,in_done    ,FALSE,gmode);
  grp =Load_List(grp1,ctree,widget,globinfo.pl.src_dir ,FALSE,in_received,FALSE,gmode);
  if (!grp) grp=grp1;
#ifndef __NOGUI__
  Set_Entry(widget,LAB_INFO,"Building tree...");
  while (g_main_iteration(FALSE));
  show_tree(ctree,grp,TRUE,TRUE);  /* redraw tree */
  Set_Entry(widget,LAB_INFO,"");
  while (g_main_iteration(FALSE));
  refresh_showmenu(grp,widget);
#endif
  return grp;
}
