/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * misc functions
 ********************************************************************/
#include "xrit2pic.h"
#include "xrit2pic_defines.h"
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <math.h>

int igamma(int p,int pmax,float g)
{
  float pg=pow((float)p/(float)pmax,1./g);
  return (int)(pg*pmax);
}

/*************************************
 * Wild char comparator
 *************************************/
int strcmpwild(char *s,char *v)
{
  while ((*v) && (*s))
  {
    if (*v=='*')
    {
      v++;
      if (!*v) return 0;                   /* "abc"=="abc*" */
      while ((*s) && (strcmpwild(s,v))) s++;
    }
    if ((*v!='?') && (*v!=*s)) return 1;   /* "abc"=="adb" */
    v++; s++;
  }
  if (*s) return 2;                        /* "abc"=="ab" */
  while (*v=='*') v++;
  if (*v) return 3;                        /* "ab"=="abc" */
  return 0;                                /* "abc"=="a*c" enz. */
} 
     
int strcasecmpwild(char *s,char *v)
{
  while ((*v) && (*s))
  {
    if (*v=='*')
    {
      v++;
      if (!*v) return 0;                   /* "abc"=="abc*" */
      while ((*s) && (strcasecmpwild(s,v))) s++;
    }
    if ((*v!='?') && (tolower(*v)!=tolower(*s))) return 1;   /* "abc"=="adb" */
    v++; s++;
  }
  if (*s) return 2;                        /* "abc"=="ab" */
  while (*v=='*') v++;
  if (*v) return 3;                        /* "ab"=="abc" */
  return 0;                                /* "abc"=="a*c" enz. */
}      

int str_ends_with(char *s,char *e)
{
  return !strcmp(s+strlen(s)-strlen(e),e);
}

 #if __GTK_WIN32__ == 1
// NIET helemaal goed, maar voor hier goed genoeg!
char *strcasestr(const char *haystack, const char *needle)
{
  char *p,*q=needle;
  for (p=haystack; *p; p++)
  {
    if (tolower(*p)==tolower(*q)) q++; else q=needle;
    if (*q==0) return p;
  }
  return NULL;
}
#endif
 

gboolean is_ir_chan(CHANNEL *chan)
{
  if (!chan) return FALSE;
  if (!strncasecmp(chan->chan_name,"ir_016",6)) return FALSE;
  if (!strncasecmp(chan->chan_name,"ir",2)) return TRUE;
  if (!strncasecmp(chan->chan_name,"wv",2)) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch04")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch05")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch06")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch07")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch08")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch09")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch10")) return TRUE;
  if (strcasestr(chan->chan_name,"msg-ch11")) return TRUE;
  if ((strcasestr(chan->chan_name,"M1C")) || 
      (strcasestr(chan->chan_name,"M2C")) ||
      (strcasestr(chan->chan_name,"M3C")))
  {// all GOES16 chans
    char *p=strchr(chan->chan_name,'C');
    if (!p) return TRUE;
    p++;
    if (atoi(p) <= 3) return FALSE;
    return TRUE;
  }
  return FALSE;
}

gboolean is_ir_group(GROUP *grp)      // one single ir in group
{
  if (!grp) return FALSE;
  if (!grp->chan) return FALSE;
  if (grp->chan->next) return FALSE;  // composite -> ignore 'is_ir'
  return is_ir_chan(grp->chan);
}

int chan_complete(CHANNEL *chan,int chnk_first,int chnk_last)
{
  int i;
  for (i=chnk_first; i<=chnk_last; i++)
    if (!Get_Segm(chan,i)) return 0;              /* Not all needed segments available */
  return 1;
}

void range2int(char *range,int *s1,int *s2)
{
  char *p;
  if (!range) return;
  *s1=atoi(range);
  if ((p=strchr(range,'-')))
    *s2=atoi(p+1);
  else
    *s2=*s1;
}

/* Check if complete.
   Return: 0  1 if OK
           1 -1 if prologue HRV missing but otherwise complete
           2 -2 if error (nrsegm > seq_end)
           3  0 if incomplete
*/
int channel_complete(CHANNEL *chan)
{
  if (chan->nr_segm==chan->seq_end-chan->seq_start+1)
  {
    if ((chan->chan_name) && (!strcmp(chan->chan_name,"HRV")) && (chan->group) && (!chan->group->pro_epi))
      return 1;
    else
      return 0;
  }
  if (chan->nr_segm>chan->seq_end) return 2;
  return 3;
}

// If area not full globe: for the moment ensure ALL segments are present!
#define DETECT_LIM_SEGM 0
/* Check if all needed segments are available. Use gmode->chanmap */
/* !!! Toevoegen: Hires: controleer of HRV compleet!!! */

int picture_complete(CHANNEL *chans,PIC_CHARS *pc,GENMODES *gmode)
{
  CHANNEL *chan;
  CHANMAP *chm;
  int i;
  int segm_first,segm_last;
  int has_something=0;
  char infostr[100];
  if (gmode->chanmap)
  {
    int err=0;
    *infostr=0;
//    prlog("          chanmap\n");
    for (chm=gmode->chanmap; chm; chm=chm->next)
    {
      // Check if chan available.
      if (!(chan=Get_Chan(chans,chm->chan_name)))
      {
        err++;
#if DEBUG_KLAD==1
        if (strlen(infostr)+strlen(chm->chan_name) < 90) sprintf(infostr,"%s %s?",infostr,chm->chan_name);
#endif
        prlog("          No chan '%s'\n",chm->chan_name);
        return 0;  // chan not available
      }
#if DEBUG_KLAD==1
      else
      {
        if (strlen(infostr)+strlen(chm->chan_name) < 90) sprintf(infostr,"%s %s!",infostr,chm->chan_name);
      }
#endif
      if (chan)
      {
        segm_first=chan->seq_start;
        segm_last=chan->seq_end;

        #if DETECT_LIM_SEGM != 0
        if (gmode->area_nr)
           segm_first=segm_last-segm_last/4+1;
        #endif


        // Check if all needed segments of this channel are available.
        for (i=segm_first; i<=segm_last; i++)
        {
          if (!Get_Segm(chan,i))
          {
//            prlog("  Missing segm: %s  %d\n",chan->chan_name,i);
#if DEBUG_KLAD==1
            if (strlen(infostr)+strlen(chm->chan_name) < 90) sprintf(infostr,"%s[%d?]",infostr,i);
#endif
            err++; //return 0;           /* Not all needed segments available */
          }
        }
        if ((chan->chan_name) && (!strcmp(chan->chan_name,"HRV")) && (chan->group) && (!chan->group->pro_epi))
          err++;
      }


    }

    if (err) return 0;
    return 1;
  }

/* Rest te vervangen door voorgaande */
  if (pc->is_color)
  {
#ifdef XXX
    for (chan=chans; chan; chan=chan->next)
    {
      if ((chan->r) || (chan->g) || (chan->b))
      {
        has_something=1;
        segm_first=chan->seq_start;
        segm_last=chan->seq_end;
        #if DETECT_LIM_SEGM != 0
        if ((gmode) && (gmode->area_nr))
           segm_first=segm_last-segm_last/4+1;
        #endif
        // Check if all needed segments of this channel are available.
        for (i=segm_first; i<=segm_last; i++)
          if (!Get_Segm(chan,i)) return 0;           /* Not all needed segments available */
      }
    }
#endif
  }
  else
  {
w_dbg("moetditnog?\n");
    chan=chans;
    has_something=1;
    segm_first=chan->seq_start;
    segm_last=chan->seq_end;
    #if DETECT_LIM_SEGM != 0
    if ((gmode) && (gmode->area_nr))
       segm_first=segm_last-segm_last/4+1;
    #endif

    for (i=segm_first; i<=segm_last; i++)
      if (!Get_Segm(chan,i)) return 0;           /* Not all needed segments available */
    if ((chan->chan_name) && (!strcmp(chan->chan_name,"HRV")) && (chan->group) && (!chan->group->pro_epi))
      return 0;

  }
  return has_something;
}


/* Open temp-file. For Linux: fd returns file destructor. */
static int gen_tempfile(char *fnt)
{
  int fd=0;
  #if __GTK_WIN32__ == 1
    char *ifnt;
    if (!(ifnt=tmpnam(NULL)))
    {
      ifnt="xrit2pic_temp.tmp";
    } 
  #else
    char ifnt[50];
    strcpy(ifnt,"/tmp/tmp_xrit2pic_XXXXXX");
    if ((fd=mkstemp(ifnt)) == -1) return -1;
  #endif
  if (fnt) strcpy(fnt,ifnt);
  return fd;
}

FPFNTMP clear_fpfntmp()
{
  FPFNTMP fpfntmp;
  memset(&fpfntmp,0,sizeof(fpfntmp));
  fpfntmp.fd=-1;
  return fpfntmp;
}

// Create a temp. file;
// If success: fpfntmp.fn allocated string with filename
FPFNTMP create_tempfile()
{
  char fnt[50];
  FPFNTMP fpfntmp=clear_fpfntmp();
  fpfntmp.fd=gen_tempfile(fnt);  // Now 
  if (fpfntmp.fd==-1)
  {
    return fpfntmp;         // nothing done; fp=NULL, fd=-1, fn=NULL
  }
  rmclose(&fpfntmp);        // only closes file descriptor, empty fnt now exists

  strcpyd(&fpfntmp.fn,fnt); // next rmclose will remove the file!
  return fpfntmp;
}


// Create a new temp file and open for write
FPFNTMP cropen()
{
  FPFNTMP fpfntmp=create_tempfile();
  if (fpfntmp.fn)
  {
    fpfntmp.fp=fopen(fpfntmp.fn,"wb");
    if (!fpfntmp.fp) rmclose(&fpfntmp);
  }
  return fpfntmp;
}

// If fni = zipped file:
//    unzip
//    fpfntmp.fp=pointer to opened unzipped file
//    fpfntmp.fn=pointer to string with name of unzipped file
// If fni = NOT zipped:
//    fpfntmp.fp=pointer to opened file
//    fpfntmp.fn=NULL
//
// bunzip2 fni into temp file fpfntmp.fn
// open for read: fpfntmp.fp
FPFNTMP cropen_bunzip(char *fni)
{
  FPFNTMP fpfntmp=clear_fpfntmp();
  if (str_ends_with(fni,".bz2"))
  {
    fpfntmp=create_tempfile();
    if (!fpfntmp.fn)
    {
      return fpfntmp;         // nothing done; fp=NULL, fd=-1, fn=NULL
    }

    if (bunzip2(fni,fpfntmp.fn))
    {                         // bunzip2 failed
      rmclose(&fpfntmp);       // remove file
      return fpfntmp;
    }
    fpfntmp.fp=fopen(fpfntmp.fn,"rb");  // open unzipped file (in temp file)
    if (!fpfntmp.fp) rmclose(&fpfntmp); // failed to open; remove and free
  }
  else
  { // non-zipped file; keep fpfntmp.fn NULL!
    fpfntmp.fp=fopen(fni,"rb");  // open non-zipped file
  }
  return fpfntmp;
}

// Close fpfntmp.fp
//    if fpfntmp.fn: remove  file; free fn
void rmclose(FPFNTMP *fpfntmp)
{
//printf("rmclose: %x  %d  %s\n",fpfntmp->fp,fpfntmp->fd,fpfntmp->fn);
  if (fpfntmp->fp)    fclose(fpfntmp->fp); fpfntmp->fp=NULL;
  #if __GTK_WIN32__ != 1
  if (fpfntmp->fd>=0) close(fpfntmp->fd);  fpfntmp->fd=-1;
  #endif

  if (fpfntmp->fn)
  {
  #if __GTK_WIN32__ != 1
    if (strstr(fpfntmp->fn,"tmp_xrit2pic"))
      remove(fpfntmp->fn);
    else
      printf("!!!ERR!!! wil '%s' weggooien!\n",fpfntmp->fn);
  #else
      remove(fpfntmp->fn);
  #endif
    freed(&fpfntmp->fn);
  }
}

int exist_prog(char *fn,char *opt)
{
  char *cmd;
  int err;
  cmd=malloc(strlen(fn)+strlen(opt)+10);
  if (!cmd) return 0;
  sprintf(cmd,"%s %s",fn,opt);
  err=system(cmd);
  free(cmd);
  if (err) return 0;
  return 1;
}

/* Detect if file exist */
int exist_file(char *fn)
{
  FILE *fp;
  if (!fn) return 0;
  if (!(*fn)) return 0;
  if (!(fp=fopen(fn,"r"))) return 0;
  fclose(fp);
  return 1;
}

int exist_file_in_dir(char *dir,char *fn)
{
  char *tmp=NULL;
  int res;
  if (!dir) return 0;
  if (!fn) return 0;
  strcpyd(&tmp,dir);
  finish_path(tmp);
  strcatd(&tmp,fn);
  res=exist_file(tmp);
  freed(&tmp);
  return res;
}

#if __GTK_WIN32__ == 1
  #define COPY_PROG "copy"
#else
  #define COPY_PROG "cp"
#endif
int copy_file(char *a,char *b)
{
  char *cmd=NULL;
  int err;
  sprintfd(&cmd,"%s \"%s\" \"%s\"",COPY_PROG,a,b);
  if (!cmd) return -1;
  err=system(cmd);
  freed(&cmd);
  return err;
}

int sprintfd(char **s,char *f,...)
{
  int len;
  char *s2;
  va_list ap;
  va_start(ap,f);
  len=vsnprintf(NULL,0,f,ap);
  va_end(ap);
  if (len<0) return 0;
  if (!((s2=malloc(len+1)))) return 0;
  va_start(ap,f);
  len=vsprintf(s2,f,ap);
  va_end(ap);
  if (*s) free(*s);
  *s=s2;
  return len;
}

int memcpyd(void **sp,void *s,int n)
{
  if (!sp) return 1;
  if (!s) return 1;
  if (!(*sp=(char *)malloc(n))) return 1;
  memcpy(*sp,s,n);
  return 0;
}

/* Allocate and copy. 2 extra bytes allocated: for closing 0 and maybe 1 extra char to add. */
int strcpyd(char **sp,char *s)
{
  if (!sp) return 1;
  if (!s) return 1;
  if (!(*sp=(char *)malloc(strlen(s)+2))) return 1;
  strcpy(*sp,s);
  return 0;
}

void freed(char **s)
{
  if ((s) && (*s)) free(*s);
  *s=NULL;
}

/* Allocate and concat. 2 extra bytes allocated: for closing 0 and maybe 1 extra char to add. */
int strcatd(char **sp,char *s)
{
  if (!sp) return 1;
  if (!*sp)
  {
    strcpyd(sp,s);
    return 0;
  }
  
  if (!(*sp=(char *)realloc(*sp,strlen(*sp)+strlen(s)+2))) return 1;
  strcat(*sp,s);
  return 0;
}

int strcmpd(char *s1,char *s2)
{
  if (s1==s2) return 0;
  if ((s1==NULL) || (s2==NULL)) return -1;
  return strcmp(s1,s2);
}

int cat_pathfiled(char **pf,char *p,char *f)
{
  if ((!*p) || (p[strlen(p)-1]==DIR_SEPARATOR))
    return sprintfd(pf,"%s%s",p,f);
  else
    return sprintfd(pf,"%s%c%s",p,DIR_SEPARATOR,f);
}

void strcatc(char *s,char c)
{
  *(s+strlen(s)+1)=0;
  *(s+strlen(s))=c;
}


/*************************************
 * Extract number defined between positions 's' and 'e' from  l
 * and return as float
 *************************************/
#define TMPLEN 300
char *get_strpart(char *l,int s,int e)
{
  static char tmp[TMPLEN];
  int nrb=e-s+1;
  if (nrb>TMPLEN-1) nrb=TMPLEN-1;
  tmp[nrb]=0;
  if (strlen(l+s) < nrb)
  {
    strcpy(tmp,"");
  }
  else
  {
    strncpy(tmp,l+s,nrb);
    tmp[e-s+1]=0;
  }
  return tmp;
}

void strtoupper(char *a)
{
  for (; *a; a++) *a=toupper(*a);
}

// ! path moet ruimte hebben voor 1 extra kar. Wordt ook door strcpyd enz. gedaan.
void finish_path(char *path)
{
  if (!path) return;
  if (RCHAR(path) != DIR_SEPARATOR)
    strcatc(path,DIR_SEPARATOR);
}

/*************************************
 * Correct tm-struct and return #seconds since 1970.
 * ntz stands for "no time-zone";
 * similar to mktime, except that daylight saving is ignored.
 *************************************/
gint32 mktime_ntz(struct tm *tm)
{
  gint32 secs;
  gint32 rdays;
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};

  while (tm->tm_sec >= 60) { tm->tm_min++;  tm->tm_sec-=60; }
  while (tm->tm_sec <   0) { tm->tm_min--;  tm->tm_sec+=60; }
  while (tm->tm_min >= 60) { tm->tm_hour++; tm->tm_min-=60; }
  while (tm->tm_min <   0) { tm->tm_hour--; tm->tm_min+=60; }
  while (tm->tm_hour>= 24) { tm->tm_mday++; tm->tm_hour-=24; }
  while (tm->tm_hour<   0) { tm->tm_mday--; tm->tm_hour+=24; }
  while (tm->tm_mon >= 12) { tm->tm_year++; tm->tm_mon-=12; }
  if (tm->tm_year/4==tm->tm_year/4.) days[1]=29;

  while (tm->tm_mday>days[tm->tm_mon])
  {
    tm->tm_mday-=days[tm->tm_mon]; tm->tm_mon++;
    while (tm->tm_mon>= 12)  { tm->tm_year++; tm->tm_mon-=12; }
    if (tm->tm_year/4==tm->tm_year/4.) days[1]=29; else days[1]=28;
  }
  secs=tm->tm_sec+60*(tm->tm_min+60*(tm->tm_hour+24*(tm->tm_mday-1)));
  for (i=0; i<tm->tm_mon; i++) secs+=(days[i]*24*3600);
  rdays=(tm->tm_year-70)*365;
  rdays+=(tm->tm_year-70+1)/4;
  secs+=(rdays*24*3600);
  tm->tm_wday=rdays%7;
  tm->tm_yday=0;
  for (i=0; i<tm->tm_mon; i++) tm->tm_yday+=days[i];
  tm->tm_yday+=(tm->tm_mday-1);

  return secs;
}

int tm2secs(struct tm *tm)
{
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  int secs,rdays;
  int i;
  secs=tm->tm_sec+60*(tm->tm_min+60*(tm->tm_hour+24*(tm->tm_mday-1)));
  for (i=0; i<tm->tm_mon; i++) secs+=(days[i]*24*3600);
  rdays=(tm->tm_year-70)*365;
  rdays+=(tm->tm_year-70+1)/4;
  secs+=(rdays*24*3600);
  return secs;
}



int difftime_tm(struct tm *tm1,struct tm *tm2)
{
  return tm2secs(tm2)-tm2secs(tm1);
}

/*************************************
 * translate month and day in tm struct into days-in-year  
 *************************************/
void mday_mon2yday(struct tm *tmref)
{
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  if (tmref->tm_year/4==tmref->tm_year/4.) days[1]=29;

  tmref->tm_yday=tmref->tm_mday-1;
  if (tmref->tm_mon>=12) tmref->tm_mon=11;
  
  for (i=0; i<tmref->tm_mon; i++)
  {
    tmref->tm_yday+=days[i];
  }

}

/*************************************
 * translate days-in-year in tm struct into month and day
 *************************************/
void yday2mday_mon(struct tm *tmref)
{
  int i;
  int days[]={31,28,31,30,31,30,31,31,30,31,30,31};
  if (tmref->tm_year/4==tmref->tm_year/4.) days[1]=29;

  tmref->tm_mday=tmref->tm_yday+1;
  tmref->tm_mon=0; 
  for (i=0; i<12; i++)
  {
    if (tmref->tm_mday<=days[i]) break;
    tmref->tm_mon++;
    tmref->tm_mday-=days[i];
  }

}

void unexp(char *frmt,...)
{
  va_list arg;

  va_start(arg,frmt);
  fprintf(stderr,"Unexpected: ");
  vfprintf(stderr,frmt,arg);
  fprintf(stderr,"\n");
  va_end(arg);
}

char ftype(char *s)
{
  FILE *fp;
  unsigned char b[4];
  if (is_a_dir(s)) return 'd';
  if (!(fp=fopen(s,"rb"))) return 0;
  if (!(fread(b,2,1,fp))) {  fclose(fp); return 0; }
  fclose(fp);
  if ((b[0]==0x50) && ((b[1]==0x35) || (b[1]==0x36))) return 'p';
  if ((b[0]==0x52) && (b[1]==0x49)) return 'a';
  if ((b[0]==0xff) && (b[1]==0xd8)) return 'j';
  return '?';
}

/*******************************************************
 * Check if file exist and determine overwrite policy
 * mode=0 -> ask to overwrite if file exist
 * mode=1 -> don't ask, overwrite       (REPLACE)
 * mode=2 -> don't ask, don't overwrite (SKIP)
 *******************************************************/ 
int overwrite(char *fn,OVERWRMODE mode)
{
  FILE *fp;
  if (!(fp=fopen(fn,"rb"))) return 1;
  fclose(fp);
  if (mode==OVRWR_REPLACE) return 1;
  if (mode==OVRWR_SKIP) return 0;

#ifndef __NOGUI__
  {
    int n;
    n=Create_Choice("Warning",2,"Overwrite","Don't","File %s exist. Overwrite?",fn);
    if (n==1) return 1;
  }
#endif
  return 0;
}

#ifndef __NOGUI__
// copy path-part of tfn to dir and file-part to fn
static void split_dirfile(char *tfn,char **dir,char **fn)
{
  char *p;
  strcpyd(dir,tfn);
  if ((p=strrchr(*dir,DIR_SEPARATOR)))
  {
    *p=0;
    strcpyd(fn,p+1);
  }
  else
  {
    strcpyd(fn,tfn);
    strcpyd(dir,".");
  }
}
#endif

int overwrite_chng(char *tfn,int tfnlen,OVERWRMODE mode)
{  
#ifndef __NOGUI__
  char *tfno;
  char *dir=NULL,*fn=NULL;
#endif
  if (!(exist_file(tfn))) return 1; // file doesn't exist
  if (mode==OVRWR_REPLACE) return 1;      // file does exist, but may be overwritten
  if (mode==OVRWR_SKIP) return 0;         // skip; don't overwrite
  
#ifndef __NOGUI__
  split_dirfile(tfn,&dir,&fn);      // split path+file into path and file
  tfno=Create_Save(fn,dir);         // dialog: ask to overwrite or not
  freed(&fn);
  freed(&dir);
  if (tfno)
  {
    strncpy(tfn,tfno,tfnlen);
    freed(&tfno);
    return 1;
  }
/*
  {
    int n=2;
    while (n==2)
    {
      n=Create_Choice("Warning",3,"Overwrite","Save as","Abort","File %s exist. Overwrite?",fn);
      if (n==1) return 1;
      if (n==2)
      {
        strcat(fn,"x");
        if (!(exist_file(fn))) return 1;
      }
    }
  }
*/
#endif
  return 0; // file exists; don't overwrite
}
int overwrite_chngd(char **tfn,int tfnlen,OVERWRMODE mode)
{
#ifndef __NOGUI__
  char *tfno;
  char *dir=NULL,*fn=NULL;
#endif
  if (!(exist_file(*tfn))) return 1; // file doesn't exist
  if (mode==OVRWR_REPLACE) return 1;      // file does exist, but may be overwritten
  if (mode==OVRWR_SKIP) return 0;         // skip; don't overwrite
  
#ifndef __NOGUI__
  split_dirfile(*tfn,&dir,&fn);      // split path+file into path and file
  tfno=Create_Save(fn,dir);         // dialog: ask to overwrite or not
  freed(&fn);
  freed(&dir);
  if (tfno)
  {
    strcpyd(tfn,tfno);
    freed(&tfno);
    return 1;
  }
#endif
  return 0; // file exists; don't overwrite
}

/*******************************************************
 * Progress bar for gui 
 *******************************************************/ 
int progress_func(char where,char *text,int prgrscnt,int prgrsmax)
{
  static GtkWidget *progress_bar;

  switch(where)
  {
    case 's':
      progress_bar=Create_Progress(NULL,text,TRUE);
    break;
    case 'S':
      progress_bar=Create_Progress(NULL,text,FALSE);
    break;
    case 'w':
      if (Update_Progress(progress_bar,prgrscnt,prgrsmax))
      {
        if (text) Create_Message("Warning",text);
        return 0;
      }
    break;
    case 'e':
      Close_Progress(progress_bar);
    break;
  }
  return 1;
}

int replace_str(char *l,char *s1,char *s2)
{
  char *p,*p1;
  int n1=strlen(s1);
  int n2=strlen(s2);
  if ((p=strstr(l,s1)))
  {
    if (n1 >= n2)
    {
      strcpy(p+n2,p+n1);
      strncpy(p,s2,n2);
    }
    else
    {
      for (p1=l+strlen(l)+(n2-n1); p1>p; p1--) *p1=*(p1-(n2-n1));
      strncpy(p,s2,n2);
    
    }
    if (strstr(s2,s1)) return 0;
    return 1;
  }
  return 0;
}

// remaining funcs only for gui
#ifndef __NOGUI__
int get_picformat(GtkWidget *widget)
{
  char cformat;
  char *opt=soption_menu_get_history(Find_Widget(widget,LAB_FFT));
  if (!opt) return -1;
  if (!strcmp(opt,LAB_FFTDEF))    cformat=PIC_DEF;
  else if (!strcmp(opt,LAB_PGM))  cformat=PIC_PGM;
  else if (!strcmp(opt,LAB_PGM8)) cformat=PIC_PGM8;
  else if (!strcmp(opt,LAB_JPG))  cformat=PIC_JPEG;
  else if (!strcmp(opt,LAB_CJPG)) cformat=PIC_CJPEG;
  else                            cformat=PIC_DEF;
  return cformat;
}


void get_genmode_from_gui(GtkWidget *widget,GENMODES *genmode)
{
  genmode->gen_pro=Get_Button(widget,LAB_PROEPI);
  genmode->gen_epi=Get_Button(widget,LAB_PROEPI);
//  genmode->europe=Get_Button(widget,LAB_EUROPE); zit nu buiten routine
  genmode->cformat=get_picformat(widget);
}

/* Determine wdir/file; remove . and .. */
static void update_flist(LIST_FILES *lf)
{
  LIST_FILES *lft,*lftnext;
  if (!lf) return;
  for (lft=lf->next; lft; lft=lftnext)
  {
    lftnext=lft->next;
    if (is_a_dir(lft->pfn))                      /* slow, but not that many files here. */
    {
      lft->is_dir=TRUE;
    }
    if ((!strcmp(lft->fn,".")) || (!strcmp(lft->fn,".")))  // leave .. 
    {
      Destroy_Listitem(lft);                      /* No valid XRIT, so remove */
    }
  }
}

void update_flist1(LIST_FILES *lf)
{
  update_flist(lf);
}


void Load_Dirfilelist(GtkCList *dlist,char *path,char dirfilecode,char *filter)
{
  LIST_FILES *list_files=NULL,*lf;
  list_files=create_filelist(path);    /* Generate a list of all files/dirs */
  update_flist(list_files);
  if (dlist)
  {
    char *tmp[3];
    gtk_clist_clear(dlist);
    for (lf=list_files; lf; lf=lf->next)
    {
      if (!lf->fn) continue;
      if ((filter) && (strcmpwild(lf->fn,filter))) continue;
      if ((dirfilecode!='d') && (!lf->is_dir))
      {
        char ts[100];
        strftime(ts,100,"%F %H:%M",localtime(&lf->time));
        tmp[0]=lf->fn;
        tmp[1]=ts;
        tmp[2]=lf->pfn;
        gtk_clist_append(GTK_CLIST(dlist), tmp);
      }
    }
    gtk_clist_set_sort_column(dlist,0);
    gtk_clist_sort(dlist);
    for (lf=list_files; lf; lf=lf->next)
    {
      if ((dirfilecode!='f') && (lf->is_dir))
      {
        GdkColor clr_dir={0,0,0,0xffff};  /* color dir */
        int row;
//        char nn[9]; int n=1;
        char ts[100];
        strftime(ts,100,"%F %H:%M",gmtime(&lf->time));
//        if (lf->is_dir) n=count_dircontent(lf->pfn);
//        sprintf(nn,"%d",n);
        tmp[0]=lf->fn;
        tmp[1]=ts; // nn;
        tmp[2]=lf->pfn;
        row=gtk_clist_append(GTK_CLIST(dlist), tmp);
        gtk_clist_set_foreground(dlist,row,&clr_dir);
      }
    }
    
  }
  Destroy_List(list_files,TRUE);  // destroy list; it's now in clist
  gtk_clist_set_sort_column(dlist,0);
  gtk_clist_sort(dlist);
}
#define LAB_THISDIR "^Archive dir"
#define LAB_STDDIR "Use Standard path"
#define LAB_DOARCH "Archive"
#define LAB_CANCARCH "Cancel"
static char *rrstr=NULL;
static char rstr[100];

static void close_entrychoice(GtkWidget *widget)
{
  destroy_window(widget);
  Close_Window(widget);
  gtk_main_quit();
}


static void efunc(GtkWidget *widget,gpointer data)
{
  char *name=(char *)data;
  GtkWidget *window=Find_Parent_Window(widget);
  if (!strcmp(name,LAB_THISDIR))
  {
  }

  if (!strcmp(name,LAB_STDDIR))
  {
    Sense_Button(widget,LAB_THISDIR,!Get_Button(widget,LAB_STDDIR));
  }

  if (!strcmp(name,LAB_DOARCH))
  {
    if (Get_Button(widget,LAB_STDDIR))
    {
      *rstr=0;
    }
    else
    {
      strncpy(rstr,Get_Entry(widget,LAB_THISDIR),99);
    }
    rrstr=rstr;
    Close_Window(window);
  }
  if (!strcmp(name,LAB_CANCARCH))
  {
    rrstr=NULL;
    close_entrychoice(window);
  }
}

/* Create window with text entry and ask for choice */
char *Create_Entrychoice(char *title,char *str)
{
  GtkWidget *wnd;
  GtkWidget *w[6];
  *rstr=0;
  wnd=Create_Window(NULL,0,0,title,close_entrychoice);

  w[1]=gtk_label_new(str);
  w[2]=Create_Entry(LAB_THISDIR,efunc,"%-10s","");
  w[3]=Create_Check(LAB_STDDIR,efunc,FALSE);
  w[4]=Create_Button(LAB_DOARCH,efunc);
  w[5]=Create_Button(LAB_CANCARCH,efunc);
  w[2]=Pack(NULL,'v',w[2],1,w[3],1,NULL);
  w[4]=SPack(NULL,"h",w[4],"1",w[5],"E1",NULL);

  w[0]=Pack(NULL,'v',w[1],1,w[2],1,w[4],1,NULL);
  gtk_container_add(GTK_CONTAINER(wnd),w[0]);
  gtk_widget_show_all(wnd);
  gtk_main();
  return rrstr;
}

#endif

#if __GTK_WIN32__ == 0
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE  /* for kill() */
#endif
#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>    /* for fork, exec, kill */
#include <sys/types.h> /* for pid_t            */
#include <sys/wait.h>  /* for waitpid          */
#include <signal.h>    /* for SIGTERM, SIGKILL */

/*
 * fork a child process, execute vlc, and return it's pid.
 * returns -1 if fork failed.
 */
#define MAXNRARGS 10
pid_t spawn_prog(char *cmd1)
{
  static char *argvs[MAXNRARGS+1]; // argvs MUST be static!
  char *p,*q;
  char cmd[MAXLENPREF];
  int argc=0,i;
  pid_t pid = fork();
  if (pid == -1)
  {
    perror("fork");
    return -1;
  }
  strncpy(cmd,cmd1,MAXLENPREF);
  while ((p=strstr(cmd,"\\ "))) *(p+1)='@';
  p=strtok(cmd," \n");
  if (p) do
  {
    if (argc>=MAXNRARGS) break;
    argvs[argc++]=p;
  } while ((p=strtok(NULL," \n")));
  argvs[argc]=NULL;

  while ((p=strstr(cmd,"\\@"))) *(p+1)=' ';
  for (i=0; i<argc; i++)
  {
    while ((p=strstr(argvs[i],"\\@")))
    {
      *(p+1)=' ';
      for (q=p; *q; q++) *q=*(q+1);
    }
  }

  /* when you call fork(), it creates two copies of your program:
   * a parent, and a child. you can tell them apart by the return
   * value from fork().  If fork() returns 0, this is is the child
   * process.  If fork() returns non-zero, we are the parent and the
   * return value is the PID of the child process. */
  if (pid == 0)
  {
    /* this is the child process.  now we can call one of the exec
     * family of functions to execute VLC.  when you call exec,
     * it replaces the currently running process (the child process)
     * with whatever we pass to exec.  So our child process will now
     * be running VLC.  exec() will never return except in an error
     * case, since it is now running the VLC code and not our code. */
    execvp(argvs[0], argvs);
    return -1; // Normally never get here!
  }
  else
  {
    /* parent, return the child's PID back to main. */
    return pid;
  }
}

int stop_prog(pid_t pid)
{
  int status = 0;
  /* kill will send the specified signal to the specified process.
   * in this case, we send a TERM signal to VLC, requesting that it
   * terminate.  If that doesn't work, we send a KILL signal.
   * If that doesn't work, we give up. */
  if (kill(pid, SIGTERM) < 0)
  {
//    perror("kill with SIGTERM");
    if (kill(pid, SIGKILL) < 0)
    {
//      perror("kill with SIGKILL");
    }
  }
  waitpid(pid, &status, 0);
  return status;
}
#endif

