/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/

#include "xrit2pic.h"
#include "xrit2pic_defines.h"
#include "avhrr.h"
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#ifdef __NOGUI__
  #include "xrit2pic_nogtk.h"
#endif
extern PREFER prefer;
extern GLOBINFO globinfo;
extern XRIT_DBASE dbase;

/**********************************
 * Copy channel info from xrit-dbase to channel
 **********************************/
static void add_chaninfo(CHANNEL *chan,XRIT_HDR *xh)
{
  chan->wavel=xh->wavel;
  chan->chan_nr=xh->chan_nr;
  chan->seq_start=xh->seq_start;
  chan->seq_end=xh->seq_end;
  chan->nb=xh->nb;
  chan->nc=xh->nc;            // may change with aspect
  chan->nl=xh->nl;            // may change with aspect
  chan->nc_orig=xh->nc_orig;  // not changed with aspect
  chan->nl_orig=xh->nl_orig;  // not changed with aspect
  chan->ifrmt=xh->image_iformat;
  chan->scan_dir=xh->scan_dir;
  chan->lines_per_sec=xh->lines_per_sec;

// NOG TE BEKIJKEN: ncext ergens anders bepalen (vlak voor generatie)
  if ((chan->chan_name) && (!strcmp(chan->chan_name,"HRV")))
  {
//    chan->x_shift=HRV_XSHIFT;
//    chan->shift_ypos=HRV_SHIFT_YPOS;
    chan->ncext=11136; // chan->nc*2;
  }
  else
  {
//    chan->x_shift=0;
//    chan->shift_ypos=0;
    chan->ncext=chan->nc;
  }
}

void copy_segminfo(SEGMENT *segm1)
{
  SEGMENT *segm;
  for (segm=segm1->next; segm; segm=segm->next)
  {
    segm->xh.seq_start=segm1->xh.seq_start;
    segm->xh.seq_end=segm1->xh.seq_end;
    segm->xh.nl=segm1->xh.nl;
  }
}

/*
3.2.6, page 111:

Note 2: The relation between the binary pixel value (the pixel count) and the 
physical radiance units (expressed in:
    -2  -1    -1 -1
 mWm  sr   (cm  )
 
 
 ) is fully defined for each spectral band by the relation:

                 Physical Units = Cal_Offset + (Cal_Slope . Pixel Count)

More details can be found in the Level1_5ImageCalibration Record.

*/

/*
Related doc: L_15_Data_Desc.pdf: MSG Level 1.5 Image Data Format Description
Types: See pdf_ten_99055_spe_msg_gsds.pdf:
 INTEGER: 4 bytes
 REAL: 4 bytes
 REAL DOUBLE: 8 bytes
*/
static void get_pro(FILE *fp,SEGMENT *pro)
{
  long lng;
  char byte;
  long pro_start;
  int i;

  union
  {
    double dbl;
    gint64 int64;
  } dbl_ch;
  union
  {
    float f;
    long l;
  } flt_ch;
  pro_start=ftell(fp);
  if (pro->chan->group->h_or_l=='H')
  {
/* ProjectionDescription starts at 5e74c (see page 177), length: 5 bytes */
    fseek(fp,pro_start,SEEK_SET);                 // offset=0x5a=90
    fseek(fp,0x5e74c,SEEK_CUR);                   // Start projection descr.
    if (!fread(&byte,1,1,fp)) return;             // TypeOfProjection 1=earth centerd in grid
    if (!fread(&flt_ch.l,1,4,fp)) return;         // LongitudeOfSSP (also in segment header type 2)
    flt_ch.l=GINT32_FROM_BE(flt_ch.l);            // nominal sub-satellite point
    pro->ProjectionDescription.LongitudeOfSSP=flt_ch.f; // subsat position (after correction)

/* ReferenceGridVIS_IR starts at 5e751 (see page 177), length: 17 bytes */
    fseek(fp,pro_start,SEEK_SET);                 // offset=0x5a=90
    fseek(fp,0x5e751,SEEK_CUR);                   // Start ReferenceGridVIS_IR
    if (!fread(&lng,1,4,fp)) return;              // NumberOfLines
//    pro->NumberOfLines=GINT32_FROM_BE(lng);  
    if (!fread(&lng,1,4,fp)) return;              // NumberOfColumns
    if (!fread(&flt_ch.l,1,4,fp)) return;         // LineDirGridStep
    if (!fread(&flt_ch.l,1,4,fp)) return;         // ColumnDirGridStep
    if (!fread(&byte,1,1,fp)) return;             // GridOrigin: 0=NW, 1=SW, 2=SE, 3=NE

/* ReferenceGridHRV  starts at 5e762 (see page 177), length: 17 bytes */
    fseek(fp,pro_start,SEEK_SET);                 // offset=0x5a=90
    fseek(fp,0x5e762,SEEK_CUR);                   // Start ReferenceGridHRV

/* PlannedCoverageVIS_IR starts at 5e773 (see page 177), length: 16 bytes */
    fseek(fp,pro_start,SEEK_SET);                 // offset=0x5a=90
    fseek(fp,0x5e773,SEEK_CUR);                   // Start PlannedCoverageVIS_IR

/* PlannedCoverageHRV starts at 5e783 (see page 178), length: 64 bytes */
    fseek(fp,pro_start,SEEK_SET);                 // offset=0x5a=90

    fseek(fp,0x5e783,SEEK_CUR);                   // Start PlannedCoverageHRV
    if (feof(fp)) return;

    if (!fread(&lng,1,4,fp)) return;              // LowerSouthLinePlanned
    if (!fread(&lng,1,4,fp)) return;              // LowerNorthLinePlanned
    pro->lower_nord_row=GINT32_FROM_BE(lng);      // huh??? -2;
    if (!fread(&lng,1,4,fp)) return;              // LowerEastColumnPlanned
    pro->lower_east_col=GINT32_FROM_BE(lng); 
    if (!fread(&lng,1,4,fp)) return;              // LowerWestColumnPlanned
    if (!fread(&lng,1,4,fp)) return;              // UpperSouthLinePlanned
    if (!fread(&lng,1,4,fp)) return;              // UpperNorthLinePlanned
    if (!fread(&lng,1,4,fp)) return;              // UpperEastColumnPlanned
    pro->upper_east_col=GINT32_FROM_BE(lng);
    if (!fread(&lng,1,4,fp)) return;              // UpperWestColumnPlanned

/* Level1_5ImageCalibration starts at 5e853= 0x5e7f9+0x5a*/
    fseek(fp,pro_start,SEEK_SET);
    fseek(fp,0x5e7f9,SEEK_CUR);                   // start Level1_5ImageCalibration
    for (i=0; i<12; i++)
    {
      if (!fread(&dbl_ch.int64,8,1,fp)) return;   // Cal_Slope
      dbl_ch.int64=GUINT64_FROM_BE(dbl_ch.int64);
      pro->chan->Cal_Slope[i]=dbl_ch.dbl;
      if (!fread(&dbl_ch.int64,8,1,fp)) return;   // Cal_Offset
      dbl_ch.int64=GUINT64_FROM_BE(dbl_ch.int64);
      pro->chan->Cal_Offset[i]=dbl_ch.dbl;
    }
  }

/* For LRIT: slope *4 to compensate 8 bpp ipv 10bpp */
  // !! PROBABLY NOT OK with nogui (self-made int64 etc., check!)
  if (pro->chan->group->h_or_l=='L')
  {
    fseek(fp,pro_start,SEEK_SET);
    fseek(fp,0x5e7f9,SEEK_CUR);                   // start Level1_5ImageCalibration
    for (i=0; i<12; i++)
    {
      if (!fread(&dbl_ch.int64,8,1,fp)) return;   // Cal_Slope
      dbl_ch.int64=GUINT64_FROM_BE(dbl_ch.int64);
      pro->chan->Cal_Slope[i]=dbl_ch.dbl*4.;

      if (!fread(&dbl_ch.int64,8,1,fp)) return;   // Cal_Offset
      dbl_ch.int64=GUINT64_FROM_BE(dbl_ch.int64);
      pro->chan->Cal_Offset[i]=dbl_ch.dbl;
    }
  }  
}

FILE *check_pro(FILE *fp,SEGMENT *pro)
{
  long pro_start=ftell(fp);
  int i;
  for (i=0; i<3; i++)
  {
    char byte;
    fseek(fp,0x5e7f9 + 12*16,SEEK_CUR);           // start Level1_5ImageCalibration
    if (!fread(&byte,1,1,fp))
    {
      fseek(fp,0 ,SEEK_END);
      if (i==0) w_dbg("%s:\n",pro->pfn);
      w_dbg("ERR pro %d, #bytes=%d\n",i,ftell(fp));
      fclose(fp);
      if (i==2)
      {
        w_dbg("Mislukt\n");
        return NULL;
      }
      fp=open_xritimage(pro->pfn,NULL,UNKDTYPE,NULL);
      if (!fp) return NULL;
      pro_start=ftell(fp);
    }
    else break;
  }
  fseek(fp,pro_start,SEEK_SET);
  return fp;
}

// Soms wordt prologue niet goed gelezen.
// Lengte file lijkt dan 125000, 25000, 375000 te zijn ipv > 400000.
// Sluiten en opnieuw openen helpt niet.
// xrit2pic opnieuw opstarten wel. ??????????????
// -> check bij elke read, breek af als fout.
// Mogelijke problemen: m.n. bij HRV noord/zuidposities!
static void read_pro(SEGMENT *pro)
{
  FILE *fp;
  if (!pro) return;
  if (!pro->chan) return;
  if (!pro->chan->group) return;
//  if (pro->chan->group->h_or_l!='H') return;  // for LRIT: different pro?
  if (!(fp=open_xritimage(pro->pfn,NULL,EXRIT,NULL))) return;
//  if (!(fp=check_pro(fp,pro))) return;
  get_pro(fp,pro);
  fclose(fp);
}

#ifdef XXX
void xxx(GROUP *grp)
{
  CHANNEL *chan;
  for (; grp; grp=grp->next)
    for (chan=grp->chan; chan; chan=chan->next)
       if (chan->Cal_Offset[0]>5)  printf(">>xxx>>! %f\n",chan->Cal_Offset[0]);
}
void yyy(GROUP *grp)
{
  CHANNEL *chan;
    for (chan=grp->chan; chan; chan=chan->next)
       if (chan->Cal_Offset[0]>5)  printf(">>yyy>>! %f\n",chan->Cal_Offset[0]);
}
#endif


// If pfn is a bzipped file: unzip; return output file name
// (which is input filename without .bz2 extension)
// Note that in case of success the zipped file is removed.  
static char *bunzip2_replace(char *pfn)
{
  char *fno=NULL,*p;
  if (!str_ends_with(pfn,".bz2")) return NULL; // not zipped

  // It's zipped  
  strcpyd(&fno,pfn);
  // Remove most right extension, expected '.bz2'
  if ((p=strrchr(fno,'.')) && ((p=strstr(p,".bz2")))) *p=0;

  // unzip, if failed: free fno (expected no file generated?)
  if ((bunzip2(pfn,fno))) freed(&fno);

  return fno;  // NULL if bunzip2 failed
}

void unzip_segmentfile(SEGMENT *segm)
{
  char *fn;
  if (!segm) return;
  if ((fn=bunzip2_replace(segm->pfn)))
  {
    remove(segm->pfn);
    freed(&segm->pfn);
    segm->pfn=fn;
    if ((fn=strrchr(segm->fn,'.'))) *fn=0;
    if (segm->orig_segm)
    {
      if (str_ends_with(segm->orig_segm->pfn,".bz2"))
      {
        if ((fn=strrchr(segm->orig_segm->pfn,'.'))) *fn=0;
      }
      if (str_ends_with(segm->orig_segm->fn,".bz2"))
      {
        if ((fn=strrchr(segm->orig_segm->fn,'.'))) *fn=0;
      }
    }
  }
}

#ifdef NOTUSED_NOW
static void unzip_sattype(GROUP *grp,DATATYPE type,int progress_func())
{
  GROUP *g;
  SEGMENT *s;
  static int prgrsmax;
  int prgrscnt=0;

  if (!progress_func) prgrsmax=0;
  if (progress_func) progress_func('s',"Unzipping...",0,prgrsmax);
  for (g=grp; g; g=g->next)
  {
    if (!g->chan) continue;
    if (!g->chan->segm) continue; prgrsmax++;
    if (g->chan->segm->xh.xrit_frmt!=type) continue;

    for (s=g->chan->segm; s; s=s->next)
    {
      if (!strstr(s->pfn,".bz2")) continue;
      if (!progress_func) prgrsmax++; 
      if (progress_func)
      {
        if (!(progress_func('w',NULL,++prgrscnt,prgrsmax)))
        {
          break;
        }
        unzip_segmentfile(s);
      }
    }
  }
  if (progress_func) progress_func('e',NULL,0,prgrsmax);
}
#endif

// Do NOT copy: 
//   hl,vers,sat,src,satsrc,chan,special,segment,
//   itime,time,compr,encry,chan_nr,sortn,id,wavel,subsat_lon,anno
#define XXX
static void copy_xrithdr_stbl(XRIT_HDR *xh1,XRIT_HDR *xh2)
{
  xh2->hdr_type=xh1->hdr_type;
  xh2->hdr_rec_len=xh1->hdr_rec_len;
  xh2->file_type=xh1->file_type;
  xh2->hdr_len=xh1->hdr_len;
  xh2->data_len=xh1->data_len;
  xh2->datalen_msb=xh1->datalen_msb;
  xh2->datalen_lsb=xh1->datalen_lsb;
//printf("%d  %d  %d  %d\n",xh1->nb,xh1->nc,xh1->nl,xh1->cf);
  xh2->nb=xh1->nb;
  xh2->nc=xh1->nc;
  xh2->nl=xh1->nl;
  xh2->cf=xh1->cf;
  xh2->image_iformat=xh1->image_iformat;
  xh2->image_oformat=xh1->image_oformat;
  strcpy(xh2->proj_name,xh1->proj_name);
  xh2->cfac=xh1->cfac;
  xh2->lfac=xh1->lfac;
  xh2->coff=xh1->coff;
  xh2->loff=xh1->loff;
  xh2->subsat_lon=xh1->subsat_lon;
  xh2->xrit_frmt=xh1->xrit_frmt;
  xh2->scan_dir=xh1->scan_dir;
  xh2->lines_per_sec=xh1->lines_per_sec;
  xh2->etime=xh1->etime;
  xh2->tlen=xh1->tlen;

  memcpy(xh2->ccsds,xh1->ccsds,7);
  xh2->gp_sc_id=xh1->gp_sc_id;
  xh2->spec_ch_id=xh1->spec_ch_id;
  xh2->seq_no=xh1->seq_no;
  xh2->seq_start=xh1->seq_start;
  xh2->seq_end=xh1->seq_end;
  xh2->dt_f_rep=xh1->dt_f_rep;
  strcpy(xh2->mdm_pos_code,xh1->mdm_pos_code);
  strcpyd(&xh2->img_obs_time,xh1->img_obs_time);
  xh2->pic_id=xh1->pic_id;
}

// detect if same type, i.e., includes same size pic! 
static int same_type(GROUP *grp1,GROUP *grp2)
{
  if (!grp1->sat_src) return 0;
  if (!grp2->sat_src) return 0;
  if (strcmp(grp1->sat_src,grp2->sat_src)) return 0;
  if (grp1->h_or_l!=grp2->h_or_l) return 0;
  if (strcmp(grp1->region,grp2->region)) return 0;
  return 1;
}

// try to get info from previous segment-files
static int get_hdrinfo_prevfile(CHANNEL *chan,GROUP *grp)
{
  GROUP *g;
  CHANNEL *ch;
  SEGMENT *segm;
  if (!grp->sat_src) return 0;
  if (!strcmp(chan->chan_name,"HRV")) return 0;
  if (!chan->segm) return 0;
  if (!chan->segm->xh.segment) return 0;

  for (g=grp->prev; g; g=g->prev)
  {
    if (!g->sat_src) continue;
    if (!same_type(grp,g)) continue;
    for (ch=g->chan; ch; ch=ch->next)
    {
      for (segm=ch->segm; segm; segm=segm->next)
      {
        if (segm->xh.segment > 0) break;
      }
      if (!segm) continue;
      if ((chan->chan_name) && (ch->chan_name))
      {
        if ((!strcmp(chan->chan_name,ch->chan_name)) && 
            (chan->segm->xh.segment==segm->xh.segment)) break;
      }
    }
    if (!ch) continue;
    if (!ch->segm) continue;
    copy_xrithdr_stbl(&ch->segm->xh,&chan->segm->xh);
    return 1;
  }
  return 0;
}

#define same_group1(xh1,xh2) \
  ((xh1.pic_id == xh2.pic_id) && \
   (!strcmp(xh1.satsrc,xh2.satsrc)) && \
   (xh1.hl == xh2.hl))    

#define same_group(grp,xh2) \
  ((grp->pic_id == xh2.pic_id) && \
   (!strcmp(grp->sat_src,xh2.satsrc)) && \
   (grp->h_or_l == xh2.hl))    

#define same_channel(xh1,xh2) \
  ((*xh1.chan) && (!strcmp(xh2.chan,xh1.chan)))

// create group/chan/segm database from list.
// If done in pieces ('live' recording) then lateron related parts will be combined.
// See load_datafiles.c, 'update_group()'.
static GROUP *listfl2grp(LIST_FILES *lf,LOCATION_SEGM where,int progress_func())
{
  GROUP *grp=NULL;
  LIST_FILES *lft;
//  LIST_FILES *lft2;
  gboolean abort=FALSE;
  int prgrsmax=0;
  int prgrscnt=0;
  for (lft=lf; lft; lft=lft->next) prgrsmax++;           /* Determine # files */
  if (progress_func) progress_func('s',"Sorting...",0,prgrsmax);

  for (lft=lf->next; lft; lft=lft->next)
  {
    GROUP *cur_grp=NULL;
    CHANNEL *cur_chan=NULL;
    SEGMENT *segm;
    if (!lft->fn) { puts("???"); continue;}
    if (lft->is_dir) continue;
    if (lft->unknown)  // not recognized by fn2xrit()
    {
      LIST_FILES *unk=NULL;
      unk=Create_List(dbase.unknown,NULL);
      strcpyd(&unk->fn,lft->fn);    
      strcpyd(&unk->pfn,lft->pfn);    
      if (!dbase.unknown) dbase.unknown=unk;
      continue;
    }
#define NIEUW
#ifdef NIEUW
    for (cur_grp=grp; cur_grp; cur_grp=cur_grp->next)
    {
      if (same_group(cur_grp,lft->pxh)) /* Same time and same xrit-type: belong to same group */
      {  // Found group
        /* Stop if also same channel name found.
           This is the case if:
           . there was a previous segment belonging to same channel
           . there was a previous segment of pro/epi belonging to same channel (GOES etc.)
           A group pro/epi, not belonging to a certain channel (MSG) is not recognized here.
        */
        for (cur_chan=cur_grp->chan; cur_chan; cur_chan=cur_chan->next)
        {
          if (!strcmpd(cur_chan->chan_name,lft->pxh.chan))
            break;
        }
        break;
      }
    }
#else
    for (lft2=lft->prev; lft2; lft2=lft2->prev) /* search back to find corresponding pieces */
    {
      if ((lft2->unknown) || (lft2->is_dir)) continue;
      /* MSG: pic_id=yyy ddddddddd hh mm; NOAA: yyy ddddddddd iiii */

      if (same_group1(lft2->pxh,lft->pxh)) /* Same time and same xrit-type: belong to same group */
      {
        // Found group
        cur_grp=lft2->grp;

        /* Stop if also same channel name found.
           This is the case if:
           . there was a previous segment belonging to same channel
           . there was a previous segment of pro/epi belonging to same channel (GOES etc.)
           A group pro/epi, not belonging to a certain channel (MSG) is not recognized here.
        */
        if (same_channel(lft->pxh,lft2->pxh))
        {
          // Found channel; ready 
          cur_chan=lft2->chan;
          break;
        }
      }
    }
#endif

    segm=NULL;
    if (!cur_grp)                                /* first occurrence this group */
    {
      cur_grp=Create_Grp(&grp);
      cur_grp->h_or_l=lft->pxh.hl;
      cur_grp->pic_id=lft->pxh.pic_id;
      cur_grp->datatype=lft->pxh.xrit_frmt;
      switch(cur_grp->h_or_l)
      {
        case 'A': case 'M': case 'G': cur_grp->orbittype=POLAR;     break;
        case 'h': case 'H':           cur_grp->orbittype=GEO;       break;
        case 'l': case 'L':           cur_grp->orbittype=GEO;       break;
        case 'J':                     cur_grp->orbittype=GEO;       break;
        default:                      cur_grp->orbittype=UNK_ORBIT; break;
      }
      strcpy(cur_grp->sat_src,lft->pxh.satsrc);
      strcpy(cur_grp->region,lft->pxh.region);
      strcpy(cur_grp->grp_time,lft->pxh.itime);
      cur_grp->grp_tm=lft->pxh.time;
      cur_chan=NULL;                           /* New group, so new channels */
    }

    if (cur_grp)                                /* always; corresponding group found or new group */
    {
      if (!cur_chan)                            /* corresponding channel not found (never if group pro/epi?) */
      {
        switch (lft->pxh.special)
        {
          case 'P': case 'E':
            cur_chan=cur_grp->pro_epi;
            if (!cur_chan)
            {
              cur_chan=Create_Chan(&cur_grp->pro_epi);  /* create special entry */
              add_chaninfo(cur_chan,&lft->pxh);
              cur_chan->group=cur_grp;
            }
          break;
          default:
            cur_chan=Create_Chan(&cur_grp->chan);      /* create normal chan entry */
            cur_chan->group=cur_grp;
            strcpyd(&cur_chan->chan_name,lft->pxh.chan);
            strcpyd(&cur_chan->data_src,lft->pxh.src);
          break;
        }
      }

      if (cur_chan)                             /* always */
      {
        switch (lft->pxh.special)
        {
          case 'P': case 'p':                           /* group or channel pro */
            segm=Create_Segm(&cur_chan->pro);
            segm->is_pro=TRUE;
            segm->chan=cur_chan;
          break;
          case 'E': case 'e':                           /* group or channel epi */
            segm=Create_Segm(&cur_chan->epi);
            segm->is_epi=TRUE;
            segm->chan=cur_chan;
          break;
          default:
            cur_chan->chan_nr=lft->pxh.chan_nr;  // maar dit is al nodig voor sorteren!
            cur_chan->nr_segm++;
            segm=Create_Segm(&cur_chan->segm);          /* add first segment to new channel */
            segm->chan=cur_chan;
          break;
        }
      }
    }

    if (segm)
    {
      segm->xh=lft->pxh;            // pxh doesn't contain dynamic allocated parts!
      segm->time=lft->time;         // creation time
      strcpyd(&segm->pfn,lft->pfn);
      strcpyd(&segm->fn,lft->fn);
      segm->where=where;
      if (segm->is_pro) read_pro(segm);
    }
    lft->grp=cur_grp;
    lft->chan=cur_chan;

    if (progress_func)
    {
      if (!(progress_func('w',NULL,++prgrscnt,prgrsmax)))
      {
        abort=TRUE;
        break;
      }
    }
  }
  /* Time-sort dbase */
  if (!abort) grp=Sort_Grp(grp);
  if (progress_func) progress_func('e',NULL,0,prgrsmax);
  return grp;
}

static void add_channel_info(CHANNEL *chan1,int  *pic_nr)
{
  int nr_chans=0;
  GROUP *grp1=chan1->group;
  /* read xrit-header to complete info. Only do this for first segm. */
  if (!chan1->segm) return;
  // Add chaninfo extracted from filename
 // add_chaninfo(chan1,&chan1->segm->xh);
  // check if another equivalent segment exists. ?? gaat FOUT als vorige niet compleet was!??
  if ((globinfo.safemode) || (!get_hdrinfo_prevfile(chan1,grp1)))
  {
    FPFNTMP fpfntmp;
  // unzip first
    if (prefer.keep_unzipped)
      unzip_segmentfile(chan1->segm);

    // open file of first segment
    fpfntmp=open_xritimage_bz(chan1->segm->pfn,&chan1->segm->xh,chan1->segm->xh.xrit_frmt,NULL);
    if (!fpfntmp.fp) return;
    rmclose(&fpfntmp);
  }
  // Add chaninfo extracted from header (complete info, if possible) 
  add_chaninfo(chan1,&chan1->segm->xh);

  if ((grp1->pro_epi) && (grp1->pro_epi->pro))
  {
    chan1->Cal_Slope[0]=grp1->pro_epi->pro->chan->Cal_Slope[chan1->chan_nr-1];
    chan1->Cal_Offset[0]=grp1->pro_epi->pro->chan->Cal_Offset[chan1->chan_nr-1];
  }

  copy_segminfo(chan1->segm);
  chan1->pic_nr=(++(*pic_nr));
  if ((grp1->pro_epi) && (!strcmp(chan1->chan_name,"HRV")))
  {
//        chan1->x_shift=grp1->pro_epi->x_shift;
//        chan1->shift_ypos=grp1->pro_epi->shift_ypos;
  }
  if (nr_chans<MAXCHANSTR-1)
  {
    if (chan1->chan_name)
    {
      if (isalpha(chan1->chan_name[0]))
      {
        grp1->chanstr[nr_chans]=chan1->chan_name[0];
      }
      else
      {            
        if (chan1->wavel<10)
          grp1->chanstr[nr_chans]='V';
        else if ((chan1->wavel>60) && (chan1->wavel<75))
          grp1->chanstr[nr_chans]='W';
        else
          grp1->chanstr[nr_chans]='I';
      }
    }
    else
      grp1->chanstr[nr_chans]='?';
    nr_chans++;
    grp1->chanstr[nr_chans]=0;
  }
/* Note: max string length=30 (see MAXLEN_ID) */
  sprintf(chan1->id,"%s    %c%s",chan1->chan_name,grp1->h_or_l,grp1->grp_time);
}

static void add_avhrr_channels(GROUP *grp1)
{
  int i;
  strcpy(grp1->chanstr,"12345");
  grp1->chan->chan_nr=1;
  for (i=2; i<=5; i++)
  {
    CHANNEL *new_chan=NULL,*ch;
    SEGMENT *segm;
    new_chan=Copy_Chan(grp1->chan);
    new_chan->group=grp1;
   /* Add new_chan at end of chan-list */
    for (ch=grp1->chan; ch; ch=ch->next) if (!ch->next) break;
    ch->next=new_chan; new_chan->prev=ch;

    /* Define id and channel name */
    new_chan->id[2]=i+'0';        // correct id needed for translate wnd
    if (new_chan->chan_name)
    {
      switch(i)
      {
        case 1: sprintf(new_chan->chan_name,A_CH1); break;
        case 2: sprintf(new_chan->chan_name,A_CH2); break;
        case 3: sprintf(new_chan->chan_name,A_CH3); break;
        case 4: sprintf(new_chan->chan_name,A_CH4); break;
        case 5: sprintf(new_chan->chan_name,A_CH5); break;
      }
    }
    new_chan->chan_nr=i;
    for (segm=new_chan->segm; segm; segm=segm->next)
    { /* These segms use file connected to segm of first channel! */
      segm->where=no_file;
    }
  }
}

/*******************************************************
 * Scan all images and create linked list 
 *******************************************************/ 
GROUP *mk_dbase(LIST_FILES *lf,GENMODES *gmode,
                LOCATION_SEGM where,
                int progress_func())
{
//  LIST_FILES *lft; // ,*lft2;
  GROUP *grp=NULL,*grp1;
  CHANNEL *chan1;
  int pic_nr=0;
  int prgrscnt=0,prgrsmax=0;

  if (!lf) return NULL;
#ifndef __NOGUI__
  while (g_main_iteration(FALSE));
#endif

/* Create dbase with GROUP, CHANNEL and SEGMENT */
  grp=listfl2grp(lf,where,progress_func);

/* Unzip METOP at (re-)load */
//  if (globinfo.keep_unzipped)
//  {
//    unzip_sattype(grp,METOP,NULL);
//    unzip_sattype(grp,METOP,progress_func);
//  }


/* Complete xrit header-info by reading header of first file of each channel */
  prgrsmax=0;
  for (grp1=grp; grp1; grp1=grp1->next) prgrsmax++;
  prgrscnt=0;
  if (progress_func) progress_func('s',"Updating...",0,prgrsmax);

  for (grp1=grp; grp1; grp1=grp1->next)
  {
    if (!globinfo.gui_mode)
    {
      if ((gmode) && (gmode->sat) && (*gmode->sat) && strcmpwild(grp1->sat_src,gmode->sat))
      {
        continue;
      }
      // if nogui and grp not selected: skip here (te doen) of bij loadlist er al uithalen?
    }
    if (progress_func)
    {
      if (!(progress_func('w',NULL,++prgrscnt,prgrsmax)))
      {
        break;
      }
    }

/* make unique id. for grouping. ! Should be the same as in mkdbase:
      MSG: pic_id=yyy ddddddddd hh mm; NOAA: yyy ddddddddd iiii 
----------------------
      if ((lft2->pxh.pic_id==lft->pxh.pic_id) &&
          (!strcmp(lft2->pxh.satsrc,lft->pxh.satsrc)) &&
          (lft2->pxh.hl==lft->pxh.hl)) 
----------------------
grp1->grp_time differs from lft->pxh.pic_id???
Rest same.
???if (grp1->chan->xh) printf("%lx  %x\n",grp1->chan->xh.pic_id,grp1->grp_time);
*/
    sprintf(grp1->id,"%c%s%s",grp1->h_or_l,grp1->grp_time,grp1->sat_src);



    for (chan1=grp1->chan; chan1; chan1=chan1->next)
    {
      add_channel_info(chan1,&pic_nr);
    }
  }

/* AVHRR (NOAA, METOP):
     determine ID
     add 4 more channels
*/
  prgrscnt=0;
  for (grp1=grp; grp1; grp1=grp1->next)
  {
    if (progress_func)
    {
      if (!(progress_func('w',NULL,++prgrscnt,prgrsmax)))
      {
        break;
      }
    }
    if ((grp1->h_or_l) && (strchr("GAM",grp1->h_or_l)))          /* AVHRR or GAC */
    {
      add_avhrr_channels(grp1);
    }
  }

  if (progress_func) progress_func('e',NULL,0,prgrsmax);
#ifndef __NOGUI__
  while (g_main_iteration(FALSE));
#endif

/* make donedir if not exist, and if not in donedir itself */
  if ((where == in_received) && (!globinfo.donedir_exist))
    globinfo.donedir_tomake=TRUE;
  else
    globinfo.donedir_tomake=FALSE;

  return grp;
}

/* Search 'node' in list */
int mark_selected(GROUP *group,GtkCTreeNode *node,gboolean selected)
{
  GROUP *grp;
  CHANNEL *chan;
  SEGMENT *segm;
  int iformat=0;
  if (!node) return 0;

  for (grp=group; grp; grp=grp->next)
  {
    if (grp->node==node) grp->nselected=selected;

    for (chan=grp->chan; chan; chan=chan->next)
    {
      if (chan->node==node) chan->nselected=selected;

      for (segm=chan->segm; segm; segm=segm->next)
      {
        if (segm->node==node) segm->nselected=selected;

/* Check if one of selected pics is JPEG, to enable PGM/JPEG generation choice */
        if ((grp->nselected) || (chan->nselected) || (segm->nselected))
        {
          if (segm->xh.image_iformat)
          {
            if (iformat!='j') iformat=segm->xh.image_iformat; 
          }
        }
      }

      for (segm=chan->pro; segm; segm=segm->next)
      {
        if (segm->node==node) segm->nselected=selected;
      }

      for (segm=chan->epi; segm; segm=segm->next)
      {
        if (segm->node==node) segm->nselected=selected;
      }
    }

    for (chan=grp->pro_epi; chan; chan=chan->next)
    {
      if (chan->node==node) chan->nselected=selected;

      for (segm=chan->pro; segm; segm=segm->next)
      {
        if (segm->node==node) segm->nselected=selected;
      }
      for (segm=chan->epi; segm; segm=segm->next)
      {
        if (segm->node==node) segm->nselected=selected;
      }
    }
  }
  return iformat;
}

void set_selected(GROUP *group,GENMODES *gmode)
{
  GROUP *grp;
  CHANNEL *chan;
  SEGMENT *segm;
  for (grp=group; grp; grp=grp->next)
  {
    grp->selected=grp->nselected;
    for (chan=grp->pro_epi; chan; chan=chan->next)
    {
      chan->selected=chan->nselected;
      for (segm=chan->pro; segm; segm=segm->next)
      {
        segm->selected=segm->nselected;
      }
      for (segm=chan->epi; segm; segm=segm->next)
      {
        segm->selected=segm->nselected;
      }
    }

    for (chan=grp->chan; chan; chan=chan->next)
    {
      chan->selected=chan->nselected;
      for (segm=chan->segm; segm; segm=segm->next)
      {
        segm->selected=segm->nselected;
      }
      for (segm=chan->pro; segm; segm=segm->next)
      {
        segm->selected=segm->nselected;
      }
      for (segm=chan->epi; segm; segm=segm->next)
      {
        segm->selected=segm->nselected;
      }
    }

    for (chan=grp->chan; chan; chan=chan->next)
    {
      if (((grp->selected) || (chan->selected)) && (gmode))
      {
        if (grp->pro_epi)
        {
          if (grp->pro_epi->pro) grp->pro_epi->pro->selected|=gmode->gen_pro;
          if (grp->pro_epi->epi) grp->pro_epi->epi->selected|=gmode->gen_epi;
        }

        if (chan->pro) chan->pro->selected|=gmode->gen_pro;
        if (chan->epi) chan->epi->selected|=gmode->gen_epi;
      }
    }
  }
}

GROUP *get_first_selected_grp(GROUP *group)
{
  GROUP *grp;
  CHANNEL *chan;
  SEGMENT *segm;
  for (grp=group; grp; grp=grp->next)
  {
    if (grp->nselected) return grp;
    for (chan=grp->chan; chan; chan=chan->next)
    {
      if (chan->nselected) return grp;
      for (segm=chan->segm; segm; segm=segm->next)
      {
        if (segm->nselected) return grp;
      }
    }
  }
  return NULL;
}

void clearsel_remnodetree(GROUP *grp)
{
  CHANNEL *chan;
  SEGMENT *segm;
  for (; grp; grp=grp->next)
  {
    grp->nselected=FALSE;
    grp->node=NULL;

    for (chan=grp->chan; chan; chan=chan->next)
    {
      chan->nselected=FALSE;
      chan->node=NULL;
      for (segm=chan->segm; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
      for (segm=chan->pro; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
      for (segm=chan->epi; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
    }
    for (chan=grp->pro_epi; chan; chan=chan->next)
    {
      chan->nselected=FALSE;
      chan->node=NULL;
      for (segm=chan->pro; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
      for (segm=chan->epi; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
      for (segm=chan->segm; segm; segm=segm->next)
      {
        segm->nselected=FALSE;
        segm->node=NULL;
      }
    }
  }
}

