/*--------------------------------------------------------------------------------
 *
 *  Contents      :  XML *parse_xml(FILE *fp)
 *                   XML *find_tag(XML *x,char *name)
 *                   XML *find_tag_cur(XML *x,char *name)
 *                   void pri_xml(XML *xml)
 *
 *
 *--------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
typedef enum {FALSE=0,TRUE=1} boolean;

#include "parse_xml.h"

#define __Create_Linked_Struct__(snw)     \
{                                         \
  XML *sc;                                \
  snw=calloc(1,sizeof(*snw));             \
  if (!snw)                               \
  {                                       \
    fprintf(stderr,"FATAL ERROR: Out of memory! (Trying allocating %d bytes...)\n",sizeof(*snw)); \
    exit(-1);                             \
  }                                       \
  if (!*s)                                \
  {                                       \
    *s=snw;                               \
  }                                       \
  else                                    \
  {                                       \
    for (sc=(*s); sc->next; sc=sc->next); \
    sc->next=snw;                         \
    snw->prev=sc;                         \
  }                                       \
}

/****************************************************
 * Create a new member of linked xml-struct.
 *  s    : first member; 
 *           . argument must be set outside func to NULL at start
 *           . will be initialized at first access
 *           . will keep pointer at next accesses
 * return: pointer to new xml-member  
 ****************************************************/
static XML *Create_XMLTag(XML **s)
{
  XML *snw;
  __Create_Linked_Struct__(snw);

  return snw;
}

void Remove_XMLTags(XML *xml)
{
  XML *xmlnext;
  for (; xml; xml=xmlnext)
  {
    xmlnext=xml->next;
    Remove_XMLTags(xml->lower);
    free(xml);
  }
}

/* Shift character into string */
void shift_str(char *str,int n,char c)
{
  int i;
  for (i=0; i<n-1; i++) str[i]=str[i+1];
  str[i]=c; str[i+1]=0;
}

// Skip commented part (hierarchical)
#define CMTXML_BEGIN "<!--"
#define CMTXML_END "-->"
static int fgetc_skip_cmt(FILE *fp)
{
  size_t pos;
  char c,n;
  int i;
  char cmt_str[sizeof(CMTXML_END)+1];
  c=fgetc(fp);
  pos=ftell(fp);

  if (c==EOF) return c;
  if (c!=CMTXML_BEGIN[0]) return c;

  // Found possible start commented part
  for (i=1; i<strlen(CMTXML_BEGIN); i++)
  {
    n=fgetc(fp);
    if (n==EOF)
    {
      fseek(fp,pos,SEEK_SET);
      return c;
    }
    if (n!=CMTXML_BEGIN[i])
    {
      fseek(fp,pos,SEEK_SET);
      return c;
    }
  }

  // commented part started
  while ((c=fgetc_skip_cmt(fp))!=EOF)    // go level deeper if comment in comment
  {
    shift_str(cmt_str,4,c);
    if (!strncmp(cmt_str,CMTXML_END,strlen(CMTXML_END))) break;
  }
  return c;
}


/****************************************************
 * detect type-of-tag:
 * 'o': start tag       ('<')
 * 'c': close tag       ('</')
 * 'e': end tag         ('>')
 * 'E': closing end-tag ('/>')
 * 'f': end-of-file
 * 'n': no tag
 ****************************************************/
static char type_tag(FILE *fp,int c)
{
  static boolean is_str;
  if (!fp)
  {
    is_str=FALSE;
    return 0;
  }

  if (c=='"') is_str=~is_str;
  if (is_str) return 'n';
  if (c=='<')
  {
    long pos=ftell(fp);
    int c1=fgetc_skip_cmt(fp);
    if (c1=='?') return '?';
    if (c1=='/') return 'c';
    fseek(fp,pos,SEEK_SET);
    if (c1==EOF) return 'f';
    return 'o';
  }
  if (c=='/')
  {
    long pos=ftell(fp);
    int c1=fgetc_skip_cmt(fp);
    if (c1=='>') return 'E';
    fseek(fp,pos,SEEK_SET);
    if (c1==EOF) return 'f';
  }
  if (c=='>') return 'e';
  return 'n';
}

static int catch_xmltag(FILE *fp,char type,char *val,char *strtag)
{
  static boolean is_str;
  int c,c1;
  int i;
  char ctype;
  int get_val=1;
  if (!fp)
  {
    is_str=FALSE;
    return 0;
  }
  while ((c=fgetc_skip_cmt(fp))!=EOF)
  {
    ctype=type_tag(fp,c);
    if (ctype=='?') continue;
    if (((!type) && (ctype!='n')) || (ctype==type))
    {                           // it's a tag, or requested tag (input 'type')
      for (i=0; i<LENSTR; i++)  // go to end tag: 'e' or closing end-tag: 'E'
      {
        if ((c=fgetc_skip_cmt(fp))==EOF) return 0;
        c1=type_tag(fp,c);
        if (c1=='e') return ctype;   // end-of-tag reached: '>'  ==> <...>
        if (c1=='E') return 'E';     // end-of-tag reached: '\>' ==> <...\>
        if (c=='"') is_str=~is_str;

        if ((c==' ') && (!is_str))
        {
          get_val=0;
          if ((c=fgetc_skip_cmt(fp))==EOF) return 0;
        }
        if ((val) && (get_val))
        {
          *val=c; val++; *val=0;
        }
        if ((strtag) && (!get_val))
        {
          *strtag=c; strtag++; *strtag=0;
        }
      }
    }
  }
  if (c==EOF) return 0;
  return type;
}

/****************************************************
 * Parse xml-file from start-of-tag to related end-tag.
 *   Recursive.
 *   Result in linked struct 'xml'
 *   Args:
 *     fp    : file pointer
 *     sp    : start position in fp
 *     d     : recursive depth (not really used)
 *     xml   : Generated xml-struct
 *     return: 0 if OK, otherwise error
 ****************************************************/
static long parse_xml1(FILE *fp,long sp, int d,XML **xml,char *ptag,int debug)
{
  XML *xml1,*xml2;
  int i,c,t,c1;
  long q1,q2;
  char tag[LENSTR+1];
  char tag1[LENSTR+1];
  char strtag[LENSTR+1];
  static int err;
  if (!fp)
  {
    err=0;
    return err;
  }
  fseek(fp,sp,SEEK_SET);
  *strtag=0;
  while ((t=catch_xmltag(fp,0,tag,strtag)))           // go to next tag
  {
    if (t=='o')                                       // open tag ('<..>')
    {
      xml1=Create_XMLTag(xml);                        // create new xml-item
      strcpy(xml1->name,tag);
      strcpy(xml1->extname,strtag);
      q1=ftell(fp);                                   // position after current tag
      t=catch_xmltag(fp,0,tag1,NULL);                 // go to next tag
      if ((t=='o')||(t=='E'))                                     // again open tag
      {
        parse_xml1(fp,q1,d+1,&xml1->lower,tag,debug); //   so one level deeper
        xml1->lower->upper=xml1;
      }
      else if (t=='E')                                 //  tag: '<../>'
      {
        xml2=Create_XMLTag(&xml1->lower);              // create new xml-item
        strcpy(xml2->name,tag1);
      }
      else if (t=='c')                                 // close tag: '</'
      {
        if ((t=='c') && (strcmp(tag1,tag)))
        {
          err++;
          if (debug) printf("XML PARSE ERROR1: %s, expected %s\n",tag1,tag);
        }

        q2=ftell(fp);                                 // pos. of tag, expected '>' or '/>'
        fseek(fp,q1,SEEK_SET);                        // go back to previous open tag

        for (i=0; i<LENSTR; i++)                      //   copy text between open/close tag to xml-struct
        {
          if ((c=fgetc_skip_cmt(fp))==EOF) break;
          c1=type_tag(fp,c);
          if ((c1=='c') || (c1=='E'))                 //   close tag
          {
            fseek(fp,q2,SEEK_SET);                    //   go to end of this tag
            break;                                    //   stop copy
          }
          xml1->str[i]=c;                             //   copy 1 character to xml
        }
      }
    }
    else if (t=='E')
    {
      xml2=Create_XMLTag(xml);                        // create new xml-item
      strcpy(xml2->name,tag);
      strcpy(xml2->extname,strtag);
    }
    else if (t=='c')                    // close tag without open tag
    {
      if ((ptag) && (strcmp(ptag,tag)))
      {
        err++;
        if (debug) printf("XML PARSE ERROR2: %s, expected %s\n",tag,ptag);
      }
      break;
    }
  }
  return err;
}

/****************************************************
 * Parse xml-file; result in linked struct 'xml'
 *   fp    : file pointer
 *   return:  Generated xml-struct
 ****************************************************/
XML *parse_xml(FILE *fp)
{
  XML *xml=NULL;
  type_tag(NULL,0);                         // reset internal flag
  catch_xmltag(NULL,0,NULL,NULL);           // reset internal flag

  parse_xml1(fp,0,0,&xml,NULL,1);
  return xml;
}


/****************************************************
 * Search a tag 'somewhere' in xml-file, at/below 'x'
 *   x     : search from here, and below
 *   name  : name of tag to find
 *   return: x with found name or NULL
 ****************************************************/
XML *find_tag(XML *x,char *name)
{
  XML *x1,*x2;
  forall_tags(x,x1)
  {
    if (!strcmp(x1->name,name)) break;
    if ((x2=find_tag(x1->lower,name))) return x2;
  }
  return x1;
}

/****************************************************
 * Search a tag in xml-file, at current level 'x'
 *   x     : search from here
 *   name  : name of tag to find
 *   return: x with found name or NULL
 ****************************************************/
XML *find_tag_cur(XML *x,char *name)
{
  XML *x1;
  forall_tags(x,x1)
  {
    if (!strcmp(x1->name,name)) return x1;
  }
  return NULL;
}

static void pri_xml1(XML *xml,int d)
{
  int i;
  for (; xml; xml=xml->next)
  {
    for (i=0; i<d; i++) printf("  ");
    printf("TAGNAME: %-20s",xml->name); 
    for (i=0; i<4-d; i++) printf("  ");
    if (*xml->extname) printf(" [ext=%s]",xml->extname);
    if (*xml->str) printf(" = %s",xml->str); 
    puts("");
    pri_xml1(xml->lower,d+1);
  }
}

void pri_xml(XML *xml)
{
  pri_xml1(xml,0);
}

