/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * Preferences functions
 ********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xrit2pic.h"
#include "xrit_prefs.h"
#include "avhrr.h"
#include "xrit2pic_funcs.h"
#include "xrit2pic_defines.h"

#define DEFSAT_SHOW "HMSG3"
#define DEFCHAN_DEP "IR_108"

extern GLOBINFO globinfo;

/* Default mappings */
FMAP fmap_vis_hmsg=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  3,
  {
    { "VIS006", 0., 0., 1., FALSE },
    { "VIS008", 0., 1., 0., FALSE },
    { "IR_016", 1., 0., 0., FALSE },
  }
};

FMAP fmap_vis_hmsgdn=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  4,
  {
    { "VIS006", 0.0, 0.0, 0.8, FALSE },
    { "VIS008", 0.0, 0.8, 0.0, FALSE },
    { "IR_016", 0.8, 0.0, 0.0, FALSE },
    { "IR_087", 0.6, 0.6, 0.6, TRUE },
  }
};

FMAP fmap_vis_lmsg=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  2,
  {
    { "VIS006", 0., 1., 1., FALSE  },
    { "IR_016", 1., 0., 0., FALSE  },
  }
};

FMAP fmap_vis_noaa=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  3,
  {
    { A_CH1, 1., 0., 0., FALSE  },
    { A_CH2, 0., 1., 0., FALSE  },
    { A_CH4, 0., 0., 1., FALSE  },
  }
};

FMAP fmap_vis_metop=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  3,
  {
    { A_CH1, 1., 0., 0., FALSE  },
    { A_CH2, 0., 1., 0., FALSE  },
    { A_CH3, 0., 0., 1., FALSE  },
  }
};

FMAP fmap_ncvis_hima=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  3,
  {
    { "VIS"   , 0., 0., 1., FALSE },
    { "B04"   , 0., 1., 0., FALSE },
    { "B05"   , 1., 0., 0., FALSE },
  }
};

FMAP fmap_ncvisdn_hima=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  4,
  {
    { "VIS"   , 0.0, 0.0, 0.8, FALSE },
    { "B04"   , 0.0, 0.8, 0.0, FALSE },
    { "B05"   , 0.8, 0.0, 0.0, FALSE },
    { "B11"   , 0.3, 0.3, 0.3, FALSE  },
  }
};

FMAP fmap_tcvis_hima=
{
  {0,0,0},
  {1.,1.,1.},
  FALSE,
  3,
  {
    { "B01"   , 0., 0., 1., FALSE },
    { "B02"   , 0., 1., 0., FALSE },
    { "B04"   , 1., 0., 0., FALSE },
  }
};

/* Defaults fixed zoom area */
#define MAX_PREDEF_AREAS 4
GEOAREADEFS gadef[NRGEODEFS]=
{
  { "Europe"  ,{ 0.,0.,0.,  6.0, 56.0 }, { 0.,0.,0., 19.7, 21.0 }},
  { "Atlantic",{ 0.,0.,0.,-18.0, 42.0 }, { 0.,0.,0., 19.7, 20.0 }},
  { ""        ,{ 0.,0.,0.,  0.0,  0.0 }, { 0.,0.,0.,  0.0,  0.0 }},
  { ""        ,{ 0.,0.,0.,  0.0,  0.0 }, { 0.,0.,0.,  0.0,  0.0 }}
};

float get_gadef(int i,int j)
{
  if (j==0) return gadef[i].center.lon;
  if (j==1) return gadef[i].center.lat;
  if (j==2) return gadef[i].delta.lon;
  if (j==3) return gadef[i].delta.lat;
  return 0.;
}

int search_file_1dir(char *rootfile,char *file,char *dir)
{
  char *fn;
  if (!dir) return 0;
  if (!*rootfile) return 0;
  if (!file) fn=malloc(strlen(dir)+strlen(rootfile)+10);
  else       fn=file;
  if (!fn) return 0;

  sprintf(fn,"%s%c%s",dir,DIR_SEPARATOR,rootfile);

  if ((exist_file(fn)))
  {
    if ((!file)&&(fn)) free(fn);
    return 1;
  }
  if ((!file)&&(fn)) free(fn);
  return 0;
}

#define EXTRA_DIR "xrit_extra"
/******************************************
 * Search root file; return full path in file.
 *  return: 1 found in  cur_dir
 *        : 2 found in  home_dir
 *        : 3 found in  prog_dir
 *        : 4 found in  prog_dir/extra_dir
 ******************************************/
int search_file(char *rootfile,char *file,
                 char *cur_dir,char *home_dir,char *prog_dir)
{
  char *extra_dir;
  if (search_file_1dir(rootfile,file,cur_dir)) return 1;
  if (search_file_1dir(rootfile,file,home_dir)) return 2;
  if (search_file_1dir(rootfile,file,prog_dir)) return 3;

  extra_dir=malloc(strlen(prog_dir)+strlen(EXTRA_DIR)+10);
  sprintf(extra_dir,"%s%c%s",prog_dir,DIR_SEPARATOR,EXTRA_DIR);
  if (search_file_1dir(rootfile,file,extra_dir))
  {
    free(extra_dir);
    return 4;
  }
  free(extra_dir);
  
  if (file) *file=0;
  return 0;
}
 
void get_dirs(char *progname,char **cur_dir,char **prog_dir,char **prog_xdir,char **home_dir)
{
  freed(cur_dir);
  freed(prog_dir); 
  freed(prog_xdir);
  freed(home_dir); 

  *cur_dir=g_get_current_dir();
  if (!*cur_dir) strcpyd(cur_dir,".");

  *prog_dir=get_path(progname);
  if (!*prog_dir) strcpyd(prog_dir,".");

  *prog_xdir=malloc(strlen(*prog_dir)+strlen(EXTRA_DIR)+10);
  sprintf(*prog_xdir,"%s%c%s",*prog_dir,DIR_SEPARATOR,EXTRA_DIR);

  *home_dir=(char *)g_get_home_dir();

  if (!(*home_dir))
  {
    #if __GTK_WIN32__ == 1
    {
      if (!strncmp(*cur_dir+1,":\\",2))
      {
        strcpyd(home_dir,*cur_dir);
        *((*home_dir)+2)=0;
      }
      else
      {
        strcpyd(home_dir,*cur_dir);
      }
    }
    #else
    strcpyd(home_dir,".");
    #endif
  }
}
/*
#define WIN_SRCDIR "c:\\tellique\\received\\"
#define LIN_SRCDIR "/dosc/tellique/received/"
*/

#define WIN_TLQDIR "\\Program Files\\T-Systems\\BusinessTV-IP\\"
#define LIN_TLQPDIR "$HOME/bin/"
#define LIN_TLQSDIR "$HOME/tq/"

//#define WIN_SRCDIR "c:\\Program Files\\T-Systems\\BusinessTV-IP\\received\\"
#define WIN_SRCDIR "\\ecast\\received\\"
#define LIN_SRCDIR "$HOME/from_msg/received/"

#define WIN_DSTDIR "\\ecast\\exported\\"
#define LIN_DSTDIR "$HOME/"

#define WIN_LUTDIR "\\lut\\"
#define LIN_LUTDIR "$HOME/lut/"
#define LUT_FNAME "LUT361_MM-HRIT-cc.bmp"

#define NOAAKEPFILE "weather.txt"
#define EARTHMAPFILE "earthmap.gif"
#define CITYGPXFILE "cities.gpx"
#define CITYGPXFILE2 "Cities.dat"

#define PART_NAME "part_name"
#define PART_LON_CENTER "part_lon_center"
#define PART_LAT_CENTER "part_lat_center"
#define PART_LON_DELTA "part_lon_delta"
#define PART_LAT_DELTA "part_lat_delta"
#define PART_WIDTH "part_width"
#define PART_HEIGHT "part_height"
#define PART_WIDTH_OFF "part_woffs"
#define PART_HEIGHT_OFF "part_hoffs"
#define PART_POLDIR "part_polar_dir"


// define preferences items
static PREFITEMS *Create_BPref(PREFITEMS **s,char *name,
    gboolean  *bval0, gboolean *bval1,gboolean  bdef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->bdef[0]=bdef;
  snw->bval[0]=bval0;
  snw->bval[1]=bval1;
  if (bval0) *bval0=bdef;
  if (bval1) *bval1=bdef;
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_IPref(PREFITEMS **s,char *name,
    int  *ival0,int  *ival1,int  idef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->idef[0]=idef;
  snw->ival[0]=ival0;
  snw->ival[1]=ival1;
  if (ival0) *ival0=idef;
  if (ival1) *ival1=idef;
  strcpyd(&snw->name,name);
  return snw;
}


static PREFITEMS *Create_2IPref(PREFITEMS **s,char *name,
    int  *ival0,int  *ival1,int idefa,int idefb,char sep)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->idef[0]=idefa;
  snw->idef[1]=idefb;
  snw->ival[0]=ival0;
  snw->ival[1]=ival1;
  snw->sep=sep;
  if (ival0) ival0[0]=idefa;
  if (ival0) ival0[1]=idefb;
  if (ival1) ival1[0]=idefa;
  if (ival1) ival1[1]=idefb;
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_FPref(PREFITEMS **s,char *name,
    float  *fval0,float  *fval1,float  fdef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->fdef[0]=fdef;
  snw->fval[0]=fval0;
  snw->fval[1]=fval1;
  if (fval0) *fval0=fdef;
  if (fval1) *fval1=fdef;
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_PPref(PREFITEMS **s,char *name,
    POINT  *ip0,POINT  *ip1,POINT pdef,char sep)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->pdef=pdef;
  snw->pval[0]=ip0;
  snw->pval[1]=ip1;
  if (sep) snw->sep=sep; else snw->sep=',';
  if (ip0) *ip0=pdef;
  if (ip1) *ip1=pdef;
  snw->name=name;
  return snw;
}

static PREFITEMS *Create_XPref(PREFITEMS **s,char *name,
    int  *ival0,int  *ival1,int  idef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->hex_nrdig=3;
  snw->idef[0]=idef;
  snw->ival[0]=ival0;
  snw->ival[1]=ival1;
  if (ival0) *ival0=idef;
  if (ival1) *ival1=idef;
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_CPref(PREFITEMS **s,char *name,
    char  *ival0,int  *ival1,char  idef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);
  snw->type='c';
  snw->idef[0]=idef;
  snw->ival[0]=(int *)ival0;
  snw->ival[1]=(int *)ival1;
  if (ival0) *ival0=idef;
  if (ival1) *ival1=idef;
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_SPref(PREFITEMS **s,char *name,
    char *sval0,char *sval1,char *sdef)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);

  snw->sdef=sdef;
  snw->sval[0]=sval0;
  snw->sval[1]=sval1;
  if ((sval0) && (sdef)) strncpy(sval0,sdef,MAXLENPREF);
  if ((sval1) && (sdef)) strncpy(sval1,sdef,MAXLENPREF);
  strcpyd(&snw->name,name);
  return snw;
}

static PREFITEMS *Create_TPref(PREFITEMS **s,char *str)
{
  PREFITEMS *snw,*sc;
  __Create_Linked_Struct__(snw,sc);

  snw->type='t';
  strcpyd(&snw->name,str);
  return snw;
}

// to use in pref. parse loop, called by each readed item in preferences file
static void Set_Pref(PREFITEMS *pf,char *w1,char *w2)
{
  for (; pf; pf=pf->next)
  {
    if (!strcmp(pf->name,w1))
    {
      if (pf->bval[0]) *pf->bval[0]=(*w2=='y'? TRUE : FALSE);
      if (pf->ival[0])
      {
        if (pf->hex_nrdig)
         *pf->ival[0]=strtol(w2,NULL,16);
        else if (pf->sep)
        {
          pf->ival[0][0]=atoi(w2);
          if ((w2=strchr(w2,pf->sep)))
            pf->ival[0][1]=atoi(w2+1);
        }
        else if (pf->type=='c')
         *pf->ival[0]=*w2;
        else
         *pf->ival[0]=atoi(w2);
      }
      if (pf->fval[0])
      {
        if (pf->sep)
        {
          pf->fval[0][0]=atof(w2);
          if ((w2=strchr(w2,pf->sep)))
            pf->fval[0][1]=atof(w2+1);
        }
        else
         *pf->fval[0]=atof(w2);
      }
      if (pf->sval[0]) strcpy(pf->sval[0],w2);
      if (pf->pval[0])
      {
        pf->pval[0]->lat=atof(w2);
        if ((w2=strchr(w2,pf->sep)))
          pf->pval[0]->lon=atof(w2+1);
      }
    }
  }
}

void Write_Pref(PREFITEMS *pf,FILE *fp)
{
  for (; pf; pf=pf->next)
  {
    if (*pf->name)
    {
      if (pf->type=='t')
      {
        fprintf(fp,"%s\n",pf->name);
        continue;
      }
      if (pf->bval[0]) fprintf(fp,"%s=%c\n",pf->name,(*pf->bval[0]? 'y' : 'n'));
      if (pf->ival[0])
      {
        char frmt[30];
        if (pf->hex_nrdig)
          sprintf(frmt,"%%s=%%0%dx\n",pf->hex_nrdig);
        else if (pf->type=='c')
          sprintf(frmt,"%%s=%%c\n");
        else
          sprintf(frmt,"%%s=%%d\n");

        if (pf->sep)
          fprintf(fp,"%s=%d%c%d\n",pf->name,pf->ival[0][0],pf->sep,pf->ival[0][1]);
        else
          fprintf(fp,frmt,pf->name,*pf->ival[0]);
      }
      if (pf->fval[0])
      {
        if (pf->sep)
          fprintf(fp,"%s=%f%c%f\n",pf->name,pf->fval[0][0],pf->sep,pf->fval[0][1]);
        else
          fprintf(fp,"%s=%f\n",pf->name,*pf->fval[0]);
      }
      if (pf->sval[0])
      {
        if ((strchr("adstT0",*pf->name)) && (*(pf->name+1)==0))
          fprintf(fp,"%s %s\n",pf->name,pf->sval[0]);
        else
          fprintf(fp,"%s=%s\n",pf->name,pf->sval[0]);
      }
      if (pf->pval[0])
      {
        if (pf->sep)
          fprintf(fp,"%s=%f%c%f\n",pf->name,pf->pval[0]->lat,pf->sep,pf->pval[0]->lon);
        else
          fprintf(fp,"%s=%f%c%f\n",pf->name,pf->pval[0]->lat,':',pf->pval[0]->lon);
      }
    }
  }
}

static void str_rshift(char *s,int n)
{
  int i=strlen(s);
  for (; i>=0; i--) s[i+n]=s[i];
}

static PREFITEMS *pf;
void Load_Defaults(PREFER *prefer)
{
  int i;
  pf=NULL;

  Create_TPref(&pf,"# xrit2pic.ini");
  Create_BPref(&pf,"use_restore_guistate"     ,&prefer->enable_restore_guistate       ,NULL,FALSE);
  Create_IPref(&pf,"window_width"             ,&prefer->wndwidth                      ,NULL,400);
  Create_IPref(&pf,"window_height"            ,&prefer->wndheight                     ,NULL,500);
  Create_IPref(&pf,"font_size"                ,&prefer->font_size                     ,NULL,0);
  Create_IPref(&pf,"prev_window_width"        ,&prefer->pr_wndwidth                   ,NULL,500);
  Create_IPref(&pf,"prev_window_height"       ,&prefer->pr_wndheight                  ,NULL,500);
  Create_BPref(&pf,"prev_window_screensize"   ,&prefer->pr_wndmax                     ,NULL,FALSE);
  Create_BPref(&pf,"prev.correct_aspect"      ,&prefer->pr_corr_aspect                ,NULL,TRUE);
  Create_IPref(&pf,"live_window_width"        ,&prefer->lv_wndwidth                   ,NULL,500);
  Create_IPref(&pf,"live_window_height"       ,&prefer->lv_wndheight                  ,NULL,500);
  Create_BPref(&pf,"live_window_screensize"   ,&prefer->lv_wndmax                     ,NULL,FALSE);
  Create_IPref(&pf,"jpeg_quality"             ,&prefer->jpeg_quality                  ,NULL,90);
  Create_IPref(&pf,"movie_quality"            ,&prefer->movie_quality                 ,NULL,60);
  Create_IPref(&pf,"film_size"                ,&prefer->movie_resol                   ,NULL,2048);
  Create_IPref(&pf,"polar_max_width"          ,&prefer->polar_size                    ,NULL,7200);
  Create_CPref(&pf,"overlay_type"             ,&prefer->prf_ovltype                   ,NULL,'P');
  Create_BPref(&pf,"add_landmarks"            ,&prefer->add_marks                     ,NULL,TRUE);
  Create_IPref(&pf,"size_landmark"            ,&prefer->size_cmark                    ,NULL,50);
  Create_SPref(&pf,"location_cities"          ,prefer->placesfile                     ,NULL,CITYGPXFILE);

  Create_BPref(&pf,"enable_anno_timestamp"    ,&prefer->annotxt[0].enable             ,NULL,FALSE);
  Create_IPref(&pf,"size_anno_timestamp"      ,&prefer->annotxt[0].size               ,NULL,0);

  Create_BPref(&pf,"enable_annotation1"       ,&prefer->annotxt[1].enable             ,NULL,FALSE);
  Create_SPref(&pf,"text_annotation1"         ,prefer->annotxt[1].str                 ,NULL,"@ Eumetsat");
  Create_IPref(&pf,"size_annotation1"         ,&prefer->annotxt[1].size               ,NULL,0);

  Create_BPref(&pf,"enable_annotation2"       ,&prefer->annotxt[2].enable             ,NULL,FALSE);
  Create_SPref(&pf,"text_annotation2"         ,prefer->annotxt[2].str                 ,NULL,"");
  Create_IPref(&pf,"size_annotation2"         ,&prefer->annotxt[2].size               ,NULL,0);

  // Store buttons in main tab
  Create_TPref(&pf,"# ---------- Main tab ----------");
  Create_BPref(&pf,"main.overlay"             ,&prefer->pm.ol.add_overlay             ,&globinfo.pm.ol.add_overlay ,FALSE);
  Create_BPref(&pf,"main.lonlat"              ,&prefer->pm.ol.add_lonlat              ,&globinfo.pm.ol.add_lonlat  ,FALSE);
  Create_BPref(&pf,"main.cities"              ,&prefer->pm.ol.add_cities              ,&globinfo.pm.ol.add_cities  ,FALSE);
  Create_BPref(&pf,"main.mark"                ,&prefer->pm.ol.add_marks               ,&globinfo.pm.ol.add_marks   ,FALSE);
  Create_BPref(&pf,"main.compose"             ,&prefer->pm.clr.compose                ,&globinfo.pm.clr.compose    ,FALSE);
  Create_BPref(&pf,"main.hrv_lum"             ,&prefer->pm.clr.hrv_lum                ,&globinfo.pm.clr.hrv_lum    ,FALSE);
  Create_SPref(&pf,"main.clrmapfixed"         ,prefer->pm.clrmapfixed                 ,globinfo.pm.clrmapfixed     ,"");
  Create_SPref(&pf,"main.overlaytype"         ,prefer->pm.oltype                      ,globinfo.pm.oltype          ,"Cntry");


  // Store buttons in record tab
  Create_TPref(&pf,"# ---------- Record tab ----------");
  Create_BPref(&pf,"record_at_start"          ,&prefer->pr.record_at_start            ,&globinfo.pr.record_at_start    ,FALSE);

  Create_TPref(&pf,"#   subtab 'Show live'");
  Create_BPref(&pf,"live.enable"              ,&prefer->pr.lvs_enable                 ,&globinfo.pr.lvs_enable             ,TRUE );
  Create_BPref(&pf,"live.colour"              ,&prefer->pr.lv_clr.compose             ,&globinfo.pr.lv_clr.compose         ,FALSE);
  Create_BPref(&pf,"live.hrvlum"              ,&prefer->pr.lv_clr.hrv_lum             ,&globinfo.pr.lv_clr.hrv_lum         ,FALSE);
  Create_BPref(&pf,"live.invert"              ,&prefer->pr.lv_clr.invert              ,&globinfo.pr.lv_clr.invert          ,FALSE);
  Create_BPref(&pf,"live.overlay"             ,&prefer->pr.lv_ol.add_overlay          ,&globinfo.pr.lv_ol.add_overlay      ,FALSE);
  Create_BPref(&pf,"live.lonlat"              ,&prefer->pr.lv_ol.add_lonlat           ,&globinfo.pr.lv_ol.add_lonlat       ,FALSE);
  Create_BPref(&pf,"live.mark"                ,&prefer->pr.lv_ol.add_marks            ,&globinfo.pr.lv_ol.add_marks        ,FALSE);
  Create_BPref(&pf,"live.cities"              ,&prefer->pr.lv_ol.add_cities           ,&globinfo.pr.lv_ol.add_cities       ,FALSE);
  Create_BPref(&pf,"live.time"                ,&prefer->pr.lv_ol.annotxt[0].enable    ,&globinfo.pr.lv_ol.annotxt[0].enable,FALSE);
  Create_BPref(&pf,"live.text1"               ,&prefer->pr.lv_ol.annotxt[1].enable    ,&globinfo.pr.lv_ol.annotxt[1].enable,FALSE);
  Create_BPref(&pf,"live.text2"               ,&prefer->pr.lv_ol.annotxt[2].enable    ,&globinfo.pr.lv_ol.annotxt[2].enable,FALSE);
  Create_SPref(&pf,"live.sat"                 ,prefer->pr.lv_sat                      ,globinfo.pr.lv_sat                  ,"HMSG2");
  Create_SPref(&pf,"live.channel"             ,prefer->pr.lv_chan                     ,globinfo.pr.lv_chan                 ,"VIS006");

  Create_TPref(&pf,"#   subtab 'Generate'");
  Create_BPref(&pf,"gen_enable"               ,&prefer->pr.lvg_enable                 ,&globinfo.pr.lvg_enable         ,FALSE);
  Create_BPref(&pf,"gen_pic0_movie1"          ,&prefer->pr.lvg_pic0_movie1            ,&globinfo.pr.lvg_pic0_movie1    ,FALSE);
  Create_BPref(&pf,"gen_essential"            ,&prefer->pr.lvg_essential_only         ,&globinfo.pr.lvg_essential_only ,FALSE);
  Create_BPref(&pf,"gen_runscript"            ,&prefer->pr.lvg_run_script             ,&globinfo.pr.lvg_run_script     ,FALSE);
  Create_BPref(&pf,"gen_rerunscript"          ,&prefer->pr.lvg_rerun_script           ,&globinfo.pr.lvg_rerun_script   ,FALSE);

  Create_TPref(&pf,"#   subtab 'Other'");
  Create_BPref(&pf,""                         ,&prefer->pr.msg_notify                 ,&globinfo.pr.msg_notify         ,FALSE);

  Create_TPref(&pf,"#   aging");
  Create_BPref(&pf,"delete_old"               ,&prefer->pr.delete_old_at_start        ,&globinfo.pr.delete_old_at_start,FALSE);
  // End Store buttons in record tab

  Create_TPref(&pf,"# ---------- Colours ----------");
  Create_XPref(&pf,"overlay_clr"              ,&prefer->ovl_clr                       ,NULL,0xfff);
  Create_XPref(&pf,"mark_clr"                 ,&prefer->mark_clr                      ,NULL,0xf00);
  Create_XPref(&pf,"cities_clr"               ,&prefer->city_clr                      ,NULL,0xf00);
  Create_XPref(&pf,"lonlat_clr"               ,&prefer->lonlat_clr                    ,NULL,0x00f);
  Create_IPref(&pf,"lonlat_incr"              ,&prefer->deltalonlat                   ,NULL,10);
  Create_XPref(&pf,"bufrovl_clr"              ,&prefer->scat_clr                      ,NULL,0x940);
  Create_XPref(&pf,"bufrovl_sel_clr"          ,&prefer->scat_selclr                   ,NULL,0xf0f);

  Create_BPref(&pf,"invert_ir"                ,&prefer->invert_ir                     ,NULL,FALSE);

// afwijkend: vishmsg,vislmsg,visnoaa,vismetop

  Create_BPref(&pf,"scandir_correct"          ,&prefer->scandir_correct               ,NULL,TRUE);
  Create_BPref(&pf,"keep_pic_in_mem"          ,&prefer->low_mem                       ,NULL,FALSE);
  Create_BPref(&pf,"avhrr_mode"               ,&prefer->avhrr_mode                    ,NULL,FALSE);
  Create_BPref(&pf,"full_globe"               ,&prefer->full_globe                    ,NULL,FALSE);
  Create_2IPref(&pf,"del_olderthan"           ,prefer->deldaymin                      ,NULL,3,0,':');
  Create_IPref(&pf,"del_newerthan"            ,&prefer->deldaymax                     ,NULL,30);
  Create_BPref(&pf,"del_roundday"             ,&prefer->roundday                      ,NULL,FALSE);

  Create_SPref(&pf,"norad_file"               ,prefer->noradfile                      ,NULL,NOAAKEPFILE);
  Create_SPref(&pf,"earthmap_file1"           ,prefer->earthmapfile1                  ,NULL,EARTHMAPFILE);
  Create_SPref(&pf,"earthmap_file2"           ,prefer->earthmapfile2                  ,NULL,EARTHMAPFILE);

  { POINT dmark; dmark.lon=5.1812; dmark.lat=52.2359;
  Create_PPref(&pf,"markpoint"                ,&prefer->lmark                         ,NULL,dmark,',');
  }

  Create_BPref(&pf,"fixed_drawable"           ,&prefer->size_draw_limit               ,NULL,TRUE);
  Create_IPref(&pf,"update_cycle"             ,&prefer->upd_cycle                     ,NULL,5);
  Create_SPref(&pf,"update_channel"           ,prefer->upd_chan                       ,NULL,"");

 #if __GTK_WIN32__ == 1
  Create_SPref(&pf,"viewer_jpeg"              ,prefer->prog_jpg                       ,NULL,WIN_VIEWER_JPG);
  Create_SPref(&pf,"viewer_pgm"               ,prefer->prog_pgm                       ,NULL,WIN_VIEWER_PGM);
  Create_SPref(&pf,"upside_down_jpg"          ,prefer->ud_jpg                         ,NULL,WIN_SNOPT_JPG);
  Create_SPref(&pf,"upside_down_pgm"          ,prefer->ud_pgm                         ,NULL,WIN_SNOPT_PGM);
  Create_SPref(&pf,"viewer_avi"               ,prefer->prog_playmovie                 ,NULL,WIN_PLAY_MOVIE);
  Create_SPref(&pf,"movie_encoder"            ,prefer->prog_genmovie                  ,NULL,WIN_GEN_MOVIE);
  Create_SPref(&pf,"bufr_extract"             ,prefer->prog_bufrextr                  ,NULL,WIN_BUFR_EXTR);
  Create_SPref(&pf,"bufrtbl_loc"              ,prefer->bufrtblloc                     ,NULL,WIN_BUFRTBL_LOC);
  Create_SPref(&pf,"lut_loc"                  ,prefer->lut_dir                        ,NULL,WIN_LUTDIR);
 #else
  Create_SPref(&pf,"viewer_jpeg"              ,prefer->prog_jpg                       ,NULL,LIN_VIEWER_JPG);
  Create_SPref(&pf,"viewer_pgm"               ,prefer->prog_pgm                       ,NULL,LIN_VIEWER_PGM);
  Create_SPref(&pf,"upside_down_jpg"          ,prefer->ud_jpg                         ,NULL,LIN_SNOPT_JPG);
  Create_SPref(&pf,"upside_down_pgm"          ,prefer->ud_pgm                         ,NULL,LIN_SNOPT_PGM);
  Create_SPref(&pf,"viewer_avi"               ,prefer->prog_playmovie                 ,NULL,LIN_PLAY_MOVIE);
  Create_SPref(&pf,"movie_encoder"            ,prefer->prog_genmovie                  ,NULL,LIN_GEN_MOVIE);
  Create_SPref(&pf,"bufr_extract"             ,prefer->prog_bufrextr                  ,NULL,LIN_BUFR_EXTR);
  Create_SPref(&pf,"bufrtbl_loc"              ,prefer->bufrtblloc                     ,NULL,LIN_BUFRTBL_LOC);
  Create_SPref(&pf,"lut_loc"                  ,prefer->lut_dir                        ,NULL,LIN_LUTDIR);
 #endif
  Create_BPref(&pf,"movie_extern"             ,&prefer->extern_moviegen               ,NULL,FALSE);
  Create_IPref(&pf,"movie_encspeed"           ,&prefer->speed                         ,NULL,1);

  Create_TPref(&pf,"# ---------- (Re)Run commands after generate ----------");
  Create_SPref(&pf,"execute_command"          ,prefer->run_cmd                        ,NULL,"");
  Create_SPref(&pf,"rpt_execute_command"      ,prefer->rerun_cmd                      ,NULL,"");
  Create_BPref(&pf,"pr.moviecmd_constr"       ,&prefer->pr.lvg_moviecmd_constr        ,&globinfo.pr.lvg_moviecmd_constr,FALSE);      
  Create_IPref(&pf,"pr.movie_speed"           ,&prefer->pr.lvg_movie_speed            ,&globinfo.pr.lvg_movie_speed,10);
  Create_BPref(&pf,"pr.movie_fullscreen"      ,&prefer->pr.lvg_movie_fullscreen       ,&globinfo.pr.lvg_movie_fullscreen,FALSE);      
  Create_IPref(&pf,"pr.movie_rpt_lastfrm"     ,&prefer->pr.lvg_movie_rpt_lastfrm      ,&globinfo.pr.lvg_movie_rpt_lastfrm,0);
  Create_IPref(&pf,"pr.movie_size_max_hr"     ,&prefer->pr.lvg_movie_size_max_hr      ,&globinfo.pr.lvg_movie_size_max_hr,24);
  Create_IPref(&pf,"pr.movie_restart_hr"      ,&prefer->pr.lvg_movie_start_utc_hr     ,&globinfo.pr.lvg_movie_start_utc_hr,-1);

  Create_TPref(&pf,"# ---------- other items ----------");
  Create_SPref(&pf,"lut_file"                 ,prefer->lut_file                       ,globinfo.lut.fname,LUT_FNAME);

  Create_SPref(&pf,"anaglyph_depthchan"       ,prefer->depth_chan                     ,NULL,DEFCHAN_DEP);
  Create_BPref(&pf,"anaglyph_depthchan_invert",&prefer->dchan_iv                      ,NULL,TRUE);

  Create_IPref(&pf,"alert_nr_recfiles"        ,&prefer->max_nr_rfiles                 ,NULL,20000);      
  Create_BPref(&pf,"use_filename_format"      ,&prefer->use_fn_frmt                   ,NULL,FALSE);      
  Create_SPref(&pf,"filename_format"          ,prefer->fn_frmt                        ,NULL,"%s_%c_%t"); 
  Create_BPref(&pf,"split_destdir"            ,&prefer->split_destdir                 ,NULL,FALSE);      
  Create_BPref(&pf,"keep_unzipped"            ,&prefer->keep_unzipped                 ,NULL,FALSE);      

//area1=(name=abc, clon=1.2, clat=3.4, dlon=3.0, dlat=4.0)

  for (i=0;  i<NRGEODEFS; i++)
  {
    char str[100];
    if (i>=MAX_PREDEF_AREAS) break;
    sprintf(str,"part_name%d",i+1);
    Create_SPref(&pf,str,prefer->geoarea[i].part_name,NULL,gadef[i].part_name);

    sprintf(str,"part_lon_center%d",i+1);
    Create_FPref(&pf,str,&prefer->geoarea[i].center.lon         ,NULL,gadef[i].center.lon);
    sprintf(str,"part_lat_center%d",i+1);
    Create_FPref(&pf,str,&prefer->geoarea[i].center.lat         ,NULL,gadef[i].center.lat);

    sprintf(str,"part_lon_delta%d",i+1);
    Create_FPref(&pf,str,&prefer->geoarea[i].delta.lon          ,NULL,gadef[i].delta.lon);
    sprintf(str,"part_lat_delta%d",i+1);
    Create_FPref(&pf,str,&prefer->geoarea[i].delta.lat          ,NULL,gadef[i].delta.lat);

    sprintf(str,"part_polar_dir%d",i+1);
    Create_IPref(&pf,str                ,&prefer->geoarea[i].pol_dir            ,NULL,0);
  }

  Create_IPref(&pf,"part_width"               ,&prefer->geoarea[0].pp_norm.pct_width  ,NULL,0);
  Create_IPref(&pf,"part_woffs"               ,&prefer->geoarea[0].pp_norm.pct_woffset,NULL,0);
  Create_IPref(&pf,"part_height"              ,&prefer->geoarea[0].pp_norm.pct_height ,NULL,0);
  Create_IPref(&pf,"part_hoffs"               ,&prefer->geoarea[0].pp_norm.pct_hoffset,NULL,0);

 #if __GTK_WIN32__ == 1
  Create_SPref(&pf,"t"                        ,prefer->tlq_sdir                       ,NULL,WIN_TLQDIR);
  Create_SPref(&pf,"T"                        ,prefer->tlq_pdir                       ,NULL,WIN_TLQDIR);
  Create_SPref(&pf,"a"                        ,prefer->src_archdir                    ,NULL,"");
  Create_SPref(&pf,"d"                        ,prefer->dest_dir                       ,NULL,WIN_DSTDIR);
  Create_SPref(&pf,"s"                        ,prefer->pl.src_dir                        ,NULL,WIN_SRCDIR );
 #else
  Create_SPref(&pf,"t"                        ,prefer->tlq_sdir                       ,NULL,LIN_TLQPDIR);
  Create_SPref(&pf,"T"                        ,prefer->tlq_pdir                       ,NULL,LIN_TLQSDIR);
  Create_SPref(&pf,"a"                        ,prefer->src_archdir                    ,NULL,"");
  Create_SPref(&pf,"d"                        ,prefer->dest_dir                       ,NULL,LIN_DSTDIR);
  Create_SPref(&pf,"s"                        ,prefer->pl.src_dir                        ,NULL,LIN_SRCDIR );
 #endif

  prefer->speed=1;

  strcpy(prefer->done_dir,"done");
  strcpy(prefer->upd_sat,DEFSAT_SHOW);

  prefer->overlay=load_overlaysfiles(prefer);

  for (i=0; i<NRGEODEFS; i++)
  {
    if (i>=MAX_PREDEF_AREAS) break;
    strcpy(prefer->geoarea[i].part_name,gadef[i].part_name);
    prefer->geoarea[i].center.lon=gadef[i].center.lon;
    prefer->geoarea[i].center.lat=gadef[i].center.lat;
    prefer->geoarea[i].delta.lon =gadef[i].delta.lon;
    prefer->geoarea[i].delta.lat =gadef[i].delta.lat;
    prefer->geoarea[i].pol_dir   =0;
  }
  
  prefer->fmap_vis_hmsg=fmap_vis_hmsg;
  prefer->fmap_vis_hmsgdn=fmap_vis_hmsgdn;
  prefer->fmap_vis_lmsg=fmap_vis_lmsg;
  prefer->fmap_vis_noaa=fmap_vis_noaa;
  prefer->fmap_vis_metop=fmap_vis_metop;
  prefer->fmap_ncvis_hima=fmap_ncvis_hima;
  prefer->fmap_ncvisdn_hima=fmap_ncvisdn_hima;
  prefer->fmap_tcvis_hima=fmap_tcvis_hima;


//  prefer->lmark.lon= 5.1812;
//  prefer->lmark.lat=52.2359;
  if (!strncmp(prefer->lut_dir,"$HOME",strlen("$HOME")))
  {
    str_rshift(prefer->lut_dir,strlen(prefer->home_dir)-strlen("$HOME"));
    strncpy(prefer->lut_dir,prefer->home_dir,strlen(prefer->home_dir));
  }
  if (!strncmp(prefer->tlq_sdir,"$HOME",strlen("$HOME")))
  {
    str_rshift(prefer->tlq_sdir,strlen(prefer->home_dir)-strlen("$HOME"));
    strncpy(prefer->tlq_sdir,prefer->home_dir,strlen(prefer->home_dir));
  }
  if (!strncmp(prefer->tlq_pdir,"$HOME",strlen("$HOME")))
  {
    str_rshift(prefer->tlq_pdir,strlen(prefer->home_dir)-strlen("$HOME"));
    strncpy(prefer->tlq_pdir,prefer->home_dir,strlen(prefer->home_dir));
  }

  strcpy(prefer->pl.src_dir,LIN_SRCDIR);
  if (!strncmp(prefer->pl.src_dir,"$HOME",strlen("$HOME")))
  {
    str_rshift(prefer->pl.src_dir,strlen(prefer->home_dir)-strlen("$HOME"));
    strncpy(prefer->pl.src_dir,prefer->home_dir,strlen(prefer->home_dir));
  }

  strcpy(prefer->dest_dir,LIN_DSTDIR);
  if (!strncmp(prefer->dest_dir,"$HOME",strlen("$HOME")))
  {
    str_rshift(prefer->dest_dir,strlen(prefer->home_dir)-strlen("$HOME"));
    strncpy(prefer->dest_dir,prefer->home_dir,strlen(prefer->home_dir));
  }

}

static void read_map(char *key,char *arg,char *name,FMAP *m)
{
  char tkey[20];
  int i;
  sprintf(tkey,"map_%s_offset",name);
  if (!strcmp(key,tkey))
  {
    sscanf(arg,"(%d,%d,%d)",&m->offset[0],&m->offset[1],&m->offset[2]);
  }

  sprintf(tkey,"map_%s_gamma",name);
  if (!strcmp(key,tkey))
  {
    sscanf(arg,"(%f,%f,%f)",&m->gamma[0],&m->gamma[1],&m->gamma[2]);
  }

  for (i=0; i<8; i++)
  {
    sprintf(tkey,"map_%s_fm[%d]",name,i);
    if (!strcmp(key,tkey))
    {
      char *p=strchr(arg,'(');
      if (p)
      {
        *p=0;
        strcpy(m->fm[i].chan,arg);
        *p='(';
        sscanf(p,"(%f,%f,%f)",&m->fm[i].r,&m->fm[i].g,&m->fm[i].b);
        m->n=i+1;
      }
    }
  }
}

// Read area using fromat:
//area<x>=(name="abc 2", clon=1.2, clat=3.4, dlon=3.0, dlat=4.0)
static void read_area(char *key,char *arg,char *name, GEOAREA *geoarea)
{
  char *p1;
  if (!strcmp(key,name))
  {
    char *p[5],*w[2];
    int i,n=0;
    if (!(p[n]=strtok(arg,"(,)"))) return;
    n++;
    for (; n<5; n++)
    {
      if (!(p[n]=strtok(NULL,"(,)"))) break;
    }

    for (i=0; i<n; i++)
    {
      if (!(w[0]=strtok(p[i],"="))) return;
      if (!(w[1]=strtok(NULL,"="))) return;
      w[0]=strtok(w[0]," ");
      for (p1=w[1]; ((*p1) && (*p1==' ')); p1++);
      w[1]=p1;
      w[1]=strtok(w[1],"\"");
      if (!w[1]) break;
      if (!strcmp(w[0],"name"))
      {
        strncpy(geoarea->part_name,w[1],GEOPARTNAMELEN);
      }
      if (!strcmp(w[0],"clon"))
      {
        geoarea->center.lon=atof(w[1]);
      }
      if (!strcmp(w[0],"clat"))
      {
        geoarea->center.lat=atof(w[1]);
      }
      if (!strcmp(w[0],"dlon"))
      {
        geoarea->delta.lon=atof(w[1]);
      }
      if (!strcmp(w[0],"dlat"))
      {
        geoarea->delta.lat=atof(w[1]);
      }
    }
  }
}

int Read_Pref(DIRSEL *dirsel,PREFER *prefer)
{
  FILE *fp;
  char l[200],*w1,*w2;
  Load_Defaults(prefer);

  if (!prefer->used_preffile) return 0;
  if ((fp=fopen(prefer->used_preffile,"r")))
  {
    while (fgets(l,200,fp))
    {
      char *p;
      int area;
      if ((p=strchr(l,0x0d))) *p=0; // remove DOS-ret
      if ((w1=strchr(l,'#'))) *w1=0;

      if (!(w1=strtok(l," \n	="))) continue;
      if (!(w2=strtok(NULL,"\n"))) continue;
//      if (*w2=='"') w2++; if ((p=strchr(w2,'"'))) *p=0; 

      Set_Pref(pf,w1,w2);

      read_map(w1,w2,"vishmsg",&prefer->fmap_vis_hmsg);
      read_map(w1,w2,"vislmsg",&prefer->fmap_vis_lmsg);
      read_map(w1,w2,"visnoaa",&prefer->fmap_vis_noaa);
      read_map(w1,w2,"vismetop",&prefer->fmap_vis_metop);
      
      for (area=1; area<=NRGEODEFS; area++)
      {
        char key[10];
        sprintf(key,"area%d",area);
        read_area(w1,w2,key,&prefer->geoarea[area-1]);
      }
    }
    fclose(fp);
  }
  prefer->deldaymin[1]=MIN(MAX(prefer->deldaymin[1],0),23);
  prefer->deldaymin[0]=MAX(prefer->deldaymin[0],0);

  prefer->geoarea[0].pp_norm.pct_width=MAX(0,MIN(100,prefer->geoarea[0].pp_norm.pct_width));
  prefer->geoarea[0].pp_hrv.pct_width=prefer->geoarea[0].pp_norm.pct_width*33/22;

  prefer->geoarea[0].pp_norm.pct_woffset=MAX(0,MIN(100,prefer->geoarea[0].pp_norm.pct_woffset));
  prefer->geoarea[0].pp_hrv.pct_woffset=prefer->geoarea[0].pp_norm.pct_woffset*51/36;

  prefer->geoarea[0].pp_norm.pct_height=MAX(0,MIN(100,prefer->geoarea[0].pp_norm.pct_height));
  prefer->geoarea[0].pp_hrv.pct_height=prefer->geoarea[0].pp_norm.pct_height;
  prefer->geoarea[0].pp_norm.pct_hoffset=MAX(0,MIN(100,prefer->geoarea[0].pp_norm.pct_hoffset));
  prefer->geoarea[0].pp_hrv.pct_hoffset=prefer->geoarea[0].pp_norm.pct_hoffset;

  load_places(prefer);

  return 1;
}

gboolean is_absdir(char *d)
{
  #if __GTK_WIN32__ == 1
    if (*(d+1)==':') return TRUE;
  #else
    if (*d==DIR_SEPARATOR) return TRUE;
  #endif
  return FALSE;
}

#define TMPDIR "movietmp"
#define TMPSAVDIR "sav"
void related_dirs(GLOBINFO *gi,PREFER *prefer)
{
  sprintf(gi->src_donedir,"%s%s",gi->pl.src_dir,prefer->done_dir);
  finish_path(gi->src_donedir);
  if (is_absdir(prefer->src_archdir))
  {
    sprintf(gi->src_archdir,"%s",prefer->src_archdir);
  }
  else
  {
    sprintf(gi->src_archdir,"%s%s",gi->pl.src_dir,prefer->src_archdir);
  }
  finish_path(gi->src_archdir);
  sprintf(gi->dest_tmpdir,"%s%s%c",gi->dest_dir,TMPDIR,DIR_SEPARATOR);
  sprintf(gi->dest_tmpsavdir,"%s%s%c",gi->dest_tmpdir,TMPSAVDIR,DIR_SEPARATOR);
}

/* Next part contains funcs not needed for non-gui mode. */
#ifndef __NOGUI__

#include "gtk/gtk.h"
static void add_dir_to_list(DIRSEL *dirsel,char *w2)
{
  char *tmp[1];
  tmp[0]=w2;

  if ((dirsel) && (dirsel->list))
  {
    if (!find_clistitem(dirsel->list,tmp[0]))
      gtk_clist_append(GTK_CLIST(dirsel->list), tmp);

  }
}


static void copy_srclisttoclist(DIRSEL *dirsel,PREFER *prefer)
{
  int row=0;
  STRLIST *srclist;
  for (srclist=prefer->pl.srclist; srclist; srclist=srclist->next)
  {
    add_dir_to_list(dirsel,srclist->str);
    if (!strcmp(srclist->str,prefer->pl.src_dir))
    {
      dirsel->selrow=row;
    }
    row++;
  }
}

void copy_clisttosrclist(DIRSEL *dirsel,PREFER *prefer)
{
  int row;
  char *text;
  Remove_Srclist(prefer->pl.srclist); prefer->pl.srclist=NULL;
  for (row=0; ; row++)
  {
    if(!gtk_clist_get_text(GTK_CLIST(dirsel->list), row, 0, &text))
      break;
    Create_Strlist(&prefer->pl.srclist,text);
  }
}


void Read_PrefDirlist(DIRSEL *dirsel,PREFER *prefer)
{
  FILE *fp;
  char l[1000],*w1,*w2;
  if (!prefer->used_preffile) return;
  if ((fp=fopen(prefer->used_preffile,"r")))
  {
    while (fgets(l,1000,fp))
    {
      if ((w1=strchr(l,'#'))) *w1=0;

      if (!(w1=strtok(l," \n	="))) continue;
      if (!(w2=strtok(NULL,"\n"))) continue;

// Handle XRIT source Directory list
      if (strlen(w1)==1)
      {
        if (strchr("0s",*w1))            // line starts with s or 0
        {
          char *p=w2+strlen(w2)-1;
          if (*p != DIR_SEPARATOR)
            sprintf(p,"%s%c",p,DIR_SEPARATOR);

          if (Create_Strlist(&prefer->pl.srclist,w2))
          {
            if (!strcmp(w1,"s"))           // This line contains active dir
              strcpy(prefer->pl.src_dir,w2);
          }
        }
      }
    }
    fclose(fp);
    copy_srclisttoclist(dirsel,prefer);
  }
}

static void Save_PrefDirlist(PREFER *prefer,FILE *fp)
{
  STRLIST *srclist;

  for (srclist=prefer->pl.srclist; srclist; srclist=srclist->next)
  {
    if (strcmp(srclist->str,prefer->pl.src_dir))
      fprintf(fp,"0 %s\n",srclist->str);
  }

}

static void wri_area(FILE *fp,GEOAREA geoarea[])
{
  int i;
  for (i=0; i<NRGEODEFS; i++)
  {
    if (*(geoarea[i].part_name))
    {
      fprintf(fp,"area%d=(name=\"%s\", clon=%.2f, clat=%.2f, dlon=%.2f, dlat=%.2f\n",i+1,
        geoarea[i].part_name,geoarea[i].center.lon,geoarea[i].center.lat,
        geoarea[i].delta.lon,geoarea[i].delta.lat);
    }
  }
}

static void wri_map(FILE *fp,char *name,FMAP m)
{
  int i;
  fprintf(fp,"map_%s_offset=(%d,%d,%d)\n",name,m.offset[0],m.offset[1],m.offset[2]);
  fprintf(fp,"map_%s_gamma=(%.2f,%.2f,%.2f)\n",name,m.gamma[0],m.gamma[1],m.gamma[2]);
  for (i=0; i<m.n; i++)
    fprintf(fp,"map_%s_fm[%d]=%s(%.2f,%.2f,%.2f)\n",name,i,m.fm[i].chan,m.fm[i].r,m.fm[i].g,m.fm[i].b);
  
}

int Save_Pref(GtkCList *clist,PREFER *prefer,char *preffile)
{
  FILE *fp=NULL;
  if (!(fp=fopen(preffile,"w")))
  {
    Create_Message("Error","Can't open pref-file %s.",preffile);
    return 0;
  }

  fprintf(fp,"#Preferences xrit2pic\n");
  Write_Pref(pf,fp);
  Save_PrefDirlist(prefer,fp);

  fprintf(fp,"\n");
  wri_map(fp,"vishmsg",prefer->fmap_vis_hmsg);
  fprintf(fp,"\n");
  wri_map(fp,"vislmsg",prefer->fmap_vis_lmsg);
  fprintf(fp,"\n");
  wri_map(fp,"visnoaa",prefer->fmap_vis_noaa);
  fprintf(fp,"\n");
  wri_map(fp,"vismetop",prefer->fmap_vis_metop);
  fprintf(fp,"\n");
  wri_area(fp,&prefer->geoarea[0]);
  fprintf(fp,"\n");

  fclose(fp);
  return 1;
}

int Save_Pref1(PREFER *prefer,char *preffile)
{
  return Save_Pref(NULL,prefer,preffile);
}

// run AFTER loading files
void pref_to_gui(GtkWidget *wnd,DIRSEL *dirsel,PREFER *prefer)
{
  if (!wnd) return;
  Set_Adjust(wnd,LAB_WNDWIDTH,"%d",prefer->wndwidth);
  Set_Adjust(wnd,LAB_WNDHEIGHT,"%d",prefer->wndheight);
  Set_Adjust(wnd,LAB_PREVWNDWIDTH,"%d",prefer->pr_wndwidth);
  Set_Adjust(wnd,LAB_PREVWNDHEIGHT,"%d",prefer->pr_wndheight);
  Set_Adjust(wnd,LAB_LIVEWNDWIDTH,"%d",prefer->lv_wndwidth);
  Set_Adjust(wnd,LAB_LIVEWNDHEIGHT,"%d",prefer->lv_wndheight);
  Set_Adjust(wnd,LAB_FONTSIZE,"%d",prefer->font_size);
  Set_Adjust(wnd,LAB_FILMSIZE,"%d",prefer->movie_resol);
//  Set_Adjust(wnd,LAB_FILMFRMMAX,"%d",prefer->movie_max_nrframes);
  Set_Adjust(wnd,LAB_UPDCYCLE,"%d",prefer->upd_cycle);
  Set_Entry(wnd,LAB_SHOWCHANPREF,prefer->upd_chan);
  Set_Entry(wnd,LAB_PPGM,prefer->prog_pgm);
  Set_Entry(wnd,LAB_UPGM,prefer->ud_pgm);
  Set_Entry(wnd,LAB_PJPG,prefer->prog_jpg);
  Set_Entry(wnd,LAB_UJPG,prefer->ud_jpg);
  Set_Entry(wnd,LAB_PAVI,"%s",prefer->prog_playmovie); // prog_playmovie may contain %s!
  Set_Entry(wnd,LAB_PMOVIE,"%s",prefer->prog_genmovie);
  Set_Entry(wnd,LAB_RUNCMD,"%s",prefer->run_cmd);
  Set_Entry(wnd,LAB_RERUNCMD,"%s",prefer->rerun_cmd);
//  Set_Entry(wnd,LAB_SMOVIE,prefer->opt_speed);
  Set_Adjust(wnd,LAB_SMOVIE,"%d",MAX(1,prefer->speed));
  Set_Button(wnd,LAB_SCDIRCOR,prefer->scandir_correct);
  Set_Button(wnd,LAB_HOLDCHNK,prefer->low_mem);
  Set_Button(wnd,LAB_FIXDRAW,prefer->size_draw_limit);
  Set_Entry(wnd,LAB_DDIR,prefer->dest_dir);
  Set_Button(wnd,LAB_SPLITDESTDIR,prefer->split_destdir);
  Set_Entry(wnd,LAB_SDIR,prefer->pl.src_dir);
  Set_Entry(wnd,LAB_LDIR,prefer->lut_dir);
  Set_Entry(wnd,LAB_ARCHDIR,prefer->src_archdir);
  Set_Entry(wnd,LAB_BUFRTBLLOC,prefer->bufrtblloc);
  Set_Entry(wnd,LAB_BUFRPROG,prefer->prog_bufrextr);

  Set_Entry(wnd,LAB_NORF,prefer->noradfile);
  Set_Entry(wnd,LAB_CITYLOC,prefer->placesfile);
  Set_Entry(wnd,LAB_EARTHMAP1,prefer->earthmapfile1);
  Set_Entry(wnd,LAB_EARTHMAP2,prefer->earthmapfile2);
  Set_Entry(wnd,LAB_ANADEPCH,prefer->depth_chan);
  Set_Button(wnd,LAB_ANADEPIV,prefer->dchan_iv);
  Set_Button(wnd,LAB_USEFNFRMT,prefer->use_fn_frmt);
  Set_Entry(wnd,LAB_FNFRMT,"%s",prefer->fn_frmt);

  Set_Button(wnd,name2showname(prefer->pr.lv_sat),TRUE);
  Set_Update_channel(wnd);
  Set_Button(wnd,LAB_CRLOAD,prefer->pr.record_at_start);
  Set_Button(wnd,LAB_CDELOLD,globinfo.pr.delete_old_at_start);
  Set_Adjust(wnd,LAB_FILMLASTFRMRPT,"%d",prefer->pr.lvg_movie_rpt_lastfrm);
  Set_Button(wnd,LAB_RUNCMD2,globinfo.pr.lvg_rerun_script);

  // next also loads source dir in 'Directory to add' field
  if ((dirsel) && (dirsel->selrow>=0))
  {
     gtk_clist_select_row(GTK_CLIST(dirsel->list),dirsel->selrow,0);
  }
}

#endif
