/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * start/stope xternal programs. 
 * NOTE: windows.h interferes with xrit2pic.h, so must be in sep. file.
 ********************************************************************/

#if __GTK_WIN32__ == 1
#include <windows.h>
#endif

#include <string.h>

#if __GTK_WIN32__ == 0
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE  /* for kill() */
#endif
#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>    /* for fork, exec, kill */
#include <sys/types.h> /* for pid_t            */
#include <sys/wait.h>  /* for waitpid          */
#include <signal.h>    /* for SIGTERM, SIGKILL */

//#include "xrit2pic_funcs_nogtk.h"
/*
 * fork a child process, execute vlc, and return it's pid.
 * returns -1 if fork failed.
 */
#define MAXNRARGS 10
#define MAXLENPREF 300
static pid_t spawn_prog(char *cmd1)
{
  static char *argvs[MAXNRARGS+1]; // argvs MUST be static!
  char *p,*q;
  char cmd[MAXLENPREF];
  int argc=0,i;
  pid_t pid = fork();
  if (pid == -1)
  {
    perror("fork");
    return -1;
  }
  strncpy(cmd,cmd1,MAXLENPREF);
  while ((p=strstr(cmd,"\\ "))) *(p+1)='@';
  p=strtok(cmd," \n");
  if (p) do
  {
    if (argc>=MAXNRARGS) break;
    argvs[argc++]=p;
  } while ((p=strtok(NULL," \n")));
  argvs[argc]=NULL;

  while ((p=strstr(cmd,"\\@"))) *(p+1)=' ';
  for (i=0; i<argc; i++)
  {
    while ((p=strstr(argvs[i],"\\@")))
    {
      *(p+1)=' ';
      for (q=p; *q; q++) *q=*(q+1);
    }
  }

  /* when you call fork(), it creates two copies of your program:
   * a parent, and a child. you can tell them apart by the return
   * value from fork().  If fork() returns 0, this is is the child
   * process.  If fork() returns non-zero, we are the parent and the
   * return value is the PID of the child process. */
  if (pid == 0)
  {
    /* this is the child process.  now we can call one of the exec
     * family of functions to execute VLC.  when you call exec,
     * it replaces the currently running process (the child process)
     * with whatever we pass to exec.  So our child process will now
     * be running VLC.  exec() will never return except in an error
     * case, since it is now running the VLC code and not our code. */
    execvp(argvs[0], argvs);
    return -1; // Normally never get here!
  }
  else
  {
    /* parent, return the child's PID back to main. */
    return pid;
  }
}


static int stop_prog(pid_t pid)
{
  int status = 0;
  /* kill will send the specified signal to the specified process.
   * in this case, we send a TERM signal to VLC, requesting that it
   * terminate.  If that doesn't work, we send a KILL signal.
   * If that doesn't work, we give up. */
  if (kill(pid, SIGTERM) < 0)
  {
//    perror("kill with SIGTERM");
    if (kill(pid, SIGKILL) < 0)
    {
//      perror("kill with SIGKILL");
    }
  }
  waitpid(pid, &status, 0);
  return status;
}

// what: 
//   0: stop/start
//   1: start
//   2: stop
int rerun_script(char *cmd,int what)
{
  static pid_t pid=0;
  if (!*cmd) return -1;
  if ((what==0) || (what==2))
  {
    if (pid>0) stop_prog(pid);
    pid=-1;
  }
  if ((what==0) || (what==1))
  {
    pid=spawn_prog(cmd);
  }
  return pid;
}

#else
// windows

static int spawn_prog(char *cmd1,PROCESS_INFORMATION *pi)
{
  STARTUPINFO si;
  ZeroMemory(&si, sizeof(si));
  si.cb=sizeof(si);

  ZeroMemory(pi, sizeof(*pi) );
  if( !CreateProcess
  (
    NULL,             // No module name (use command line)
    cmd1,             // Command line
    NULL,             // Process handle not inheritable
    NULL,             // Thread handle not inheritable
    FALSE,            // Set handle inheritance to FALSE
    CREATE_NO_WINDOW, // No window
    NULL,             // NULL: Use parent's environment block
    NULL,             // NULL: Use parent's starting directory 
    &si,              // Pointer to STARTUPINFO structure
    pi )              // Pointer to PROCESS_INFORMATION structure
  ) 
  {
    return -1;
  }
  return 1;
}

static void stop_prog(PROCESS_INFORMATION pi)
{
  TerminateProcess(pi.hProcess,0); 
  CloseHandle( pi.hProcess );
  CloseHandle( pi.hThread );
}

int rerun_script(char *cmd,int what)
{
  static int pid=0;
  static PROCESS_INFORMATION pi;
  if (!*cmd) return -1;
  if ((what==0) || (what==2))
  {
    if (pid>0) stop_prog(pi);
    pid=-1;
  }
  if ((what==0) || (what==1))
  {
    pid=spawn_prog(cmd,&pi);
  }

  return pid;
}
#endif

