/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
#define NWCMD
/*******************************************************************
 * Main function XRIT2PIC
 ********************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "xrit2pic.h"
#include "avhrr.h"

#ifndef RELEASE
#define RELEASE "Undefined"
#endif

PREFER prefer;
GLOBINFO globinfo;
GROUP *globgrp;
XRIT_DBASE dbase;

static void set_preoptions(GENMODES *genmode)
{
  int i;
  char *p;
  strncpy(globinfo.pl.src_dir,prefer.pl.src_dir,300);
  finish_path(globinfo.pl.src_dir);
  strncpy(globinfo.dest_dir,prefer.dest_dir,300);
  finish_path(globinfo.dest_dir);
  related_dirs(&globinfo,&prefer);
  strncpy(globinfo.lut.dir,prefer.lut_dir,300);
  finish_path(globinfo.lut.dir);
  if (!*globinfo.lut.fname)
    strncpy(globinfo.lut.fname,prefer.lut_file,300);

  strncpy(globinfo.lut.name,globinfo.lut.fname,300);
  if ((p=strchr(globinfo.lut.name,'.'))) *p=0;

  // get sat info
  globinfo.satlist=create_list_of_sats();

  // Set some defaults
  strcpy(globinfo.releasenr,RELEASE);
  strcpy(globinfo.bdate,__DATE__);
  strcpy(globinfo.btime,__TIME__);
  globinfo.bitshift=1;
  globinfo.shift_3d=MAX_3DSHIFT;

//  genmode->overwrite_mode=SKIP;
  globinfo.overwrite_mode=OVRWR_ASK;
  globinfo.avhrr_lin=TRUE;
  globinfo.correct_aspect=TRUE; // currently fixed!

  // Copy preferences in 'prefer' into globinfo and some into genmode
  globinfo.pr=prefer.pr;
  globinfo.pm=prefer.pm;

  if (exist_file_in_dir(prefer.prog_dir,BZIPPROG))
  {
    snprintf(globinfo.prog_bzip2,299,"\"%s%c%s\"",prefer.prog_dir,DIR_SEPARATOR,BZIPPROG);
  }
  else
  {
    strcpy(globinfo.prog_bzip2,BZIPPROG);
  }
  if (exist_file_in_dir(prefer.prog_dir,WGETPROG))
  {
    snprintf(globinfo.prog_wget,299,"\"%s%c%s\"",prefer.prog_dir,DIR_SEPARATOR,WGETPROG);
  }
  else
  {
    strcpy(globinfo.prog_bzip2,BZIPPROG);
    strcpy(globinfo.prog_wget,WGETPROG);
  }

  globinfo.invert_ir=prefer.invert_ir;


  for (i=0; i<=NRANNO; i++)
  {
    genmode->spm.ol.annotxt[i]=globinfo.pm.ol.annotxt[i];
    strcpy(genmode->spm.ol.annotxt[i].str,globinfo.pm.ol.annotxt[i].str); // nodig als later dyn. alloc. str!
  }

  // copy only string part of annotxt from pref.
  // enable and size have their own prefs from record tab (string cannot be defined in record tab)
  strcpy(globinfo.pr.lv_ol.annotxt[1].str,prefer.annotxt[1].str);
  strcpy(globinfo.pr.lv_ol.annotxt[2].str,prefer.annotxt[2].str);

  globinfo.jpeg_quality=prefer.jpeg_quality;
  globinfo.movie_quality=prefer.movie_quality;
}


int main(int argc,char **argv)
{
  GENMODES genmode;   // genmode for non-gui

  memset(&prefer,0,sizeof(prefer));
  memset(&genmode,0,sizeof(genmode));
  memset(&globinfo,0,sizeof(globinfo));
  memset(&dbase,0,sizeof(dbase));
  genmode.test=FALSE;

  // Get locations, preferences file; load preferences
  get_dirs(argv[0],&prefer.cur_dir,&prefer.prog_dir,&prefer.prog_xdir,&prefer.home_dir);
  search_file(PREFFILE,prefer.used_preffile,
              prefer.cur_dir,prefer.home_dir,prefer.prog_dir);
  printf("Using preferences '%s'...\n",prefer.used_preffile);

  Read_Pref(NULL,&prefer);         // load first defaults, then preferences in 'prefer'
  set_preoptions(&genmode);        // settings from prefer to globinfo and gemode
  do_options(argc,argv,&genmode);  // Get options; saved in globinfo and genmode
  related_dirs(&globinfo,&prefer);

  if (globinfo.debug) dump_dbase(globgrp);

  if (globinfo.lut.use)
  {
    if (!exist_file_in_dir(prefer.lut_dir,globinfo.lut.fname))
    {
      printf("Error: can't find lut file '%s' in '%s'\n",globinfo.lut.fname,prefer.lut_dir);
      return 1;
    }
  }

#ifndef __NOGUI__
  if (globinfo.gui_mode)
  {
    if (globinfo.log)
      globinfo.fplog=fopen("xrit2pic.log","w");
    create_gui(argc,argv);
    if (globinfo.fplog) fclose(globinfo.fplog); globinfo.fplog=NULL;
    return 0;
  }
  else
#endif
  {
    if (globinfo.overwrite_mode==OVRWR_ASK) globinfo.overwrite_mode=OVRWR_SKIP; // do NOT ask in non-gui mode
    if (!(nogui(&genmode,&prefer,genmode.test,globinfo.log)))
    {
      fprintf(stderr,"ERROR: Can't read XRIT source directory '%s'\n",globinfo.pl.src_dir);
      return 1;
    }
  }

  return 0;
}
