/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * Header XRIT2PIC
 ********************************************************************/
#define TEST_MEMn
#ifdef TEST_MEM
  #ifndef malloc
    #define malloc tmalloc
  #endif
  #ifndef realloc
    #define realloc trealloc
  #endif
  #ifndef calloc
    #define calloc tcalloc
  #endif
  #ifndef free
    #define free tfree
  #endif
#endif

#define DEBUG_STR(s) if (gdebug) puts(s);
extern int gdebug;

#ifdef __NOGUI__
  #include "xrit2pic_nogtk.h"
  #include <stdio.h>
#else
  #include "sgtk.h"
#endif
#include "vcdu.h"
#include "vcdu_funcdef.h"

#include "xrit2pic_types.h"

#define MEMALLOCDBGn
#ifdef MEMALLOCDBG
#define malloc(a) dbg_malloc(a)
#define calloc(a,b) dbg_calloc(a,b)
#define free(a) dbg_free(a)
#endif

/* Files preferences */
#define PREFFILE "xrit2pic.ini"
#define GPREFFILE "xrit2pic.gini"

/* Determine if/when warnings are given */
#define PARANOIDE 1

#define IFRMT2TEXT(f) (f=='j'? "JPEG" : \
                       f=='w'? "WVT"  : \
                               "Unknown")
#define RCHAR(l) l[strlen(l)-1]
#define rewind_s(str) while ((str) && (str->prev)) str=str->prev
#define wind_s(str) while ((str) && (str->next)) str=str->next
#define R2D(a) ((a)*180./3.14159)
#define SIGN(a) ((a)>0? 1 : (a)<0? -1 : 0)

#define NINT(i) ((i) < 0? (int)((i)-0.5) : (int)((i)+0.5))

#if __GTK_WIN32__ == 1
  #define BZIPPROG "bzip2.exe"
  #define WGETPROG "wget.exe"
#else
  #define BZIPPROG "bzip2"
  #define WGETPROG "wget"
#endif

#define LAB_HOLDCHNK "Minimize"

/* Defaults external programs: viewer and mencoder */
#define LIN_VIEWER_PGM "wsat -w 500 -h 500 -i"
#define LIN_SNOPT_PGM "-sn"
#define LIN_VIEWER_JPG "xv"
#define LIN_SNOPT_JPG "-rotate 180"
#define LIN_PLAY_MOVIE "mplayer"
#define LIN_GEN_MOVIE "mencoder mf://%s%s -o %s%s -mf type=jpeg -ovc lavc -lavcopts vcodec=msmpeg4v2 -mf fps=1 > /dev/null"
#define LIN_BUFR_EXTR "Bufr2Asc"
#define LIN_BUFRTBL_LOC "$HOME/bin/bufr_tables/"
//#define LIN_OPTSPEED "-mf fps=1"

#define WIN_VIEWER_PGM "i_view32"
#define WIN_SNOPT_PGM "/rotate_r /rotate_r"
#define WIN_VIEWER_JPG "i_view32"
#define WIN_SNOPT_JPG "/rotate_r /rotate_r"
#define WIN_PLAY_MOVIE_PATH "c:\\Program Files\\MPlayer for Windows"
#define WIN_PLAY_MOVIE "mplayer.exe"
#define WIN_GEN_MOVIE "mencoder mf://%s%s -o %s%s -mf type=jpeg -ovc lavc -lavcopts vcodec=msmpeg4v2 -mf fps=1"
#define WIN_BUFR_EXTR "Bufr2Asc"
#define WIN_BUFRTBL_LOC "c:\\bufr_tables\\"  // NO spaces allowed!
//#define WIN_OPTSPEED "-mf fps=1"


#define REMOVEWINDOW "Channels to keep"
#define LAB_VIS   "!Show in list"

#define LAB_PREVIEW "Preview"
#define LAB_MAKE "Export "

#define nrchanlist 12
#define nrsatlist 22

#define MAXCHAN 25
#define MAX_3DSHIFT 50
#define DEF_3DSHIFT 20

#define MAIN_TAB "!main_tab"
#define RECORD_TAB "!record_tab"

#define MENU_VIEW "/View"
#define MENU_LONLAT "View/lon_lat"
#define MENU_CITIES "View/cities"
#define MENU_MARK "View/mark"
#define MENU_EVIEW "View/Exported"
#define MENU_3DIM "View/Anaglyph"
#define MENU_FIRE "View/Fires"

#define MENU_IRNORM "View/IR Raw"
#define MENU_TEMPG  "View/IR Temp_BW"
#define MENU_TEMPC  "View/IR Temp_CLR"
#define MENU_LUT    "View/IR Lut"

#define MENU_SLUT "View/Select Lut"

#define MENU_GMAP  "/Projection "
#define MENU_AVLIN  "Projection/AVHRR lin"
#define MENU_GEOGR  "/Projection/proj"
#define MENU_GEOGR_NR  "Projection/proj/normal"
#define MENU_GEOGR_PC  "Projection/proj/Plate Carree"
#define MENU_GEOGR_MT  "Projection/proj/Mercator"
#define MENU_GEOGR_PN  "Projection/proj/Polar North"
#define MENU_GEOGR_PS  "Projection/proj/Polar South"

#define LAB_SEGMFRST "Start:"
#define LAB_SEGMNR "Nbr.:"
#define LAB_BITSHFT "!Bitshift"

#define OOR -1000

#define forall_items(s,s1) for (s1=s; s1; s1=s1->next)

#define __Create_Linked_Struct__(snw,sc)     \
{                                         \
  snw=calloc(1,sizeof(*snw));             \
  if (!snw)                               \
  {                                       \
    fprintf(stderr,"FATAL ERROR: Out of memory! (Trying allocating %d bytes...)\n",sizeof(*snw)); \
    exit(-1);                             \
  }                                       \
  if (!*s)                                \
  {                                       \
    *s=snw;                               \
  }                                       \
  else                                    \
  {                                       \
    for (sc=(*s); sc->next; sc=sc->next); \
    sc->next=snw;                         \
    snw->prev=sc;                         \
  }                                       \
}



#define PI 3.1415926
#define D2R(g) ((g)*PI/180.)
#define IS_POLAR(g) (g->orbittype==POLAR)
#define IS_GEO(g)   (g->orbittype==GEO)


#include "xrit2pic_funcs.h"
