/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/

#define PREFFILE "xrit2pic.ini"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include "xrit2pic_types.h"
#include "xrit2pic_args.h"
#include "xrit2pic_dbasefuncs.h"
#include "xrit2pic_funcs_nogtk.h"
#include "avhrr.h"


extern GLOBINFO globinfo;
extern PREFER prefer;
#define __Create_Linked_Struct2__(s,snw,sc) \
  snw=calloc(1,sizeof(*snw));              \
  snw->refcount++;                         \
  if (!snw)                                \
  { \
    fprintf(stderr,"FATAL ERROR: Out of memory! \n"); \
    exit(-1); \
  } \
  if (s)                                    \
  {                                         \
    if (!*s)                                \
    {                                       \
      *s=snw;                               \
    }                                       \
    else                                    \
    {                                       \
      for (sc=(*s); sc->next; sc=sc->next); \
      sc->next=snw;                         \
      snw->prev=sc;                         \
    }                                       \
  }

static ARG *Create_Arg(ARG **s,char *name,char *opt,char *frmt1,char *frmt2,ARGCODE code,char *help,...)
{
  ARG *snw,*sc;
  __Create_Linked_Struct2__(s,snw,sc);

  snw->name=name;
  snw->opt=opt;
  snw->frmt1=frmt1;
  snw->frmt2=frmt2;
  snw->code=code;
  snw->help=help;

  // collect complete help string
  {
    int len;
    char *s2;
    va_list ap;
    va_start(ap,help);
    len=vsnprintf(NULL,0,help,ap);
    va_end(ap);
    if ((s2=malloc(len+1)))
    {
      va_start(ap,help);
      len=vsprintf(s2,help,ap);
      va_end(ap);
      snw->help=s2;
    } // else allocation problem
  }
  return snw;
}

static void Remove_SetArgs(SETARG *sa)
{
  SETARG *sanext;
  ARG *arg,*argnext;
  for (; sa; sa=sanext)
  {
    sanext=sa->next;
    for (arg=sa->arg; arg; arg=argnext)
    {
      argnext=arg->next;
      free(arg);
    }
    free(sa);
  }
}

static SETARG *Create_SetArg(SETARG **s,char *name)
{
  SETARG *snw,*sc;
  __Create_Linked_Struct2__(s,snw,sc);

  snw->name=name;
  return snw;
}

typedef struct optionlist
{
  struct optionlist *prev,*next;
  int type;
  char *name;
  gboolean force_msb7;
  int refcount;
} OPTION_LIST;


static OPTION_LIST *Create_Optionlist(OPTION_LIST **s,char *name,int type,int fmsb7)
{
  OPTION_LIST *snw,*sc;
  __Create_Linked_Struct2__(s,snw,sc);

  snw->name=name;
  snw->type=type;
  snw->force_msb7=fmsb7;
  return snw;
}

static void Remove_Optionlist(OPTION_LIST *s)
{
  OPTION_LIST *s1,*s1next;
  for (s1=s; s1; s1=s1next)
  {
    s1next=s1->next;
    free(s1);
  }
}

static void usage(SETARG *sa,char *item)
{
  FILE *fp=stdout;
  SETARG *sa1;
  ARG *arg;
  fprintf(fp," +------------------------------------------------+\n");
  fprintf(fp," |  xrit2pic release %s                      |\n",globinfo.releasenr);
  fprintf(fp," | Translate XRIT/AVHRR files into pictures.      |\n");
  fprintf(fp," | R. Alblas                                      |\n");
  fprintf(fp," | werkgroep Kunstmanen                           |\n");
  fprintf(fp," +------------------------------------------------+\n");
  fprintf(fp,"\n");
  fprintf(fp," xrit2pic [options]\n");
  fprintf(fp,"  Options:\n");
  for (sa1=sa; sa1; sa1=sa1->next)                    // all main items
  {
    if ((!item) && (sa1->name)) printf("  %s\n",sa1->name);
    for (arg=sa1->arg; arg; arg=arg->next)            // all sub-items (actual possible options)
    {
      if ((!item) || (!strncmp(item,arg->name,strlen(arg->name))))
      {
        if (*arg->help != '!')
        {
          char str[100],*p;
          snprintf(str,99,"%s%s",arg->name,(arg->opt? arg->opt : ""));
          
          p=strtok(arg->help,"\n");
          while (p)
          {
            printf("    %-35s %s\n",str,p);
            *str=0;
            p=strtok(NULL,"\n");
          }
        }
      }
    }
  }
  exit(0);
}

static void examples()
{
  FILE *fp=stdout;
  fprintf(fp,"------------------------------------------------\n");
  fprintf(fp,"Example MSG:\n");
  fprintf(fp,"   xrit2pic -nogui -type H -chan VIS* -segm 3-7 -date 13-02-* -time 21:00 \n");
  fprintf(fp,"Example MSG colour:\n");
  fprintf(fp,"   xrit2pic -nogui -type H -compose\n");
  fprintf(fp,"   xrit2pic -nogui -type H -r VIS006=90 -r VIS008=-90 -go 10 -g VIS006 -b IR_016\n");
  fprintf(fp,"------------------------------------------------\n");
  fprintf(fp,"Example AVHRR: translate all avhrr files into .rah format \n");
  fprintf(fp,"  xrit2pic -nogui -type A -rah \n");
  fprintf(fp,"Example AVHRR in colour \n");
  fprintf(fp,"  xrit2pic -nogui -type A -colour \n");
  fprintf(fp,"  xrit2pic -nogui -type A -r %s -g %s -b %s \n",A_CH1,A_CH2,A_CH4);
  fprintf(fp,"------------------------------------------------\n");
  fprintf(fp,"Example overlay options: \n");
  fprintf(fp,"  xrit2pic -nogui -ol \n");
  fprintf(fp,"  xrit2pic -nogui -ol=80 \n");
  fprintf(fp,"  xrit2pic -nogui -ol=80 coast\n");
  fprintf(fp,"------------------------------------------------\n");
  fprintf(fp,"Example Anaglyph options: \n");
  fprintf(fp,"  xrit2pic -nogui -anagl \n");
  fprintf(fp,"  xrit2pic -nogui -anagl 10 \n");
  fprintf(fp,"  xrit2pic -nogui -anagl 10:20 \n");
  fprintf(fp,"  xrit2pic -nogui -anagl 10:20-200 \n");
  fprintf(fp,"------------------------------------------------\n");
  fprintf(fp,"Example MSG complex compose:\n");
  fprintf(fp,"  xrit2pic -nogui -type H -compose dust\n");
  fprintf(fp,"\n");
  fprintf(fp,"Example MSG complex compose, custom: (result is same as 'dust' mapping)\n");
  fprintf(fp,"  xrit2pic -nogui -type H -r [IR_120,IR_108,-4,2,1.] -g [IR_108,IR_087,0,15,2.5] -b [IR_108,,261,289,1]\n");
  fprintf(fp,"Calculation [chanp,chann,t1,t2,g]:\n");
  fprintf(fp,"  (chanp-chann)=t1 ==> pixvalue=0\n");
  fprintf(fp,"  (chanp-chann)=t2 ==> pixvalue=255\n");
  fprintf(fp,"  t1 < (chanp-chann) < t2 ==> use gamma 'g'\n");
  fprintf(fp,"------------------------------------------------\n");
  exit(0);
}

static int get_str_is_dig(char *opt,char *str,int slen,int *v)
{
  char *p;
  *v=0;
  strncpy(str,opt,slen);
  if ((p=strchr(str,'=')))
  {
    *p=0;
    *v=atoi(p+1);
    return 2;
  }
  return 1;
}

// opt=[stra,strb,1.,2.,3.]
// 
static int get_dbl_str(char *opt,char *str1,char *str2,int slen,int *a,int *b,float *c)
{
  int n;
  char *p;
  *str1=*str2=0;
  *a=*b=*c=0.;
  if (opt[0]!='[') return 0;
  strncpy(str1,opt+1,slen);

  if (!(p=strchr(str1,','))) return 1;
  *p=0;
  strncpy(str2,p+1,slen);  
  if (!(p=strchr(str2,','))) return 2;
  *p=0;
  n=sscanf(p+1,"%d,%d,%f",a,b,c);
  return n+2;
}

static char *get_str(char *opt,char *str,int slen,char sep)
{
  char *p;
  strncpy(str,opt,slen);
  if ((p=strchr(str,sep)))
  {
    *p=0;
  }
  if ((p=strchr(opt,sep))) return p;
  return NULL;
}

#ifndef __NOGUI__
static void genini()
{
  if (!exist_file(PREFFILE))
  {
    Save_Pref1(&prefer,PREFFILE);
    printf("Generated ini-file '%s'.\n",PREFFILE);
  }
  else
  {
    printf("%s already exists.\n",PREFFILE);
  }
}
#endif

// Define list of special clr-mappings
static OPTION_LIST *Create_Comptypes()
{
  OPTION_LIST *ct=NULL;
  Create_Optionlist(&ct,"vis_dn",map_vis_hmsgdn,FALSE);
  Create_Optionlist(&ct,"vis",map_vis_hmsg,FALSE);
  Create_Optionlist(&ct,"nrgb_dn",map_vis_hmsgdn,FALSE);
  Create_Optionlist(&ct,"nrgb",map_vis_hmsg,FALSE);
  Create_Optionlist(&ct,"noaavis",map_noaavis,FALSE);
  Create_Optionlist(&ct,"metopvis",map_metopvis,FALSE);
  Create_Optionlist(&ct,"airmass",map_airm,TRUE);
  Create_Optionlist(&ct,"eview",map_eview,TRUE);
  Create_Optionlist(&ct,"dust",map_dust,TRUE);
  Create_Optionlist(&ct,"fog",map_nfog,TRUE);
  Create_Optionlist(&ct,"nfog",map_nfog,TRUE);
  Create_Optionlist(&ct,"dfog",map_dfog,TRUE);
  Create_Optionlist(&ct,"uph_cloud",map_ucld,TRUE);
  Create_Optionlist(&ct,"uph_dust",map_udust,TRUE);
  Create_Optionlist(&ct,"uph_ash",map_uash,TRUE);
  Create_Optionlist(&ct,"uph_night",map_unight,TRUE);
  Create_Optionlist(&ct,"uph_day",map_uday,TRUE);
  Create_Optionlist(&ct,"conv_storm",map_cnv_strm,TRUE);
  Create_Optionlist(&ct,"snow_fog",map_snowfog,TRUE);
  return ct;
}

static OPTION_LIST *Create_Projectionlist()
{
  OPTION_LIST *ct=NULL;
  Create_Optionlist(&ct,"plate_carree",plate_carree,FALSE);
  Create_Optionlist(&ct,"mercator",mercator,FALSE);
  Create_Optionlist(&ct,"polar_n",polar_n,FALSE);
  Create_Optionlist(&ct,"polar_s",polar_s,FALSE);
  Create_Optionlist(&ct,"normal",normal,FALSE);
  return ct;
}

static int do_options_misc(ARG *arg,SETARG *sa)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_help:
      usage(sa,NULL);
      ok=1;
    break;
    case opt_example:
      examples();
      ok=1;
    break;
    case opt_nogui:
      globinfo.gui_mode=FALSE;
      ok=1;
    break;
    case opt_debug:
      globinfo.debug=TRUE;
      ok=1;
    break;
    case opt_version:
      fprintf(stdout,"Release %s\n",globinfo.releasenr);
      fprintf(stdout,"Build at %s, %s\n",globinfo.bdate,globinfo.btime);
    exit(0);
      ok=1;
    break;
#ifndef __NOGUI__
    case opt_genini:
      genini();
      exit(0);
      ok=1;
    break;
#endif
    case opt_olfiles:
      pri_overlayfiles(stdout,prefer.overlay,(void *)fprintf);
      exit(0);
      ok=1;
    break;
    case opt_safemode:
      globinfo.safemode=TRUE;
      ok=1;
    break;
    default:
    break;
  }
  return ok;
}

// These options only for nogui; no equivalents for 'cformat' in globinfo
static int do_options_formats(ARG *arg,char *opt1,GENMODES *genmode)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_of_pgm:  
      genmode->cformat=PIC_PGM;
      ok=1;
    break;
    case opt_of_pgm8:  
      genmode->cformat=PIC_PGM8;
      ok=1;
    break;
    case opt_of_jpg:
      if (opt1)
      {
        int q=0;
        sscanf(opt1,arg->frmt1,&q);
        globinfo.jpeg_quality=MIN(MAX(q,0),100);
      }
      genmode->cformat=PIC_JPEG;
      ok=1;
    break;
    case opt_of_cjpg:  
      genmode->cformat=PIC_CJPEG;
      ok=1;
    break;
    case opt_of_rah:
      genmode->cformat=PIC_RAH;
      ok=1;
    break;
    case opt_of_avi:  
      genmode->skip_incomplete=TRUE;
      genmode->gen_film=TRUE;
      ok=1;
    break;
    case opt_of_avi_kp:  
      genmode->skip_incomplete=TRUE;
      genmode->gen_film=TRUE;
      globinfo.filmframe_ovrwrmode=TMP_REPLACE;
      ok=1;
    break;
    default:
    break;
  }
  return ok;
}

// genmode set for: adapt_lum,lmin,lmax,gamma
static int do_options_clrmap(ARG *arg,char *opt2,GENMODES *genmode)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_compose:
      // hier e.e.a. aanpassen!
      globinfo.spm.compose=TRUE;
      globinfo.spm.compose_type=map_vis_hmsg;
genmode->spm.compose=TRUE;
      globinfo.pm.clr.compose=TRUE;
//      globinfo.pm.compose_type=map_vis_hmsg;

      ok=1;
      if (opt2)
      {
        OPTION_LIST *ct1,*ct=Create_Comptypes();
        globinfo.spm.compose_type=map_unk;
        for (ct1=ct; ct1; ct1=ct1->next)
        {
          if (!strcmp(opt2,ct1->name))
          {
            globinfo.spm.compose_type=ct1->type;
            if (ct1->force_msb7) globinfo.bitshift=0;
          }
        }
        if (globinfo.spm.compose_type==map_unk)
        {
          ok=0;
          fprintf(stderr,"Error: Unknown compose type '%s'; use:\n",opt2);
          for (ct1=ct; ct1; ct1=ct1->next)
          {
            fprintf(stderr,"  %s\n",ct1->name);
          }
        }
        Remove_Optionlist(ct);
      }
    break;
    case opt_temp:
      globinfo.spm.map_temp=TRUE;   
      ok=1;
    break;
    case opt_temp_bw:
      globinfo.spm.map_temp_G_mA=TRUE; globinfo.bitshift=0;
      ok=1;
    break;
    case opt_hrvlum:
      globinfo.hrv_lum=TRUE;
      ok=1;
    break;
    case opt_inv:
      if (!opt2)
      {
        globinfo.pm.clr.invert=TRUE;
      }
      else
      {
        CHANMAP *cm;
        if (!(cm=Get_Chanmap(globinfo.pixmap.chanmap,opt2)))
          cm=Create_Chanmap(&globinfo.pixmap.chanmap,opt2,0.,0.,0.,FALSE); 
        cm->invert=TRUE;
      }
      ok=1;
    break;
    case opt_msb:
      if (opt2)
      {
        int a=0,n;
        n=sscanf(opt2,arg->frmt2,&a);
        if ((n==1) && (a>=7) && (a<=9))
        {
          globinfo.bitshift=a-7;    // 2,1 or 0
          ok=1;
        }
        else
        {
          fprintf(stderr,"Error: Wrong msb %s; use 9, 8 or 7\n",opt2);
        }
      }
    break;

    case opt_lmin:
      if (opt2)
      {
        genmode->adapt_lum=TRUE;
        genmode->lmin=atoi(opt2);
        if (!genmode->lmax) genmode->lmax=511;
        ok=1;
      }
    break;
    case opt_lmax:  
      if (opt2)
      {
        genmode->adapt_lum=TRUE;
        genmode->lmax=atoi(opt2);
        ok=1;
      }
    break;
    case opt_gamma:
      if (opt2)
      {
        genmode->adapt_lum=TRUE;
        genmode->gamma=atof(opt2);
        ok=1;
      }
   break;
    case opt_redoffs:
      if (opt2)
      {
        globinfo.pixmap.offset[0]=atoi(opt2);
        ok=1;
      }
    break;
    case opt_grnoffs:
      if (opt2)
      {
        globinfo.pixmap.offset[1]=atoi(opt2);
        ok=1;
      }
    break;
    case opt_bluoffs:
      if (opt2)
      {
        globinfo.pixmap.offset[2]=atoi(opt2);
        ok=1;
      }
    break;
    case opt_red: case opt_green: case opt_blue:
      if (opt2)
      { 
        int n;
        char str[2][30];
        if (globinfo.pixmap.compose_type!=map_cust)
        {
          globinfo.spm.compose=TRUE;
          globinfo.spm.compose_type=map_cust;
          Remove_Chanmap(&globinfo.pixmap.chanmap);
          globinfo.pixmap.compose_type=map_cust;
          genmode->spm.compose=TRUE;
        }

        RGBMAPS *rgbm=(arg->code==opt_red?   &globinfo.pixmap.clrmapspec.r:
                       arg->code==opt_green? &globinfo.pixmap.clrmapspec.g:
                                             &globinfo.pixmap.clrmapspec.b);

        n=get_dbl_str(opt2,str[0],str[1],30,&rgbm->valmin,&rgbm->valmax,&rgbm->gamma);
        if (n>1)
        {
          strncpy(rgbm->chanp,str[0],9);
          strncpy(rgbm->chann,str[1],9);
          if (!globinfo.pixmap.clrmapspec.b.gamma) globinfo.pixmap.clrmapspec.b.gamma=1.;
          globinfo.pixmap.clrmapspec.use_temp=TRUE;
          strncpy(globinfo.pixmap.clrmapspec.name,"custom",20);
          strncpy(globinfo.pixmap.clrmapspec.name_sh,"cust",20);
          globinfo.spm.compose_type=map_cust1;
          ok=1;
        }
        else
        {
          CHANMAP *cm;
          int v;
          n=get_str_is_dig(opt2,str[0],30,&v);
          if (!(cm=Get_Chanmap(globinfo.pixmap.chanmap,str[0])))
            cm=Create_Chanmap(&globinfo.pixmap.chanmap,str[0],0.,0.,0.,FALSE); 
          if (n==1) v=100;
          switch(arg->code)
          {
            case opt_red:   cm->r=(float)v/100.; break;
            case opt_green: cm->g=(float)v/100.; break;
            case opt_blue:  cm->b=(float)v/100.; break;
            default: break;
          }
          ok=1;
        }
      }
      else
      {
        break;
      }
    break;
    case opt_lut:
      if (opt2) // opt2=NULL, {%d}, %s, %s{%d}
      { 
        int a=0;
        char str[30],*p1,*p2;
        if ((p1=get_str(opt2,str,30,'{')))
        {
          if ((p2=strchr(arg->frmt2,'{')))
          {
            sscanf(p1,p2,&a);   // {<colnr>}
            globinfo.lut.sellut=a;
          }
          else break;
        }

        strncpy(globinfo.lut.fname,str,30);
        globinfo.lut.use=TRUE;
        ok=1;
      }
      else
      {
        globinfo.lut.use=TRUE;
        ok=1;   // def. lut without column nr
      }
    break;
    default:
    break;
  }
  return ok;
}

static int do_options_geomap(ARG *arg,char *opt2)
{
  int ok=0;
  int i;
  switch(arg->code)
  {
    case opt_nolin:
      globinfo.avhrr_lin=FALSE;  //
      ok=1;
    break;
    case opt_europe:
      globinfo.area_nr=1;
      ok=1;
    break;
    case opt_area:
      if (opt2)
      {
        if (isdigit(*opt2))
        {
          globinfo.area_nr=atoi(opt2);
          ok=1;
        }
        else
        {
          for (i=0; i<NRGEODEFS; i++)
          {
            if (!strcmp(opt2,prefer.geoarea[i].part_name))
            {
              globinfo.area_nr=i+1;
              ok=1;
              break;
            }
          }
          if (i==NRGEODEFS)
          { // may give 2x error emssage because -area 2x defined:
            //  for nr or name
            printf("ERROR: No area '%s'. Defined areas:\n",opt2);
            for (i=0; i<NRGEODEFS; i++)
            {
              if (*prefer.geoarea[i].part_name)
                printf("  %s\n",prefer.geoarea[i].part_name);
            }
          }
        }
      }
    break;
    case opt_projection:
      if (opt2)
      {
        OPTION_LIST *pl=Create_Projectionlist(),*pl1;
        for (pl1=pl; pl1; pl1=pl1->next)
        {
          if (!strcmp(opt2,pl1->name))
          {
            globinfo.gmap=pl1->type;
            break;
          }
        }
        if (!pl1)
        {
          printf("ERROR: %s%s%s\n       %s\n","projection '",opt2,"' unknown","Supported:");
          for (pl1=pl; pl1; pl1=pl1->next) printf("         %s\n",pl1->name);
        }

          
        Remove_Optionlist(pl);
        ok=1;
      }
    break;
    default:
    break;
  }
  return ok;
}

// genmode set for: ofile,gen_pro,gen_epi,skip_incomplete,test
static int do_options_outputs(ARG *arg,char *opt2,GENMODES *genmode)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_ofile:
      if (opt2)
      {
        strcpyd(&genmode->ofile,opt2);
        ok=1;
      }
    break;
    case opt_genprol:
      genmode->gen_pro=TRUE;
      ok=1;
    break;
    case opt_genepil:
      genmode->gen_epi=TRUE;
      ok=1;
    break;
    case opt_force:
      globinfo.overwrite_mode=OVRWR_REPLACE;
      ok=1;
    break;

    case opt_skipincompl:
      genmode->skip_incomplete=TRUE;
      ok=1;
    break;
    case opt_test:
      genmode->test=TRUE;  
      ok=1;
    break;
    case opt_log:
      globinfo.log=TRUE;
      ok=1;
    break;
    default:
    break;
  }
  return ok;
}

// genmode set for: agl
static int do_options_overlays(ARG *arg,char *opt1,char *opt2,GENMODES *genmode)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_ltimestamp:    // -ltimestamp or -ltimestamp@[<x>,<y>]<s>
      globinfo.pm.ol.annotxt[0].xflag=TRUE;
    case opt_timestamp:     // -timestamp or -timestamp@[<x>,<y>]<s>
      if (opt2)
      { 
        break;
      } 
      if (opt1)
      {
        int a=0,b=0,c=0,n;
        n=sscanf(opt1,arg->frmt1,&a,&b,&c);
        globinfo.pm.ol.annotxt[0].x=a+1;
        globinfo.pm.ol.annotxt[0].y=b+1;
        if (n>2) globinfo.pm.ol.annotxt[0].size=c;
        globinfo.pm.ol.annotxt[0].enable=TRUE;
        ok=1;
      }
      else
      {
        globinfo.pm.ol.annotxt[0].enable=TRUE;
        ok=1;
      } 
    break;
    case opt_addtext: // -add_text <str> or -add_text@[<x>,<y>]<s> <str>
    {
      int m;
      for (m=1; m<NRANNO; m++)
      {
        if (!globinfo.pm.ol.annotxt[m].enable) break;  // alles vol
      }
      if (globinfo.pm.ol.annotxt[m].enable)
      {
        fprintf(stderr,"ERROR: Too much strings added. Max. is %d\n(Check also proeferences!)\n",NRANNO);
        return 1;
      }

      if (!opt2)
      { 
        break;
      } 
      if (opt1)
      {
        int a=0,b=0,c=0,n;
        n=sscanf(opt1,arg->frmt1,&a,&b,&c);
        printf("opt1: %d  %d  %d  %d  %s\n",n,a,b,c,opt2);
        globinfo.pm.ol.annotxt[m].x=a+1;
        globinfo.pm.ol.annotxt[m].y=b+1;
        if (n>2) globinfo.pm.ol.annotxt[m].size=c;
        ok=1;
      }
      else
      {
        ok=1;
      }
      strncpy(globinfo.pm.ol.annotxt[m].str,opt2,90);
      globinfo.pm.ol.annotxt[m].enable=TRUE;
    }
    break;
    case opt_anagl:
      globinfo.dim3=TRUE;
      genmode->agl.shift_3d=20;
      genmode->agl.lmin=0;
      genmode->agl.lmax=511;
      if (opt2)
      { 
        int a=0,b=0,c=0,n;
        n=sscanf(opt2,arg->frmt2,&a,&b,&c);
        if (n>=1) genmode->agl.shift_3d=a;
        if (n>=2) genmode->agl.lmin=b;
        if (n>=3) genmode->agl.lmax=c;
      }
      ok=1;
    break;

    case opt_olay:                       // -ol=10 coast
      globinfo.pm.ol.ol_lum=0xff;
      globinfo.pm.ol.overlaytype=country_p; // currently fixed in preferences: see prefer.c, "Cntry"
      if (opt2) // opt2="coast"
      {
        if (!strncmp(opt2,"coa",3))      globinfo.pm.ol.overlaytype=coast_p;
        else if (!strncmp(opt2,"cou",3)) globinfo.pm.ol.overlaytype=country_p;
        else if (!strncmp(opt2,"l",1))   globinfo.pm.ol.overlaytype=lonlat_p;
        else if (!strncmp(opt2,"b",1))   globinfo.pm.ol.overlaytype=both_p;
        else
        {
          fprintf(stderr,"Error: Wrong overlay type %s\n",opt2);
          return 1;
        }
      }
      if (opt1)
      {
        int a=0;
        if (!strncmp(opt1+1,"0x",2))
        {
          prefer.ovl_clr=strtol(opt1+1,NULL,16);
        }
        else
        {
          sscanf(opt1,arg->frmt1,&a);
          globinfo.pm.ol.ol_lum=a;
          globinfo.pm.ol.ol_lum=MIN(globinfo.pm.ol.ol_lum,255);
          globinfo.pm.ol.ol_lum=MAX(globinfo.pm.ol.ol_lum,0);
        }
      }
      globinfo.pm.ol.add_overlay=TRUE;
      ok=1;
    break;
    case opt_lonlat:   // opt1=NULL, =%d
      if (opt2)
      { 
        break;
      } 
      globinfo.pm.ol.ol_lum=0xff;
      if (opt1) // opt1=NULL, %s; opt1= NULL,=%d
      {
        int a=0;
        if (!strncmp(opt1+1,"0x",2))
        {
          prefer.lonlat_clr=strtol(opt1+1,NULL,16);
        }
        else
        {
          sscanf(opt1,arg->frmt1,&a);
          globinfo.pm.ol.ol_lum=a;
          globinfo.pm.ol.ol_lum=MIN(globinfo.pm.ol.ol_lum,255);
          globinfo.pm.ol.ol_lum=MAX(globinfo.pm.ol.ol_lum,0);
        }
      }
      globinfo.pm.ol.add_lonlat=TRUE;
      ok=1;
    break;
    case opt_mark:                            // Dit werkt nog niet! 
      if (opt2)
      { 
        break;
      } 
      globinfo.pm.ol.ol_lum=0xff;
      if (opt1) // opt1=NULL, %s; opt1= NULL,=%d
      {
        int a=0;
        if (!strncmp(opt1+1,"0x",2))
        {
          prefer.mark_clr=strtol(opt1+1,NULL,16);
        }
        else
        {
          sscanf(opt1,arg->frmt1,&a);
          globinfo.pm.ol.ol_lum=a;
          globinfo.pm.ol.ol_lum=MIN(globinfo.pm.ol.ol_lum,255);
          globinfo.pm.ol.ol_lum=MAX(globinfo.pm.ol.ol_lum,0);
        }
      }
      globinfo.pm.ol.add_marks=TRUE;
      ok=1;
    break;
    case opt_cities:                              // Dit werkt nog niet! 
      if (opt2)
      { 
        break;
      } 
      if (opt1) // opt1=NULL, %s; opt1= NULL,=%d
      {
        int a=0;
        if (!strncmp(opt1+1,"0x",2))
        {
          prefer.city_clr=strtol(opt1+1,NULL,16);
        }
        else
        {
          sscanf(opt1,arg->frmt1,&a);
          globinfo.pm.ol.ol_lum=a;
          globinfo.pm.ol.ol_lum=MIN(globinfo.pm.ol.ol_lum,255);
          globinfo.pm.ol.ol_lum=MAX(globinfo.pm.ol.ol_lum,0);
        }
      }
      globinfo.pm.ol.add_cities=TRUE;
      ok=1;
    break;
    default:
    break;
  }
  return ok;
}

// genmode set for: chan_name,segm_range,type,sat,date,time
static int do_options_select(ARG *arg,char *opt2,GENMODES *genmode)
{
  int ok=0;
  switch(arg->code)
  {
    case opt_locsrc:
      if (opt2)
      { 
        strncpy(prefer.pl.src_dir,opt2,300);
        strncpy(globinfo.pl.src_dir,prefer.pl.src_dir,300);
        finish_path(globinfo.pl.src_dir);
        ok=1;
      } 
    break;
    case opt_locdst:
      if (opt2)
      { 
        strncpy(prefer.dest_dir,opt2,300);
        strncpy(globinfo.dest_dir,prefer.dest_dir,300);
        finish_path(globinfo.dest_dir);
        ok=1;
      } 
    break;
    case opt_selchan:
      if (opt2)
      { 
        strcpyd(&genmode->chan_name,opt2);
        ok=1;
      } 
    break;
    case opt_selsegm:
      if (opt2)
      { 
        strcpyd(&genmode->segm_range,opt2);
        ok=1;
      } 
    break;
    case opt_seltype:
      if (opt2)
      { 
        genmode->type=opt2[0];
        ok=1;
      } 
    break;
    case opt_selsat:
      if (opt2)
      { 
        strcpyd(&genmode->sat,opt2);
        ok=1;
      } 
    break;
    case opt_seldate:
      if (opt2)
      { 
        strcpyd(&genmode->date,opt2);
        ok=1;
      } 
    break;
    case opt_seltime:
      if (opt2)
      { 
        char *p;
        strcpyd(&genmode->time,opt2);
        /* Allow '-' as separator instead of ':' (':' not always accepted as arg) */
        while ((p=strchr(genmode->time,'-'))) *p=':';
        ok=1;
      } 
    break;
    default:
    break;
  }
  return ok;
}

static void parse_cmdopt(SETARG *sa,int argc,char **argv,GENMODES *genmode)
{
  SETARG *sa1;
  ARG *arg;
  int i;
  // set defaults
  globinfo.gui_mode=TRUE;
  for (i=1; i<argc; i++)                                // all commandline args
  {
    int ok=0;
    if (argv[i][0]!='-') continue;
    ok=0;
    for (sa1=sa; sa1; sa1=sa1->next)                    // all main items
    {
      for (arg=sa1->arg; arg; arg=arg->next)            // all sub-items (actual possible options)
      {
        char *opt2=NULL,*opt1=NULL;
        if ((!strcmp(argv[i],arg->name)) || 
            ((arg->frmt1) && (!strncmp(argv[i],arg->name,strlen(arg->name))) && (argv[i][strlen(arg->name)]==*arg->frmt1)))
        {
          if (strcmp(argv[i],arg->name))
          {
            opt1=argv[i]+strlen(arg->name);
//            if (last_ch) opt1=strchr(argv[i],last_ch)+1;
          }
          if (arg->frmt2)
          {
            if (i+1>=argc)       { ok=-1; break; }// missing arg.
            if (*argv[i+1]=='-') { ok=-1; break; }// next arg. is unexpected option 
            opt2=argv[i+1];
          }
          else
          {
            if ((i+1<argc) && (*argv[i+1]!='-')) continue; // usage(sa,argv[i]);
          }
          ok=0;
          if (!ok) ok=do_options_misc(arg,sa);
          if (!ok) ok=do_options_formats(arg,opt1,genmode);
          if (!ok) ok=do_options_clrmap(arg,opt2,genmode);
          if (!ok) ok=do_options_geomap(arg,opt2);
          if (!ok) ok=do_options_outputs(arg,opt2,genmode);
          if (!ok) ok=do_options_overlays(arg,opt1,opt2,genmode);
          if (!ok) ok=do_options_select(arg,opt2,genmode);
          if (ok) break;
        }
      }
      if (ok<0) break;
    }
    if (ok==0)
    {
      fprintf(stderr,"Error: Wrong or incomplete option '%s' \n",argv[i]);
      exit(0);
    }
    if (ok<0)
    {
      fprintf(stderr,"Error: Missing option for %s \n",argv[i]);
      exit(0);
    }
  }     
}

/****************************************************
 * Define options
 ****************************************************/
static void options_misc(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,NULL);
  //                    option          shown_frmt                opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-h"            ,NULL                     ,NULL        ,NULL       ,opt_help       ,"this help.");
  Create_Arg(&sa1->arg,"-e"            ,NULL                     ,NULL        ,NULL       ,opt_example    ,"non-gui examples");
  Create_Arg(&sa1->arg,"-nogui"        ,NULL                     ,NULL        ,NULL       ,opt_nogui      ,"command line mode");
  Create_Arg(&sa1->arg,"-src"          ,NULL                     ,NULL        ,"%s"       ,opt_locsrc     ,"path to source XRIT files (def.: see ini-file).");
  Create_Arg(&sa1->arg,"-dest"         ,NULL                     ,NULL        ,"%s"       ,opt_locdst     ,"path to destination dir (def.: see ini-file).");

  Create_Arg(&sa1->arg,"-debug"        ,NULL                     ,NULL        ,NULL       ,opt_debug      ,"!debug mode");
  Create_Arg(&sa1->arg,"-v"            ,NULL                     ,NULL        ,NULL       ,opt_version    ,"Release info");
#ifndef __NOGUI__
  Create_Arg(&sa1->arg,"-gen_ini"      ,NULL                     ,NULL        ,NULL       ,opt_genini     ,"generate a xrit2pic.ini and quit");
#endif
  Create_Arg(&sa1->arg,"-safe_mode"      ,NULL                     ,NULL        ,NULL       ,opt_safemode     ,"don't use info of other files");
}

static void options_formats(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for file formats:");
  //                    option          shown_frmt                opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-pgm"           ,NULL                     ,NULL        ,NULL       ,opt_of_pgm     ,"generate PGM  (see below for defaults");
  Create_Arg(&sa1->arg,"-pgm8"          ,NULL                     ,NULL        ,NULL       ,opt_of_pgm8    ,"generate PGM (forced to 8 bits pp)");
  Create_Arg(&sa1->arg,"-jpg"           ,NULL                     ,NULL        ,NULL       ,opt_of_jpg     ,"generate JPEG (see below for defaults)");
  Create_Arg(&sa1->arg,"-jpg"           ,"=<q>"                   ,"=%d"       ,NULL       ,opt_of_jpg     ,"generate JPEG, q=quality (1...100 (def. 70))");
  Create_Arg(&sa1->arg,"-cjpg"          ,NULL                     ,NULL        ,NULL       ,opt_of_cjpg    ,"generate JPEG (no interm. decompression)");
  Create_Arg(&sa1->arg,"-rah"           ,NULL                     ,NULL        ,NULL       ,opt_of_rah     ,"generate rah from AVHRR");
  Create_Arg(&sa1->arg,"-movie"         ,NULL                     ,NULL        ,NULL       ,opt_of_avi     ,"generate movie");
  Create_Arg(&sa1->arg,"-movie_kp"      ,NULL                     ,NULL        ,NULL       ,opt_of_avi_kp  ,"generate movie, keep frame pictures");
}

#define TEXT1 " \"[<chp>,<chn>,<kmin>,<kmax>,<g>]\""
static void options_clrmap(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for rgb mapping:");
  //                    option      shown_frmt                opt vast     opt los            option code     help 
  Create_Arg(&sa1->arg,"-compose"  ,NULL                     ,NULL        ,NULL              ,opt_compose    ,"make false-color pics");
  Create_Arg(&sa1->arg,"-compose"  ," <type>"                ,NULL        ,"%s"              ,opt_compose    ,"same, use pre-defined type:\n  nrgb, nrgb_dn, airm, dust,\n  eview, nfog, dfog,\n  uph_cloud, uph_dust,uph_ash,\n  uph_night, uph_day,\n  conv_storm, snow_fog");
  Create_Arg(&sa1->arg,"-temp"     ,NULL                     ,NULL        ,NULL              ,opt_temp       ,"generate temperature mapping (IR only)");
  Create_Arg(&sa1->arg,"-temp_bw"  ,NULL                     ,NULL        ,NULL              ,opt_temp_bw    ,"same, GOES mode-A mapping (IR only)");
  Create_Arg(&sa1->arg,"-hrvlum"   ,NULL                     ,NULL        ,NULL              ,opt_hrvlum     ,"use HRV for lum (high-res colour)");
  Create_Arg(&sa1->arg,"-inv"      ,NULL                     ,NULL        ,NULL              ,opt_inv        ,"invert");
  Create_Arg(&sa1->arg,"-inv"      ," <chan>"                ,NULL        ,"%s"              ,opt_inv        ,"invert this channel");
  Create_Arg(&sa1->arg,"-msb"      ," 9,8 or 7"              ,NULL        ,"%d"              ,opt_msb        ,"msb for translation 10 bits->8 bits");
  Create_Arg(&sa1->arg,"-lmin"     ," <val>"                 ,NULL        ,"%d"              ,opt_lmin       ,"minval; this val mapped on 'black'");
  Create_Arg(&sa1->arg,"-lmax"     ," <val>"                 ,NULL        ,"%d"              ,opt_lmax       ,"maxval; this val mapped on 'white'");
  Create_Arg(&sa1->arg,"-gamma"    ," <val>"                 ,NULL        ,"%f"              ,opt_gamma      ,"gamma");
  Create_Arg(&sa1->arg,"-ro"       ," <offset>"              ,NULL        ,"%d"              ,opt_redoffs    ,"offset for red (def. 0)");
  Create_Arg(&sa1->arg,"-go"       ," <offset>"              ,NULL        ,"%d"              ,opt_grnoffs    ,"offset for geen (def. 0)");
  Create_Arg(&sa1->arg,"-bo"       ," <offset>"              ,NULL        ,"%d"              ,opt_bluoffs    ,"offset for blue (def. 0)");

  Create_Arg(&sa1->arg,"-r"        ,TEXT1                    ,NULL        ,"[%s,%s,%f,%f,%f]",opt_red        ,"complex compose on red:\n  <chp>-<chn>,\n  T-range 'kmin-kmax' mapped to 0...255\n  using 'gamma'");
  Create_Arg(&sa1->arg,"-r"        ," <channame>=<val>"      ,NULL        ,"%s=%d"           ,opt_red        ,"same, with val (-100...100)");
  Create_Arg(&sa1->arg,"-r"        ," <channame>"            ,NULL        ,"%s"              ,opt_red        ,"use this channel for red");

  Create_Arg(&sa1->arg,"-g"        ,TEXT1                    ,NULL        ,"[%s,%s,%f,%f,%f]",opt_green      ,"complex compose on green, see red");
  Create_Arg(&sa1->arg,"-g"        ," <channame>=<val>"      ,NULL        ,"%s=%d"           ,opt_green      ,"same, with val (-100...100");
  Create_Arg(&sa1->arg,"-g"        ," <channame>"            ,NULL        ,"%s"              ,opt_green      ,"use this channel for green");

  Create_Arg(&sa1->arg,"-b"        ,TEXT1                    ,NULL        ,"[%s,%s,%f,%f,%f]",opt_blue       ,"complex compose on blue, see red");
  Create_Arg(&sa1->arg,"-b"        ," <channame>=<val>"      ,NULL        ,"%s=%d"           ,opt_blue       ,"same, with val (-100...100");
  Create_Arg(&sa1->arg,"-b"        ," <channame>"            ,NULL        ,"%s"              ,opt_blue       ,"use this channel for blue ");

  Create_Arg(&sa1->arg,"-lut"      ,NULL                     ,NULL        ,NULL              ,opt_lut        ,"use def. lut ");
  Create_Arg(&sa1->arg,"-lut"      ," {n}"                   ,NULL        ,"{%d}"            ,opt_lut        ,"same, use n'th row");
  Create_Arg(&sa1->arg,"-lut"      ," <lutname>"             ,NULL        ,"%s"              ,opt_lut        ,"use this lut");
  Create_Arg(&sa1->arg,"-lut"      ," <lutname>{n}"          ,NULL        ,"%s{%d}"          ,opt_lut        ,"same, use n'th row");
}

static void options_geomap(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for geo mapping:");
  //                    option          shown_frmt                opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-nolin"         ,NULL                     ,NULL        ,NULL      ,opt_nolin       ,"don't linearize AVHRR");
  // test op "-eur*"
  Create_Arg(&sa1->arg,"-europe"        ,NULL                     ,NULL        ,NULL      ,opt_europe      ,"Europe part only (MSG)");
  Create_Arg(&sa1->arg,"-area"          ," <area_nr>"             ,NULL        ,"%s"      ,opt_area        ,"fixed areas defined in prefs tab Mapping\n  0=full, 1/2/3/4: see prefs(def.: 1=europe)");
  Create_Arg(&sa1->arg,"-area"          ," <area_name>"           ,NULL        ,"%s"      ,opt_area        ,"same, use  name instead of number.");
  Create_Arg(&sa1->arg,"-proj"          ," <projection>"          ,NULL        ,"%s"      ,opt_projection  ,"projection: plate_carree, mercator,\n            polar_n, polar_s,\n            normal (default)");
}


static void options_outputs(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for outputs to generate:");
  //                    option            shown_frmt                opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-o"            ," <file>"                  ,NULL        ,"%s"       ,opt_ofile      ,"name output file\n  see for defaults:\n    Preferences, tab 'Files', subtab 'Output'");
  Create_Arg(&sa1->arg,"-pro"            ,NULL                     ,NULL        ,NULL       ,opt_genprol    ,"extract also prologue file");
  Create_Arg(&sa1->arg,"-epi"            ,NULL                     ,NULL        ,NULL       ,opt_genepil    ,"extract also epilogue file");
  Create_Arg(&sa1->arg,"-overwrite"      ,NULL                     ,NULL        ,NULL       ,opt_force      ,"force overwrite");
  Create_Arg(&sa1->arg,"-f"              ,NULL                     ,NULL        ,NULL       ,opt_force      ,"same");
  Create_Arg(&sa1->arg,"-skip_incomplete",NULL                     ,NULL        ,NULL       ,opt_skipincompl,"don't process incomplete images");
  Create_Arg(&sa1->arg,"-test"           ,NULL                     ,NULL        ,NULL       ,opt_test       ,"don't generate, just show what will happen.");
  Create_Arg(&sa1->arg,"-log"            ,NULL                     ,NULL        ,NULL       ,opt_log        ,"generate log file 'xrit2pic.log' (in dest. dir)");
}

static void options_overlays(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for overlays:");
  //                    option          shown_frmt                 opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-overlay_files",NULL                      ,NULL         ,NULL       ,opt_olfiles    ,".");
  Create_Arg(&sa1->arg,"-anagl"        ,NULL                      ,NULL         ,NULL       ,opt_anagl      ,"generate anaglyph");
  Create_Arg(&sa1->arg,"-anagl"        ," <shift:lmin-lmax> "     ,NULL         ,"%d:%d-%d" ,opt_anagl      ,"same, shift and lum-range");
  Create_Arg(&sa1->arg,"-anagl"        ," <shift:lmin>"           ,NULL         ,"%d:%d"    ,opt_anagl      ,"same, shift and lum-start");
  Create_Arg(&sa1->arg,"-anagl"        ," <shift>"                ,NULL         ,"%d"       ,opt_anagl      ,"anaglyph, shift in pixels");
  Create_Arg(&sa1->arg,"-ol"           ,NULL                      ,NULL         ,NULL       ,opt_olay       ,"use def. overlay");
  Create_Arg(&sa1->arg,"-ol"           ," <type>"                 ,NULL         ,"%s"       ,opt_olay       ,"same, this type");
  Create_Arg(&sa1->arg,"-ol"           ,"=<lum>"                  ,"=%d"        ,NULL       ,opt_olay       ,"def., with lum: 0..255 (def. 255)\n  if hex notation: define rgb, e.g. 0x800");
  Create_Arg(&sa1->arg,"-ol"           ,"=<lum> <type>"           ,"=%d"        ,"%s"       ,opt_olay       ,"same, this type\ntype=coa(st),cou(ntry),b(oth)");
  Create_Arg(&sa1->arg,"-lonlat"       ,NULL                      ,NULL         ,NULL       ,opt_lonlat     ,"add lon/lat lines");
  Create_Arg(&sa1->arg,"-lonlat"       ,"=<lum>"                  ,"=%d"        ,NULL       ,opt_lonlat     ,"same, with lum; see -ol");
  Create_Arg(&sa1->arg,"-mark"         ,""                        ,NULL         ,NULL       ,opt_mark       ,"add marker");
  Create_Arg(&sa1->arg,"-mark"         ,"=<lum>"                  ,"=%d"        ,NULL       ,opt_mark       ,"same, with lum; see -ol");
  Create_Arg(&sa1->arg,"-cities"       ,""                        ,NULL         ,NULL       ,opt_cities     ,"add cities");
  Create_Arg(&sa1->arg,"-cities"       ,"=<lum>"                  ,"=%d"        ,NULL       ,opt_cities     ,"same, with lum; see -ol");
  Create_Arg(&sa1->arg,"-timestamp"    ,NULL                      ,NULL         ,NULL       ,opt_timestamp  ,"add date/time to pic.");
  Create_Arg(&sa1->arg,"-timestamp"    ,"@[<x>,<y>]s<size>"       ,"@[%d,%d]s%d",NULL       ,opt_timestamp  ,"same,font size <size>");
  Create_Arg(&sa1->arg,"-timestamp"    ,"@[<x>,<y>]"              ,"@[%d,%d]"   ,NULL       ,opt_timestamp  ,"same, at position (x,y)");
  Create_Arg(&sa1->arg,"-ltimestamp"   ,NULL                      ,NULL         ,NULL       ,opt_ltimestamp ,"see -timestamp, show local time");
  Create_Arg(&sa1->arg,"-ltimestamp"   ,"@[<x>,<y>]s<size>"       ,"@[%d,%d]s%d",NULL       ,opt_ltimestamp ,"same,font size <size>");
  Create_Arg(&sa1->arg,"-ltimestamp"   ,"@[<x>,<y>]"              ,"@[%d,%d]"   ,NULL       ,opt_ltimestamp ,"same, at position (x,y)");
  Create_Arg(&sa1->arg,"-add_text"     ," <text>"                 ,NULL         ,"%s"       ,opt_addtext    ,"add text");
  Create_Arg(&sa1->arg,"-add_text"     ,"@[<x>,<y>]s<size> <text>","@[%d,%d]s%d","%s"       ,opt_addtext    ,"same,  font size <size>");
  Create_Arg(&sa1->arg,"-add_text"     ,"@[<x>,<y>] <text>"       ,"@[%d,%d]"   ,"%s"       ,opt_addtext    ,"same, at position (x,y)");
}




static void options_select(SETARG **sa)
{
  SETARG *sa1=NULL;
  sa1=Create_SetArg(sa,"\n  options for selecting (wild-cards allowed)s:");
  //                    option           shown_frmt                opt vast     opt los     option code     help 
  Create_Arg(&sa1->arg,"-chan"   	," <chan>"                ,NULL        ,"%s"       ,opt_selchan     ,"translate only this channel");
  Create_Arg(&sa1->arg,"-segm"   	," <segm-range>"          ,NULL        ,"%s"       ,opt_selsegm     ,"translate only these segments");
  Create_Arg(&sa1->arg,"-type"   	," <H or L or A>"         ,NULL        ,"%s"       ,opt_seltype     ,"translate only this type (HRIT/LRIT/AVHRR)");
  Create_Arg(&sa1->arg,"-sat"   	," <sattype>"             ,NULL        ,"%s"       ,opt_selsat      ,"translate only this sat (MSG1, MET7...)");
  Create_Arg(&sa1->arg,"-date"   	," <yy-mm-dd>"            ,NULL        ,"%s"       ,opt_seldate     ,"translate only this date");
  Create_Arg(&sa1->arg,"-time"   	," <hh:mm>"               ,NULL        ,"%s"       ,opt_seltime     ,"translate only this time");

}

void do_options(int argc,char **argv,GENMODES *genmode)
{
  SETARG *sa=NULL;
  options_misc(&sa);
  options_formats(&sa);
  options_clrmap(&sa);
  options_geomap(&sa);
  options_outputs(&sa);
  options_overlays(&sa);
  options_select(&sa);
  parse_cmdopt(sa,argc,argv,genmode);       // parse commandline
  Remove_SetArgs(sa);
}
