/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
/*******************************************************************
 * Type definitions XRIT2PIC
 ********************************************************************/
#include <time.h>
#ifdef __NOGUI__
#else
#include "gtk/gtk.h"
#endif
#include "vcdu.h"

typedef struct point
{
  float x,y,z;
  float lon,lat;
} POINT;

typedef struct places
{
  struct places *prev,*next;
  POINT p;
  char name[40];
} PLACES;

typedef struct dirsel
{
  GtkCList *list;
  int selrow;
} DIRSEL;

typedef struct fpfntmp
{
  char *fn;
  int fd;
  FILE *fp;
} FPFNTMP;

typedef enum { UNK_ORBIT=0,GEO=1,POLAR=2} SAT_ORBITTYPE;

typedef enum { UNK_DATA=0,MSG_HRIT=1,MSG_LRIT=2, MFG_LRIT, GOES_LRIT,JMA_HRIT} DATA_TYPE;

typedef enum { OVRWR_ASK=0,OVRWR_REPLACE=1,OVRWR_SKIP=2} OVERWRMODE;

typedef enum { TMP_REMOVE=0,TMP_REPLACE=1,TMP_REUSE=2} TMPFILEMODE;

typedef enum { in_received=0,in_done=1,in_archive,no_file } LOCATION_SEGM;

typedef enum { map_unk=0,map_vis_hmsg=1,map_vis_hmsgdn=15,map_vis_lmsg=2,
               map_airm=3,map_dust=4,map_nfog=5,map_dfog=6,
               map_ucld=7,map_udust=8,map_uash=9,map_unight=10,
               map_uday=11,map_cnv_strm=12,map_snowfog=13,map_eview=14,
               map_noaavis=100,map_metopvis=101,
               map_cust=200,map_cust1=201,
               map_hima_nc,map_hima_ncdn,map_hima_tc
             } COMPOSE_TYPE;

// output formats
typedef enum { PIC_DEF=0,
               PIC_PGM='p' ,PIC_PGM8='P',
               PIC_JPEG='j',PIC_CJPEG='J',
               PIC_RAH='r' ,PIC_TIFF='t'
              } PIC_FORMAT;
              
// input formats; hm, dit moet anders, nog even niet gebruikt
//typedef enum { C_UNK='?'   ,C_PLAIN='p',
//               C_WVT='w'   ,C_JPG='j',
//               C_LMETOP='y',C_METOP='Z',C_LNOAA='z',
//               C_TIFF='t'  ,C_NWSCHRT='b'
//             } FILE_FORMAT;

#define NO_SATPOS 9999
typedef struct satlist
{
  struct satlist *prev,*next;
  char *header; // show header name
  char *grp_name;
  char *name;
  char *show_name;
  gboolean subsat_lon_defined;
  float subsat_lon;               // longitude sub-sat
  struct satlist *chans;
  SAT_ORBITTYPE type;
  DATA_TYPE data_type;
  int min_per_pic;
  gboolean set_default;        // select this by default if no ini file
  gboolean use_in_showlist;
  gboolean use_in_deletelist;
  gboolean remove;
  gboolean enable_remove;      // only set/used in first item of this list!
  gboolean unk_sel;            // only set/used in first item of this list!
  COMPOSE_TYPE defmap_id;
} SATLIST;

typedef enum { SATLIST_NAME,SATLIST_SHOW_NAME,SATLIST_DEFAULT } SATLIST_ITEM;


typedef struct strlist
{
  struct strlist *prev,*next;
  char *str;
} STRLIST;

typedef struct result
{
  int failed;
  int success;
  int err;
  char *message;
} RESULT;

typedef struct rgbmaps
{
  float gamma;
  char chanp[10];  // MSG channel; pos. contr.
  char chann[10];  // MSG channel; neg. contr
  int  valmin;     // lower level of this combination
  int  valmax;     // higher level of this combination
  gboolean temp;   // map to temperatures
} RGBMAPS;

typedef struct msgcmap
{
  char name[20];
  char name_sh[20];
  gboolean use_temp;
  struct rgbmaps r,g,b;
} MSGCMAP;

typedef enum 
{
  normal=0,plate_carree=1, mercator=2, 
  polar_n=3,polar_s=4,
  polar_cn=5,polar_cs=6
} GEOMAP;

typedef enum 
{
  coast_p='c',country_p='y', lonlat_p='l',both_p='b',
  coast_v='C',country_v='Y', lonlat_v='L',both_v='B',
  ovl_err=-1
} OVERLAYTYPE;

#define ANNOSTRLEN 100
typedef struct plottext
{
  int x,y;
  int size;
  gboolean enable;
  gboolean xflag;            // flag for 'something'; timestamp: true=use local time
  char str[ANNOSTRLEN+1];
} PLOTTEXT;

#define NRANNO 2
typedef struct overlays
{
  OVERLAYTYPE overlaytype;           /* l(atlong), c(ountry), 0(just coast) */
  int ol_lum;
  gboolean add_overlay;
  gboolean add_lonlat;
  gboolean add_cities;
  gboolean add_marks;
  struct plottext annotxt[NRANNO+1];
} OVERLAYS;

typedef struct ascatpoint
{
  struct ascatpoint *prev,*next;
  struct scatpoint *sp;
} ASCATPOINT;

typedef struct scatpoint
{
  struct scatpoint *prev,*next;
  float lon,lat;
  float speed,sdir;
  char *errmessage;
} SCATPOINT;

typedef struct showpixmode1
{
  gboolean compose;
  gboolean fire_detect;
  gboolean map_temp;        // map IR channels onto temperature (colour)
  gboolean map_temp_G_mA;   // map IR channels onto temperature (Goes mode-A)
  COMPOSE_TYPE compose_type;
  int bitshift;
} SHOWPIXMODE1;

typedef struct showpixmode
{
  gboolean compose;
  gboolean fire_detect;
  gboolean map_temp;        // map IR channels onto temperature (colour)
  gboolean map_temp_G_mA;   // map IR channels onto temperature (Goes mode-A)
  COMPOSE_TYPE compose_type;
  gboolean invert;
  struct overlays ol;
  int bitshift;
  gboolean inv_ir;
} SHOWPIXMODE;

typedef enum
{
  S_FIRST=0,
  S_messages, // ='S',
  S_hritp1  , // ='h',
  S_hritp2  , // ='g',
  S_hritp3  , // ='T',
  S_hritp4  , // ='Q',
  S_hrita   , // ='H',
  S_lritp1  , // ='l',
  S_lritp2  , // ='k',
  S_lritp3  , // ='t',
  S_lritp4  , // ='q',
  S_lrita   , // ='L',
  S_hima    , // ='J',
  S_other   , // ='f',
  S_met5    , // ='5',
  S_met6    , // ='6',
  S_met7    , // ='7',
  S_mpef    , // ='m',
  S_noaa    , // ='A',
  S_metop   , // ='M',
  S_dwdsat  , // ='D',
  S_bufr    , // ='B',
  S_ers     , // ='E',
  S_ers_geo , // ='s',
  S_all     , // ='a',
  S_geo_all , // = 0xf00,
  S_pol_all , // = 0xf01
  S_LAST
} SHOWTREE;

#define MAXLENPREF 300

#define NRLUTPF 2
typedef struct lut
{
  struct lut *prev,*next;
  char dir[MAXLENPREF];
  char name[MAXLENPREF];
  char fname[MAXLENPREF];
  int w;                  // lut-len
  int nrluts;             // # luts in this file
  int sellut;             // selected lut
  int *rgb[NRLUTPF];
  gboolean use;
} LUT;

typedef struct pixmap
{
  struct chanmap *chanmap;
  int offset[3];
  float gamma[3];
  struct msgcmap clrmapspec;
  COMPOSE_TYPE compose_type;
} PIXMAP;


typedef struct prefitems
{
  struct prefitems *prev,*next;
  int hex_nrdig;      // write as hex, values=nr. digits
  char type;          // 'c': 1 character instead of string; 't': comment in name
  gboolean bdef[2];   // boolean default
  int      idef[2];   // integer default
  float    fdef[2];   // float default
  char     *sdef;     // string default
  POINT    pdef;      // position default

  char     sep;       // separator in pref-value between name and value

  gboolean *bval[2];
  int      *ival[2];
  float    *fval[2];
  char     *sval[2];
  POINT    *pval[2];
  char     *name;
} PREFITEMS;

typedef struct colour
{
  gboolean compose;
  gboolean hrv_lum;
  gboolean invert;
//  gboolean inv_ir;
} COLOUR;

typedef struct pref_main
{
  struct colour clr;
  struct overlays ol;
  char oltype[MAXLENPREF];
  char clrmapfixed[MAXLENPREF];
} PREF_MAIN;


typedef struct pref_rec
{
  //---- nb_record, main tab
  gboolean record_at_start;

  //---- nb_record, subtab "Show live":
  gboolean lvs_enable;        // enable live picture show
  struct colour lv_clr;         // for live and generation
  struct overlays lv_ol;
  char lv_sat[MAXLENPREF];
  char lv_chan[MAXLENPREF];

  //---- nb_record, subtab "Generate":
  gboolean lvg_enable;        // enable live generation
  gboolean lvg_pic0_movie1;
  gboolean lvg_movie_rmjpeg;
  int lvg_size_timestamp;
  gboolean lvg_essential_only;
  gboolean lvg_run_script;
  gboolean lvg_rerun_script;

  gboolean lvg_moviecmd_constr;
  gboolean lvg_movie_fullscreen;
  int lvg_movie_speed;
  int lvg_movie_size_max_hr;
  int lvg_movie_start_utc_hr;
  int lvg_movie_rpt_lastfrm;
  float lvg_gamma;
  gboolean lvg_fire_detect;
  // Aging
  gboolean delete_old_at_start;
  
  // Other
  gboolean msg_notify;
} PREF_REC;

typedef struct pref_rcvloc
{
  char src_dir[MAXLENPREF];
  STRLIST *srclist;
} RCVLOC;

typedef struct
{
  struct filelist *fl;
  int width,height;
  int nrframes;
  int repeated_at_end;
  int uspfr;
} MOVIEINFO;

typedef struct globinfo
{
  gboolean safemode;
  char releasenr[100];
  char bdate[100];
  char btime[100];
  OVERWRMODE overwrite_mode;
  TMPFILEMODE filmframe_ovrwrmode;
  SHOWTREE vis_mode;
  gboolean abort;
  gboolean mirror_negval;
  gboolean gui_mode;
  gboolean debug;
  gboolean log;
  FILE *fplog;
  int busy_gen;       // >0 if busy with generation item (see gen_item())
  int jpeg_quality;
  int movie_quality;

  struct showpixmode1 spm;
  struct showpixmode1 spm_live;

  int nr_pics;
  int nr_selpics;
  unsigned long tot_size;
  float mbsize;
  gboolean move_to_done;       // altijd actief!
  gboolean donedir_tomake;
  gboolean donedir_exist;
  gboolean gen_new;
  gboolean rm_raw;
  gboolean pic_only;
  int bitshift;
  gboolean avhrr_lin;
  gboolean view_exported;
  int area_nr;              // 0=full
  gboolean dim3;
  GEOMAP gmap;
  gboolean correct_aspect;
  gboolean essential_only;
  gboolean hrv_lum;
  gboolean invert_ir;

  int shift_3d;
  int nrfiles;

// Limit polar pics
  char pdir_show;     // 'a', 'd' or 'b'=0
  int ltlat,gtlat;
  gboolean marktype_vec;   // draw marks as vector in drawable

//  struct satchannel_list keep_chanlist;
  int segmrange[2];
  
/* list received dirs */
  struct dirsel dirsel;

  struct pixmap pixmap;              // mapping of pixels on RGB window
  struct pixmap pixmap_gen;          // mapping of pixels on RGB generate
  struct pixmap pixmap_live;         // mapping of pixels on RGB live

  struct plottext annotxt[NRANNO+1];

  char src_archdir[MAXLENPREF];
  char src_donedir[MAXLENPREF];
  char dest_dir[MAXLENPREF];
  char dest_tmpdir[MAXLENPREF];
  char dest_tmpsavdir[MAXLENPREF];
  char prog_bzip2[MAXLENPREF];
  char prog_wget[MAXLENPREF];
  char run_cmd[MAXLENPREF];
  char rerun_cmd[MAXLENPREF];

  struct segments *bufr;
  SCATPOINT *scatp;
  ASCATPOINT *ascatp;
  gboolean scat_plot_all;
  gboolean test_bufr;
  
  LUT lut;
  SATLIST *satlist;
  int time_last_rec;        // time (hour) last received

  struct pref_main pm;      // from preferences for main tab
  struct pref_rec pr;       // from preferences for record tab
  struct pref_rcvloc pl;    // from preferences for received tab
} GLOBINFO;


typedef struct chanmap
{
  struct chanmap *prev,*next;
  char chan_name[20];
  float r,g,b;
  gboolean lum;
  gboolean temperature;
  gboolean invert;
} CHANMAP;

typedef struct overlay
{
  struct overlay *prev,*next;
  char fname[80];
  char *dir;
  char satname[20];                  /* met,goes,mtsat */
  int sattype;                       /* 8,7,6,5 (met), 10,11,12 (goes) */
  OVERLAYTYPE overlaytype;           /* l(atlong), c(ountry), 0(just coast) */
  char channel[10];                  /* channel: hrv, ir, vis or "" */
  int satpos;                        /* position satellite (east) */
  gboolean rapidscan;
} OVERLAY;

typedef struct picpart
{
  int fix_width;
  int fix_woffset;
  int fix_height;
  int fix_hoffset;
  int pct_width;
  int pct_woffset;
  int pct_height;
  int pct_hoffset;
  int center_x1;
  int center_x2;
  int center_y1;
  int center_y2;
} PICPART;

#define GEOPARTNAMELEN 50
typedef struct geoarea
{
  char part_name[GEOPARTNAMELEN];
  struct picpart pp_norm;
  struct picpart pp_hrv;
  struct point center;
  struct point delta;
  int pol_dir;
} GEOAREA;

typedef struct f1map
{
  char chan[10];
  float r, g, b;
  gboolean invert;
} F1MAP;

typedef struct fmap
{
  int offset[3];
  float gamma[3];
  gboolean use_temp;
  int n;
#if __GTK_WIN32__ == 1
   F1MAP fm[8];
#else
   F1MAP fm[8];
#endif
} FMAP;

#define NRGEODEFS 50
typedef struct geoareadefs
{
  char part_name[50];  
  struct point center;
  struct point delta;
} GEOAREADEFS;


typedef struct pref
{
  char used_preffile[MAXLENPREF];
  char gui_inifile[MAXLENPREF];
  char *cur_dir;
  char *home_dir;
  char *prog_dir;
  char *prog_xdir;
  char *ini_file;
  gboolean enable_restore_guistate;
  int font_size;

  int movie_resol;
//  int movie_max_nrframes;

  int polar_size;
  int max_nr_rfiles;    // amx. nr. of received fiels (above: do 'something')

  // external programs
  char prog_jpg[MAXLENPREF];
  char ud_jpg[MAXLENPREF];
  char prog_pgm[MAXLENPREF];
  char ud_pgm[MAXLENPREF];
  char prog_playmovie[MAXLENPREF];
  char prog_genmovie[MAXLENPREF];
  char prog_bufrextr[MAXLENPREF];
  char bufrtblloc[MAXLENPREF];

  int speed;
  gboolean extern_moviegen;
  char src_archdir[MAXLENPREF];

  // Tellique related items; not used by xrit2pic (see weview)
  char tlq_cmd[MAXLENPREF];
  char tlq_pdir[MAXLENPREF];      // tellique program location
  char tlq_sdir[MAXLENPREF];      // tellique start location

  char dest_dir[MAXLENPREF];
  gboolean split_destdir;
  char done_dir[20];
  char lut_dir[MAXLENPREF];
  char lut_file[MAXLENPREF];
  struct overlay *overlay;
  struct lut *lut;
  char prf_ovltype;      // prefered overlaytype: 'p'(icture) or 'v'(ector)
  int mark_clr;          // colour markers (1 nibble per clr;'rgb')
  int city_clr;          // colour city names (1 nibble per clr;'rgb')
  int ovl_clr;           // colour overlay (1 nibble per clr;'rgb')
  int lonlat_clr;        // colour lon/lat lines
  int scat_clr;          // colour scat arrows
  int scat_selclr;       // colour scat arrows selected
  gboolean add_marks;
  gboolean invert_ir;
  int size_cmark;    // size cross mark
  int deltalonlat;
  POINT lmark;
  PLACES *places;
  gboolean show_marks;
  struct plottext annotxt[NRANNO+1];    // text to add to pictures

  FMAP fmap_vis_hmsg;
  FMAP fmap_vis_hmsgdn;
  FMAP fmap_vis_lmsg;
  FMAP fmap_vis_noaa;
  FMAP fmap_vis_metop;
  FMAP fmap_ncvis_hima;
  FMAP fmap_ncvisdn_hima;
  FMAP fmap_tcvis_hima;
  MSGCMAP msgmap_cust;

  gboolean scandir_correct;
  gboolean low_mem;
  gboolean size_draw_limit;
  gboolean avhrr_mode;
  gboolean full_globe;
  
  int deldaymin[2];     // [0]: days, [1]: hours
  int deldaymax;
  gboolean roundday;

  GEOAREA geoarea[NRGEODEFS];

  gboolean use_fn_frmt;
  char fn_frmt[MAXLENPREF];
  char noradfile[MAXLENPREF];
  char placesfile[MAXLENPREF];
  char earthmapfile1[MAXLENPREF];
  char earthmapfile2[MAXLENPREF];
  char run_cmd[MAXLENPREF];
  char rerun_cmd[MAXLENPREF];
  char upd_sat[MAXLENPREF];
  char upd_chan[MAXLENPREF];
  int upd_cycle;
  int wndwidth;
  int wndheight;
  int pr_wndwidth;
  int pr_wndheight;
  int pr_wndmax;
  gboolean pr_corr_aspect;
  int lv_wndwidth;
  int lv_wndheight;
  int lv_wndmax;
  char depth_chan[MAXLENPREF];             // anaglyph depth chan
  gboolean dchan_iv;
  gboolean essential_only;
  gboolean keep_unzipped;
  int jpeg_quality;
  int movie_quality;

  struct pref_main pm;      // preferences for main tab
  struct pref_rec pr;       // preferences for record tab
  struct pref_rcvloc pl;    // preferences for received tab
} PREFER;

typedef enum { OK=0 ,Unk_Frmt,Tmp_fn,JPG_Rd,JPG_Wr,JPG_Conc,
               Open_Rd,Open_Wr,Open_Wrt,Out_fn,WVT_Err,Aborted,Exist_fn,Mis_segm,Mis_pro,Bzip2_Err,Err_OSize,
               Avi_picsize=-1,Avi_nrframes=-2,Avi_write=-3   // These MUST be negative!
             } ERRCODE;


typedef struct anagl
{
  int shift_3d;
  int lmin;
  int lmax;
  gboolean init;
  gboolean depthchan;
  gboolean ol_3d;
} ANAGL;


typedef struct genmodes
{
  char type;               /* L or H */
  char *sat;
  char *chan_name;
  char *segm_range;
  char *date;
  char *time;
  char *ofile;
  char otype;       /* 'v(iew)'  or 'f(ile)' */  
  char oformat;     /* 'j(peg)': direct, 'J(peg)': indir, 'p(gm)' */ 
  char cformat;     /* choosen format: 'd(efault)','j(peg)','p(gm)' */ 
  struct chanmap *chanmap;
  gboolean for_live;
  gboolean do_one_pic;
  gboolean gen_film;
  gboolean run_script;
  gboolean rerun_script;
  gboolean essential_only;
  struct showpixmode spm;
  gboolean tempmap;

  char *wnd_clrmap;
  gboolean use_hrv_lum;
  OVERWRMODE overwrite_mode;
  TMPFILEMODE filmframe_ovrwrmode;
  gboolean gen_pro;
  gboolean gen_epi;
  gboolean gen_rah;
  gboolean skip_incomplete;
  gboolean test;
  char image_oformat;
  void (*log_func)();
  FILE *fplog;
//  int xoff,yoff,xfac,yfac;  // for mapped polar
  int dx,dy;
  float zx,zy;
  int ox,oy;
  int odepth;
  int frame_minwidth;   // min. width used frames
  gboolean adapt_lum;
  int lmin,lmax;
  int lminrgb[3],lmaxrgb[3];
  float gamma;
  int area_nr;              // 0=full
  gboolean avhrr_lin;
  gboolean avhrr_lin_dblx;
  int jpeg_quality;
  GEOMAP gmap;
  struct anagl agl;
  GEOAREA geoarea;
} GENMODES;

typedef struct luminfo
{
  gboolean sep_rgblum;
  unsigned int *lumstat;
  int lumscale;
  int lminrgb[3];
  int lmaxrgb[3];
  int lmin;
  int lmax;
} LUMINFO;

#define HTTP_KEPLER "http://celestrak.com/NORAD/elements"
#define NORADFILE "weather.txt"
#ifndef gint32
#define gint32 int
#endif
typedef struct kepler
{
  gint32 epoch_year;
  float  epoch_day;
  float  decay_rate;
  float  inclination;
  float  raan;
  float  eccentricity;
  float  perigee;
  float  anomaly;
  float  motion;
  gint32 epoch_rev;
} KEPLER;

typedef struct orbit
{
  gboolean valid;                  /* flag: kepler data valid? */

/* Start time record (used) */
  struct tm start_tm;              /* record start time  */
  int       start_ms;              /* record start time msec */

/* Start time record (satellite) */
  struct tm start_tm_sat;          /* record start time  */
  int       start_ms_sat;          /* record start time msec */

/* Start time record (computer) */
  struct tm start_tm_comp;         /* record start time  */
  int    loop_nr;                  /* # loops from ref. at start pic */

/* Reference time Kepler data */
  gint32    ref_time;              /* total reference time  */
  struct tm ref_tm;                /* reference time year ... sec */
  int       ref_ms;                /* reference time msec */

  float height;                    /* sat height [meters] */ 
  float loop_time;                 /* time 1 loop [seconds] */
  float k2_n;
  float k2_w;
  float dh_w;

  int age_days;                    /* age Kepler data in days */
  gboolean use_sattime;
  int offset_gmt;
  int offset_sec;
  float offset_lon;
} ORBIT;

typedef struct geochar
{
  gint32 coff,cfac,loff,lfac;
  int upper_x_shift;     /* shift after shift_ypos (HRV upper) */
  int lower_x_shift;     /* shift before shift_ypos (HRV lower) */
  int shift_ypos;        /* pos after which x shifts (HRV) */
  float subsat_lon;
  float aspect_ratio;    // aspect source
} GEOCHAR;

typedef struct
{
// info pic with combined channels
  gboolean done;
  int widthmax;
  int heightmax;
  
  int first_chunck;
  int last_chunck;
  int nrchuncks;
  int y_chunck;
  int width;
  int org_width;            // org. width (for HRV: width total globe)
  int x_w,x_e,y_n,y_s;      // actual pos. earth globe
  int height;
  int depth;
  int lines_per_sec;        // for polar sats
  char image_iformat;
  char fno[1000];
  char picname[1000];
  char chan[50];
  char sat_src[20];         // see satsrc in vcdu.h
  int scan_dir;

  KEPLER kepler;
  ORBIT orbit;
  gboolean avhrr_lin;
  GEOMAP gmap;
  float *ypos_tbl[2];
  int xoff,yoff,xfac,yfac;  // for mapped polar, NOT geo! See for that struct geochar.
  float aspect_ratio;       // use this aspect
  SAT_ORBITTYPE orbittype;
  gboolean incomplete;      // picture incomplete (missing segments)
  gboolean incomplete_pro;  // picture incomplete (missing needed prologue)
  gboolean partly;          // part of picture to generate
  gboolean is_color;
  gboolean map_temp;        // map IR channels onto temperature
  gboolean map_temp_G_mA;   // map IR channels onto temperature GOES mode-A (dog-leg)
  gboolean use_lut;
  gboolean invert;
  gboolean avhrr_lin_dblx;
  gboolean force_allsegm;   // asume all segments present (needed for live pic)
  int bitshift;
  struct geochar gchar;
  int o_width,o_height;     // actual output size
  int o_xoffset,o_yoffset;  // actual output offset

} PIC_CHARS;

typedef struct orbit_segm
{
  POINT subsat_start;
  POINT subsat_end;

  int orbit_start;
  int epoch_year;
  float epoch_day;
  float eccentricity;
  float inclination;
  float perigee;
  float raan;
  float anomaly;
  int total_mdr;
  int duration_of_product;
} ORBIT_SEGM;

typedef struct
{
  float LongitudeOfSSP;  /* sub-satellite point in degrees (after correction) */
} ProjectionDescription;

/* NOTE! If link to allocated mem added: See Copy_Segm; set NULL in copy! */ 
typedef struct segments
{
  struct segments *prev,*next;
  struct channels *chan;
  struct segments *orig_segm;
  gboolean is_pro,is_epi;
  GtkCTreeNode *node;
  gboolean selected;
  gboolean nselected;
  gboolean shown;
  struct xrit_hdr xh;
  char *fn;              /* filename segment */
  char *pfn;             /* path + filename segment */
  time_t time;           /* time file */
  LOCATION_SEGM where;   /* location of file: received, done, archive */
  gboolean corrupt;      /* corrupt file */
  guint16 *chnk;
  char chan3_ab;          /* channel 3 'a'or b', 0: unknown */
  char *ovlchnk;
  int width,height;
  int y_offset;          /* y where actual pic starts (partly filled segm) */
  OVERLAYTYPE loaded_ovl;
  int upper_east_col;    /* shift after shift_ypos (HRV upper) */
  int lower_east_col;    /* shift before shift_ypos (HRV lower) */
  int lower_nord_row;    /* pos after which x shifts (HRV) */
  ProjectionDescription ProjectionDescription;
  ORBIT_SEGM orbit;
} SEGMENT;

struct calibration
{
  int caltbl_bpp;
  int caltbl_type;
  float *caltbl[2];
  int nrcalpoints;
} CALIBRATION;

#define MAXLEN_ID 40
typedef struct channels
{
  struct channels *prev,*next;
  struct group *group;
  GtkCTreeNode *node;
  gboolean selected;
  gboolean nselected;
  gboolean hide;
  struct segments *segm;
  struct segments *pro;
  struct segments *epi;
  char *chan_name;
  char *data_src;
  char ifrmt;
  char id[MAXLEN_ID+1];
  char scan_dir;
  int chan_nr;
  int wavel;
  int pic_nr;
  int nr_segm;
  int seq_start;
  int seq_end;
  int nc,nl,nb;
  int nc_orig,nl_orig;     // orig. sizes, before taking  into account aspects
  int lines_per_sec;        // for polar sats
  double Cal_Slope[12];  /* calibration slope */
  double Cal_Offset[12]; /* calibration offset */
  int *xlut[2],*ylut,*slut;
  int *rgblut[3];
  struct calibration cal;

  int upper_x_shift;     /* shift after shift_ypos (HRV upper) */
  int lower_x_shift;     /* shift before shift_ypos (HRV lower) */
  int shift_ypos;        /* pos after which x shifts (HRV) */
  int max_x_shift;       /* max of upper_x_shift and lower_x_shift */
  int width_totalglobe;  /* if !=0: # lines if pic would be total globe */
  int height_totalglobe; /* if !=0: # lines if pic would be total globe (for MDM) */
  int ncext;             /* extended nc for full earth (HRV) */
  float r,g,b;           /* contr. to r/g/b */
  gboolean invert;       /* invert this channel */
  gboolean lum;          /* use for luminance in comb. with rgb of other chans */
} CHANNEL;

#define MAXCHANSTR 20
typedef struct group
{
  struct group *prev,*next;
  struct group *org;
  struct list_files *list_files;
  GtkCTreeNode *node;
  gboolean selected;
  gboolean nselected;
  gboolean hide;
  gboolean compose;
  gboolean gen_rah;
  gboolean has_kepler;
  gboolean done;
  gboolean new;             // group is new
  int width,height;
  struct channels *chan;
  struct channels *pro_epi;
  char sat_src[20];         // see satsrc in vcdu.h
  char region[20];
  struct tm grp_tm;
  char grp_time[20];        // see satsrc in vcdu.h
  char id[MAXLEN_ID+1];
  guint32 pic_id;
  int pic_nr;
  char h_or_l;
  DATATYPE datatype;       // 
  SAT_ORBITTYPE orbittype;
  char scan_dir;
  char chanstr[MAXCHANSTR];
  PIC_CHARS pc;
  guint16 *rgbpicstr[4];   // rgb and mask
  gboolean *line_in_mem;
  gboolean *ovl_in_mem;
  gboolean keep_in_mem;
  gboolean conv_temp;
  int offset[3];
  float gamma[3];
  int *rgblut[3];
  int segm_first,segm_last;
  int overload;         // >0: too(?) much files, # = amount of files
} GROUP;


typedef struct list_files
{
  struct list_files *prev,*next;
  struct list_files *cparent;
  struct list_files *tparent;
  GtkCTreeNode *pnode;
  GtkCTreeNode *cnode;
  GtkCTreeNode *tnode;

  struct xrit_hdr pxh;
  struct group *grp;
  struct channels *chan;
  gboolean set_selected;
  gboolean chan_selected;
  gboolean segm_selected;
  gboolean compose;
  char *path;            /* path to file */
  char *fn;              /* file name */
  char *pfn;             /* path + file name */
  time_t time;           /* time file */
  int age;               /* age in days */
  int nr_files;          /* # files found (only in top-struct, so with prev==NULL) */
  int set_nr;
  int pic_nr;
  int segm_nr;
  int nr_childs;
  int nr_segments;
  int lps;         // lines per second (polar)
  char frmt[20];
  gboolean is_dir;
  gboolean unknown;
  float r,g,b;
} LIST_FILES;

typedef struct filelist
{
  struct filelist *prev,*next;
  char *fn;
  unsigned long size;
  unsigned long offset;
  gboolean new;
  gboolean repeated;
  gboolean dont_use;
} FILELIST;

typedef struct xrit_dbase
{
//  GROUP *group;
  LIST_FILES *unknown;
  GROUP *grp_last;             // last group in list, at start recording
  GtkCTree *main_tree;
//  struct globinfo globinfo;  // nog te doen
//  struct pref prefer;        // nog te doen
} XRIT_DBASE;
