/*******************************************************************
 * Extract info from annotation as present in filenames and xrit header files 
 * int extract_anno(XRIT_HDR *xh)
 *
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 ********************************************************************/
 /* To adapt, use 'rules':
    extract_anno_message()
    extract_anno_mtp()
    extract_anno_avh()
    extract_anno_bufr()
    extract_anno_dwdsat()
 */
 
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "vcdu.h"
#include "xrit2pic.h"
#include "avhrr.h"
extern GLOBINFO globinfo;
extern char *channellist_msg[];

// Start common part
#define NRKEYS 10
typedef struct strings
{
  int nr;
  char *str[NRKEYS];
  char *ostr[NRKEYS];
} STR;

static void clear_str_rule(STR *keys)
{
  int i;
  for (i=0; i<keys->nr; i++)
  {
    freed(&keys->ostr[i]);
    freed(&keys->str[i]);
  }
}

// Extract info from annotation string 'str', use rule 'rstr' and keys 'keys'
// Example rule: "<HL>-<vers>-<dsat>-<sat>-<chan>-<segnr>-<time>-<CE>"
//   Keys are <HL>, <vers>, <dsat> etc.
//   '-' is separator between keys.
//   First key is HL, = first to extract
//   Second key is <vers> = second to extract
// etc. Length of key has no meaning in relation to string to extract; the separator is used for that.
// Note: trailing '_' to remove afterwards!
static int parse_str_rule(char *str,STR rstr,STR *keys)
{
  char *c,*r,*ke;
  char *cn,*rn;
  char *rule;
  int nk,nr;
  int err=0;
  for (nr=0; nr<rstr.nr; nr++)               // rstr.nr is # tokens to check
  {
    err=0;
    rule=rstr.str[nr];                       // 'rule' is one of tokens
    c=str;
    r=rule;
    do
    {
      if (!*c) { err=1; break; }
      rn=r+1; cn=c+1;
      if (!strncmp(r,"<...>",5))
      {
        rn=r+5;
        for (cn=c; *cn; cn++) if (*cn==*rn) { c=cn; break;}
        if (!*cn) { err=3; break; }
        r=rn;
      }
      else if (!strncmp(r,"<#",2))
      {
        int n;
        rn=r+2;
        n=atoi(r);
        if (!(r=strchr(r,'>'))) { err=1; break; }
        cn=c+n;
      }
      else                                   // standard token search
      {
        for (nk=0; nk<keys->nr; nk++)        // for all keys
        {
          char *key=keys->str[nk];
          if (!strncmp(r,key,strlen(key)))   // check key at start of 'r'
          {
            rn=r+strlen(key);                // rn at pos. end of key which is separator
            if ((ke=strchr(c,*rn)))          // search separator in annotation string
            {
              int len=ke-c;
              if (len>=100) break;
              keys->ostr[nk]=calloc(len+2,1); // allocate mem for this string
              strncpy(keys->ostr[nk],c,len);  // extract the string
              cn=ke;
            }
            break;
          }
        }
        c=cn;
        r=rn;
        if (*r!='<') if (*c!=*r) { err=2; break; }
      }
      if (!*r) break;  // end-of=rules string; exit loop (doesn't work with 'for' because increment r done)

    } while (*r);
    if (!err) break;
  }
  return err;
}

// End common part

// Remove trailing underscores
// abc_def__ ==> abc_def
static void remove_tr_usc(char *s)
{
  char *p;
  p=s+strlen(s)-1;
  while ((p>=s) && (*p=='_'))
  {
    *p=0;
    p--;
  } 
}

/*************************************
 * translate channel name into number 
 * 
 *************************************/
/* Note: Only info in file-name known here! */
static void channame2nr(XRIT_HDR *xrit_hdr)
{
  if      (xrit_hdr->special=='p')           xrit_hdr->chan_nr=0;
  else if (xrit_hdr->special=='P')           xrit_hdr->chan_nr=0;
  else if (xrit_hdr->special=='e')           xrit_hdr->chan_nr=0;
  else if (xrit_hdr->special=='E')           xrit_hdr->chan_nr=0;
  else if (!strcmp(xrit_hdr->chan,"VIS006")) xrit_hdr->chan_nr=1;
  else if (!strcmp(xrit_hdr->chan,"VIS008")) xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"IR_016")) xrit_hdr->chan_nr=3;
  else if (!strcmp(xrit_hdr->chan,"IR_039")) xrit_hdr->chan_nr=4;
  else if (!strcmp(xrit_hdr->chan,"WV_062")) xrit_hdr->chan_nr=5;
  else if (!strcmp(xrit_hdr->chan,"WV_073")) xrit_hdr->chan_nr=6;
  else if (!strcmp(xrit_hdr->chan,"IR_087")) xrit_hdr->chan_nr=7;
  else if (!strcmp(xrit_hdr->chan,"IR_097")) xrit_hdr->chan_nr=8;
  else if (!strcmp(xrit_hdr->chan,"IR_108")) xrit_hdr->chan_nr=9;
  else if (!strcmp(xrit_hdr->chan,"IR_120")) xrit_hdr->chan_nr=10;
  else if (!strcmp(xrit_hdr->chan,"IR_134")) xrit_hdr->chan_nr=11;
  else if (!strcmp(xrit_hdr->chan,"HRV"))    xrit_hdr->chan_nr=12;
/* MTP */
  else if (!strcmp(xrit_hdr->chan,"VIS007")) xrit_hdr->chan_nr=1;
  else if (!strcmp(xrit_hdr->chan,"IR_064")) xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"WV_064")) xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"IR_115")) xrit_hdr->chan_nr=3;
/* GOES */
  else if (!strcmp(xrit_hdr->chan,"IR_066")) xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"IR_068")) xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"IR_107")) xrit_hdr->chan_nr=3;
  else if (!strcmp(xrit_hdr->chan,"VIS"))    xrit_hdr->chan_nr=1;
  else if (!strcmp(xrit_hdr->chan,"IR1"))    xrit_hdr->chan_nr=2;
  else if (!strcmp(xrit_hdr->chan,"IR2"))    xrit_hdr->chan_nr=3;
  else if (!strcmp(xrit_hdr->chan,"IR3"))    xrit_hdr->chan_nr=4;
  else if (!strcmp(xrit_hdr->chan,"IR4"))    xrit_hdr->chan_nr=5;
  else if (isdigit(*(xrit_hdr->chan+3)))     xrit_hdr->chan_nr=atoi(xrit_hdr->chan+3); /* not defined here */
  else if (isdigit(*(xrit_hdr->chan+1)))     xrit_hdr->chan_nr=atoi(xrit_hdr->chan+1); /* not defined here */
  else                                       xrit_hdr->chan_nr=-1; /* not defined here */

  if (xrit_hdr->xrit_frmt==JMA)
  {
    if (!strcmp(xrit_hdr->chan,"VIS")) xrit_hdr->chan_nr=3;
    else if (!strcmp(xrit_hdr->chan,"IR1")) xrit_hdr->chan_nr=13;
    else if (!strcmp(xrit_hdr->chan,"IR2")) xrit_hdr->chan_nr=15;
    else if (!strcmp(xrit_hdr->chan,"IR3")) xrit_hdr->chan_nr=8;
    else if (!strcmp(xrit_hdr->chan,"IR4")) xrit_hdr->chan_nr=7;
    else if (isdigit(*(xrit_hdr->chan+1)))     xrit_hdr->chan_nr=atoi(xrit_hdr->chan+1); /* not defined here */
  }
}

// frmt: e.g. yyyyDDDHHMM, yyyymmddHHMM or in any other order
static struct tm timestr2tm(char *p,char *frmt)
{
  struct tm tm;
  int n,m;
//printf(">%s<  >%s<\n",p,frmt);
  memset(&tm,0,sizeof(tm));
  for (n=0,m=0; frmt[n]; n++)
  {
    if (frmt[n]!=frmt[n+1])
    {
      switch (frmt[m])
      {
        case 'y': tm.tm_year=atoi(get_strpart(p,m,n))-1900; break;
        case 'D': tm.tm_yday=atoi(get_strpart(p,m,n))-1; yday2mday_mon(&tm); break;
        case 'd': tm.tm_mday=atoi(get_strpart(p,m,n)); break;
        case 'm': tm.tm_mon=atoi(get_strpart(p,m,n))-1; break;
        case 'H': tm.tm_hour=atoi(get_strpart(p,m,n)); break;
        case 'M': tm.tm_min=atoi(get_strpart(p,m,n)); break;
        default: break;
      }
      m=n+1;
    }
  }
  return tm;
}

/*************************************
 * Extract annotation info messages (xml only)
 * Examples:
 *
 * E-UNS_Alert_Message_20151119041122.xml
 * E-UNS_Weekly_Operations_Schedule_20151117142913.xml
 *************************************/
static int extract_anno_message(XRIT_HDR *xh)
{
  char *p;
  char tmp[20];
  if ((p=strrchr(xh->anno,'_')))
  {
    p++;
    strcpy(xh->itime,get_strpart(p,0,13));

    memset(&xh->time,0,sizeof(xh->time));

    strncpy(tmp,p,4); tmp[4]=0; p+=4;
    xh->time.tm_year=atoi(tmp)-1900;

    strncpy(tmp,p,2); tmp[2]=0; p+=2;
    xh->time.tm_mon=atoi(tmp)-1;

    strncpy(tmp,p,2); tmp[2]=0; p+=2;
    xh->time.tm_mday=atoi(tmp);

    strncpy(tmp,p,2); tmp[2]=0; p+=2;
    xh->time.tm_hour=atoi(tmp);

    strncpy(tmp,p,2); tmp[2]=0; p+=2;
    xh->time.tm_min=atoi(tmp);

    mday_mon2yday(&xh->time);
    strcpy(xh->satsrc,"Srvc");
    xh->hl='m';
    xh->file_type=2;                          /* means: txt */
  }
  return 1;
}

/*************************************
  Change channel names to make it more clear and to match with overlay files
  For foreign sats, via Eumetcast.
  Format: e.g. 00_7_000E: 00_7 = spectr, 000E=pos
  Save sat-pos from name, in case it is needed for subsat pos. lateron
  Used in extract_anno_esat
 *************************************/
static void change_channelnames(XRIT_HDR *xh)
{
  char ch[10];
  if (isdigit(xh->chan[0]))
  {
    /* Extract position */
    if (xh->subsat_lon==NO_SATPOS)
    {
      xh->subsat_lon=atoi(xh->chan+5);
      if (strchr(xh->chan+5,'W'))
        xh->subsat_lon*=-1.;
    }

    /* Remove position */
    *(xh->chan+5)=0;
    if (xh->chan[2]=='_')
    {
      sprintf(ch,"%c%c%c",xh->chan[0],xh->chan[1],xh->chan[3]);
    }
    else
    {
      strncpy(ch,xh->chan,4);
      xh->chan[4]=0;
    }
    if (atoi(ch) < 9)
      sprintf(xh->chan,"VIS%s",ch);
    else if ((atoi(ch) > 61) && (atoi(ch) < 74))
      sprintf(xh->chan,"WV_%s",ch);
    else
      sprintf(xh->chan,"IR_%s",ch);
    
  }
  xh->wavel=atoi(xh->chan+3);
}

static void rule2key(char *rule,STR *keys)
{
  char *tmp=NULL,*p;
  char *p1=NULL;
  strcpyd(&tmp,rule);
  keys->nr=0;
  for (p=tmp; *p; p++)
  {
    if (*p=='<') p1=p;
    if ((*p=='>') && (p1))
    {
      char ctmp;
      if (keys->nr<NRKEYS)
      {
        p++;
        ctmp=*p;
        *p=0;
        strcpyd(&keys->str[keys->nr],p1);
        *p=ctmp;
        p--;
        p1=NULL;
        keys->nr++;
      }
    }
  }
  freed(&tmp);
}

/*************************************
 * Extract annotation info Eumetsat / Eumetcast naming style
 * These include foreign satellites relayed by Eumetcast!
 * Examples:
 * 
 * L-000-MSG1__-GOES7_______-IR_107___-00004____-200202020202-CE
 * L-000-MSG1__-MSG1________-IR_016___-00001____-200202020202-CE
 * H-000-MSG1__-MSG1________-_________-EPI______-200305040944-__
 * H-000-MSG1__-MSG1________-_________-PRO______-200305040914-__
 * L-000-MSG3__-GOES15______-03_9_135W-PRO______-201608021900-__
 * H-000-MSG3__-MSG3_RSS____-HRV______-000021___-201912121315-C_
 *
 * return: 1 if all info to group files available
 *************************************/
#define ESAT_RULE1 "<HL>-<vers>-<dsat>-<sat>-<chan>-<segnr>-<time>-<CE>"
static int extract_anno_esat(XRIT_HDR *xh)
{
  STR rules,keys;
  char *segnr_str=NULL;
  char *itime=NULL;
  char *crypt=NULL;
  int nk;
  int ret=0;
  memset(&rules,0,sizeof(rules));
  memset(&keys,0,sizeof(keys));
  rules.nr=1;
  rules.str[0]=ESAT_RULE1;
  rule2key(rules.str[0],&keys);

  if (!(parse_str_rule(xh->anno,rules,&keys)))
  {
    for (nk=0; nk<keys.nr; nk++)
    {
      if (keys.ostr[nk])
      {
        if (!strcmp(keys.str[nk],"<HL>"))     xh->hl=*keys.ostr[nk]; 
        if (!strcmp(keys.str[nk],"<vers>"))   strncpy(xh->vers,keys.ostr[nk],4); // longest expected string + 1
        if (!strcmp(keys.str[nk],"<dsat>"))   strncpy(xh->sat,keys.ostr[nk],7); 
        if (!strcmp(keys.str[nk],"<sat>"))    strncpy(xh->src,keys.ostr[nk],13); 
        if (!strcmp(keys.str[nk],"<chan>"))   strncpy(xh->chan,keys.ostr[nk],10);
        if (!strcmp(keys.str[nk],"<segnr>"))  segnr_str=keys.ostr[nk];
        if (!strcmp(keys.str[nk],"<time>"))   itime=keys.ostr[nk];
        if (!strcmp(keys.str[nk],"<CE>"))     crypt=keys.ostr[nk];
      }
    }
    remove_tr_usc(xh->sat);
    remove_tr_usc(xh->src);
    remove_tr_usc(xh->chan);

    // do some renaming of name satellite source data
    if (!strncmp(xh->src,"MSG",3))
      strcpy(xh->satsrc,xh->src);
    else if (!strncmp(xh->src,"SERVICE",7))
      strcpy(xh->satsrc,"Srvc");
    else if (!strncmp(xh->src,"MPEF",4))
      strcpy(xh->satsrc,xh->src);
    else if (!strncmp(xh->src,"MET",3))
      strcpy(xh->satsrc,xh->src);
    else if (!strncmp(xh->src,"GOES",4))
      strcpy(xh->satsrc,xh->src);
    else if (!strncmp(xh->src,"MTSAT",5))
      strcpy(xh->satsrc,xh->src);
    else
      strcpy(xh->satsrc,"Frgn");

    if (segnr_str)
    {
      xh->special=0;
      if (!strncmp(segnr_str,"PRO_",4))
      {
        if (!*xh->chan)
          xh->special='P';                    /* group PRO */
        else
          xh->special='p';                    /* channel PRO */
      }
      else if (!strncmp(segnr_str,"EPI_",4))
      {
        if (!*xh->chan)
          xh->special='E';                    /* group EPI */
        else
          xh->special='e';                    /* channel EPI */
      }
      else if (!strcmp(xh->satsrc,"Srvc"))    /* service: segmentnr=servicenr */
      {  /* ignore */
        xh->segment=0;
      }
      else
      {
        xh->segment=atoi(segnr_str); 
      }
    }

    if (itime)
    {
      char tmp[20],*p;
      strcpy(xh->itime,itime);
      memset(&xh->time,0,sizeof(xh->time));
      p=xh->itime;
      strncpy(tmp,p,4); tmp[4]=0; p+=4;
      xh->time.tm_year=atoi(tmp)-1900;

      strncpy(tmp,p,2); tmp[2]=0; p+=2;
      xh->time.tm_mon=atoi(tmp)-1;

      strncpy(tmp,p,2); tmp[2]=0; p+=2;
      xh->time.tm_mday=atoi(tmp);

      strncpy(tmp,p,2); tmp[2]=0; p+=2;
      xh->time.tm_hour=atoi(tmp);

      strncpy(tmp,p,2); tmp[2]=0; p+=2;
      xh->time.tm_min=atoi(tmp);

      mday_mon2yday(&xh->time); 
    }
    
    xh->tlen=0;
    if (crypt)
    {
      xh->compr=xh->encry='_';
      if (strchr(crypt,'C')) xh->compr='C';
      if (strchr(crypt,'E')) xh->encry='E';
    }

    /* Determine sort-order number: [yyyymmddhhmm][t][c] MUST be always equal length! */
    channame2nr(xh);
    if (xh->chan_nr>=0)
      sprintf(xh->sortn,"%s%c%x%02x",xh->itime,xh->hl,xh->chan_nr,xh->segment);
    else
      sprintf(xh->sortn,"%s%c%s%02x",xh->itime,xh->hl,xh->chan,xh->segment);
    sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
    strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
    change_channelnames(xh);  // also extracts satpos from file name
    ret=1;
  }
  clear_str_rule(&keys);
  return ret;
}
 
/*************************************
 * Extract annotation info AVHRR METOP
 * Examples:
 * 
 * AVHR_xxx_1B_M02_20060528092503Z_20060528092803Z_N_O_20060528093309Z.bz2
 *    xxx=
 *    1B
 *    MO2
 *    200605280925   starttime
 *    03Z
 *    200605280928   stoptime
 *    03Z
 *    N_O
 *    200605280933
 *    09Z
 *
 * return: 1
 *************************************/
static int extract_anno_mtp(XRIT_HDR *xh)   /*METOP*/
{
  int n;
  xh->segment=1; 
  xh->image_iformat='Z';
  xh->file_type=0;                          /* means: pic */
  xh->scan_dir='n';                         /* maybe 's'? */

  n=12;
  
  strcpy(xh->satsrc,"METOP"); 
  if (!strncmp(get_strpart(xh->anno,n+0,n+3),"M02",3))
  {
    if (!strncmp(xh->anno,"AVHR_HRP",strlen("AVHR_HRP")))
      strcpy(xh->satsrc,"metop-A");         // hrpt format
    else
      strcpy(xh->satsrc,"METOP-A");         // mtp format
    xh->hl='M';                               /* metop */
  }
  else if (!strncmp(get_strpart(xh->anno,n+0,n+3),"M01",3))
  {
    strcpy(xh->satsrc,"METOP-B");
    xh->hl='M';                               /* metop */
  }
  else if (!strncmp(get_strpart(xh->anno,n+0,n+3),"M03",3))
  {
    strcpy(xh->satsrc,"METOP-C");
    xh->hl='M';                               /* metop */
  }
  else  if (!strncmp(get_strpart(xh->anno,n+0,n+3),"N1",2))
  {
    sprintf(xh->satsrc,"NOAA%s",get_strpart(xh->anno,n+1,n+2));
    xh->hl='G';
  }
  else
  {
    strcpy(xh->satsrc,"METOP-?"); 
    xh->hl='M';                               /* metop */
  }

  n=16;
  xh->time.tm_year=atoi(get_strpart(xh->anno,n+0,n+3))-1900;
  xh->time.tm_mon=atoi(get_strpart(xh->anno,n+4,n+5))-1;
  xh->time.tm_mday=atoi(get_strpart(xh->anno,n+6,n+7));
  xh->time.tm_hour=atoi(get_strpart(xh->anno,n+8,n+9));
  xh->time.tm_min=atoi(get_strpart(xh->anno,n+10,n+11));
  xh->time.tm_sec=0;
  mday_mon2yday(&xh->time);                /* alternative for mktime */

  n=32;
  xh->etime.tm_year=atoi(get_strpart(xh->anno,n+0,n+3))-1900;
  xh->etime.tm_mon=atoi(get_strpart(xh->anno,n+4,n+5))-1;
  xh->etime.tm_mday=atoi(get_strpart(xh->anno,n+6,n+7));
  xh->etime.tm_hour=atoi(get_strpart(xh->anno,n+8,n+9));
  xh->etime.tm_min=atoi(get_strpart(xh->anno,n+10,n+11));
  xh->etime.tm_sec=0;
  mday_mon2yday(&xh->etime);                /* alternative for mktime */

  xh->tlen=difftime_tm(&xh->time,&xh->etime);
  strcpy(xh->itime,get_strpart(xh->anno,16,27));

  strcpy(xh->sat,"MSG");
  strcpy(xh->src,xh->satsrc);    
  strtoupper(xh->src);
  
  xh->compr=xh->encry='_';
  if (!strcmp(xh->anno+strlen(xh->anno)-4,".bz2"))
    xh->compr='C';                                /* compressed */

/* Determine sort-order number: [yyyymmddhhmm][t][c] MUST be always equal length! */
  channame2nr(xh);
  strcpy(xh->chan,A_CH1); xh->chan_nr=1;
  if (xh->chan_nr>=0)
    sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,xh->segment);
  else
    sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,xh->segment);
  sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
  strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
  return 1;
}
 
/*************************************
 * Extract annotation info AVHRR NOAA
 * same as for METOP
 *************************************/
static int extract_anno_mavh(XRIT_HDR *xh)  /*NOAA*/
{
  extract_anno_mtp(xh);

  xh->image_iformat='y';
//  xh->hl='m';                               /* ??? metop avhrr */
  return 1;
}


/*************************************
 * Extract annotation info AVHRR NOAA
 * Examples:
 * 
 * avhrr_20060320_233600_noaa17.hrp.bz2     1 min segment 23:36
 * avhrr_20060320_233700_noaa17.hrp.bz2     1 min segment 23:37
 * avhrr_20060320_233800_noaa17.hrp.bz2     1 min segment 23:38
 *************************************/
static int extract_anno_avh(XRIT_HDR *xh)  /*NOAA*/
{
  char *p;
  int n;
  if (strcmp(get_strpart(xh->anno,28,31),".hrp")) return 0;

  xh->hl='A';                               /* avhrr */
  xh->segment=1; 
  xh->image_iformat='z';
  xh->file_type=0;                          /* means: pic */
  xh->scan_dir='n';                         /* maybe 's'? */
  
  n=6;
  xh->time.tm_year=atoi(get_strpart(xh->anno,n+0,n+3))-1900;
  xh->time.tm_mon=atoi(get_strpart(xh->anno,n+4,n+5))-1;
  xh->time.tm_mday=atoi(get_strpart(xh->anno,n+6,n+7));
  n++;
  xh->time.tm_hour=atoi(get_strpart(xh->anno,n+8,n+9));
  xh->time.tm_min=atoi(get_strpart(xh->anno,n+10,n+11));
  xh->time.tm_sec=0;
  mday_mon2yday(&xh->time);                /* alternative for mktime */
  xh->tlen=60;

  strcpy(xh->itime,get_strpart(xh->anno,6,13));
  strcat(xh->itime,get_strpart(xh->anno,15,18));

  strcpy(xh->satsrc,get_strpart(xh->anno,22,27)); 
  strcpy(xh->sat,"MSG");
  strcpy(xh->src,xh->satsrc);    
  strtoupper(xh->src);

  if ((p=strstr(xh->src,"NOAA"))) *(xh->src+4)=0;
  
  if (!strcmp(get_strpart(xh->anno,32,35),".bz2"))
    xh->compr='C';

  xh->compr=xh->encry='_';
  if (!strcmp(xh->anno+strlen(xh->anno)-4,".bz2"))
    xh->compr='C';                                /* compressed */

/* Determine sort-order number: [yyyymmddhhmm][t][c] MUST be always equal length! */
  channame2nr(xh);
  strcpy(xh->chan,A_CH1); xh->chan_nr=1;
  if (xh->chan_nr>=0)
    sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,xh->segment);
  else
    sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,xh->segment);

  snprintf(xh->id,MAXLEN_ID,"%-10s %c   ",xh->chan,xh->hl);
  strftime(xh->id+14,MAXLEN_ID-14,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
  return 1;
}

/*************************************
 * Extract annotation info bufr
 * (still ok?)
 *************************************/
static int extract_anno_bufr(XRIT_HDR *xh)  /*BUFR*/
{
  char *p,*q;
  strcpy(xh->sat,"MSG");
  strcpy(xh->src,"BUFR");
  strcpy(xh->satsrc,"BUFR");
  xh->hl='B';
  if (!(p=strchr(xh->anno,'_'))) return 0;
  strcpy(xh->chan,get_strpart(xh->anno,0,p-xh->anno-1));
  p++;
  xh->time.tm_year=atoi(get_strpart(p,2,3))+100;
  xh->time.tm_mon=atoi(get_strpart(p,4,5))-1;
  xh->time.tm_mday=atoi(get_strpart(p,6,7));

  xh->time.tm_hour=atoi(get_strpart(p,9,10));
  xh->time.tm_min=atoi(get_strpart(p,11,12));
  xh->time.tm_sec=0;
  mday_mon2yday(&xh->time);                /* alternative for mktime */
  strcpy(xh->itime,get_strpart(p,0,7));
  strcat(xh->itime,get_strpart(p,9,12));
  if (!(p=strchr(p+1,'_'))) return 0;
  if (!(p=strchr(p+1,'_'))) return 0;
  if (!(q=strchr(p+1,'_'))) return 0;
  strcpy(xh->satsrc,get_strpart(p,1,q-p-1));

  sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
  strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */

  sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,0);
  return 1;
}


/*************************************
 * Extract annotation info dwsat
 * (still ok?)
 *************************************/
static int extract_anno_dwdsat(XRIT_HDR *xh)  /*DWDSAT*/
{
  char *p;
  int n=0;

  strcpy(xh->sat,"MSG");
  strcpy(xh->src,"DWDSAT");
  strcpy(xh->satsrc,"DWDSAT");
  xh->hl=xh->anno[0];
  if (!strncmp(xh->anno,"grb",3)) xh->hl='R';
  if (!strcmp(xh->anno+strlen(xh->anno)-4,"tiff"))
  {
    xh->image_iformat='t';
    xh->segment=0; 
    xh->file_type=0;                          /* means: pic */
    xh->nb=8;            /* # bitplanes */
  }
  else if (strstr(xh->anno,".pl"))
  {
    xh->image_iformat='z';                    /* means: bz2 */
    xh->file_type=0;      
  }
  else
  {
    xh->file_type=2;                          /* means: txt */
  }
  
  if (strstr(xh->anno,".pl"))
  {
    strcpy(xh->chan,get_strpart(xh->anno,0,2));
    strcat(xh->chan,"_");
    strcat(xh->chan,get_strpart(xh->anno,15,17));
    xh->time.tm_year=atoi(get_strpart(xh->anno,4,5))+100;
    xh->time.tm_mon=atoi(get_strpart(xh->anno,6,7))-1;
    xh->time.tm_mday=atoi(get_strpart(xh->anno,8,9));

    xh->time.tm_hour=atoi(get_strpart(xh->anno,10,11));
    xh->time.tm_min=atoi(get_strpart(xh->anno,12,13));
    xh->time.tm_sec=0;
    mday_mon2yday(&xh->time);                /* alternative for mktime */
    strcpy(xh->itime,"20");
    strcat(xh->itime,get_strpart(xh->anno,4,13));
    sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
    strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
  }
  else
  {
    strcpy(xh->chan,get_strpart(xh->anno,6,15));
    if ((p=strchr(xh->anno,'-')) && (p=strchr(p+1,'-')))
    {
      p++;
      xh->time.tm_year=atoi(get_strpart(p,n+0,n+1))+100;
      xh->time.tm_mon=atoi(get_strpart(p,n+2,n+3))-1;
      xh->time.tm_mday=atoi(get_strpart(p,n+4,n+5));

      xh->time.tm_hour=atoi(get_strpart(p,n+6,n+7));
      xh->time.tm_min=atoi(get_strpart(p,n+8,n+9));
      xh->time.tm_sec=0;
      mday_mon2yday(&xh->time);                /* alternative for mktime */
      strcpy(xh->itime,"20");
      strcat(xh->itime,get_strpart(p,0,9));
      sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
      strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
    }
  }
  sprintf(xh->sortn,"%s%c%02x",xh->itime,xh->hl,0);
  return 1;
}

/*************************************
 * Extract annotation info GOES 15 and earlier
 * Examples:
 *
 * gos11chnWV03rgnFDseg001res08dat233053600109.lrit
 * gos12chnWV03rgnUSseg005res04dat128153709981_030.lrit
 *                                ---daynr
 *                                   ----UTC
 *                                       -----??
 *    TEXTdat128155459046_063.lrit
 * NWSTEXTdat138091306287.lrit
 *
 * OR_ABI-L2-CMIPM2-M3C02_G16_s20170861628568_e20170861629026_c20170861629090.lrit
 *        --channr?            ----year
 *                                 ---daynr 
 *                                    ------UTC
 *
 * return: -1! Time info NOT suitable for grouping!
 *************************************/
// ! <...><xx> werkt niet!
#define GOS_RULE1 "<sat>chn<chn>rgn<rng>seg<segnr>res<res>dat<date>.lrit"
#define GOS_RULE2 "<sat>chn<chn>rgn<rng>seg<segnr>res<res>dat<date>_<xxx>.lrit"
#define GOS_RULE3 "<sat>dat<date>.lrit"
// See 'case 4:' xrithdr_extr.c: clip anno to right length!
static int extract_anno_gos(XRIT_HDR *xh)
{
  STR rules,keys;
  int nk;
  memset(&rules,0,sizeof(rules));
  memset(&keys,0,sizeof(keys));
  rules.nr=3;
  rules.str[0]=GOS_RULE1;
  rules.str[1]=GOS_RULE2;
  rules.str[2]=GOS_RULE3;
  rule2key(rules.str[1],&keys);
  if (!(parse_str_rule(xh->anno,rules,&keys)))
  {
    for (nk=0; nk<keys.nr; nk++)
    {
      if (keys.ostr[nk])
      {
        if (!strcmp(keys.str[nk],"<sat>"))   strncpy(xh->sat,keys.ostr[nk],10); 
        if (!strcmp(keys.str[nk],"<chn>"))   strncpy(xh->chan,keys.ostr[nk],5); 
        if (!strcmp(keys.str[nk],"<rng>"))
        {
          strncpy(xh->region,keys.ostr[nk],5); 
          strncpy(xh->src,xh->region,2);         // used in tree
        }
        if (!strcmp(keys.str[nk],"<segnr>")) xh->segment=atoi(keys.ostr[nk]); 
        if (!strcmp(keys.str[nk],"<date>"))
        {
          memset(&xh->itime,0,sizeof(xh->itime));
          strncpy(xh->itime,keys.ostr[nk],7); // "128153709981_030" ==> 1281537
          xh->time=timestr2tm(keys.ostr[nk],"DDDHHMM");  // from filename; in the end extracted from hdr nr. 6!
          // extract time from hdr nr. 6; see extract_lritgoestime
          xh->time.tm_year=2000-1900;       // No time in annotation/filename!
        }
      }
    }
    if (!strncasecmp(xh->sat,"gos",3)) 
      sprintf(xh->satsrc,"GOES%s",xh->sat+3);
    else
      strcpy(xh->satsrc,xh->sat);

    xh->hl='l';
    if (!xh->lfac) xh->scan_dir='n';  // if lfac==0: asume 'n'
  }
  clear_str_rule(&keys);

  return -1; // time in filename not consistent for segments! Fixed lateron. 
}

/*************************************
 * Extract annotation info GOES 16 
 * Format:
 *   single-segment, so no segm-nr:
 *     OR_ABI-L2-CMIPF-M3C14_G16_s20170861545382_e20170861556149_c20170861556231.lrit
 *   GOES16, multi-segm, segmnr starts at 0:
 *     OR_ABI-L2-CMIPF-M3C14_G16_s20170861545382_e20170861556149_c20170861556231-img16166-seg14.lrit
 *   GOES17, multi-segm, segmnr starts at 0:
 *     OR_ABI-L2-CMIPF-M6C13_G17_s20190571750341_e20190571759419_c20190571759464_000.lrit
 * return: normally 1
 *************************************/

#define GOES_RULE1 "OR_ABI-L2-<dat>-<chan>_<sat>_<tim_s>_<tim_e>_<tim_c>-<...>-seg<segnr>.lrit"
#define GOES_RULE2 "OR_ABI-L2-<dat>-<chan>_<sat>_<tim_s>_<tim_e>_<tim_c>_<segnr>.lrit"
#define GOES_RULE3 "OR_ABI-L2-<dat>-<chan>_<sat>_<tim_s>_<tim_e>_<tim_c>.lrit"

static int extract_anno_goes16(XRIT_HDR *xh)
{
  STR rules,keys;
  int nk;
  memset(&rules,0,sizeof(rules));
  memset(&keys,0,sizeof(keys));
  rules.nr=3;
  rules.str[0]=GOES_RULE1;
  rules.str[1]=GOES_RULE2;
  rules.str[2]=GOES_RULE3;
  rule2key(rules.str[1],&keys);

  xh->hl='L';
  if (!(parse_str_rule(xh->anno,rules,&keys)))
  {
    char *sat=NULL,*dat=NULL;
    for (nk=0; nk<keys.nr; nk++)
    {
      if (keys.ostr[nk])
      {
        if (!strcmp(keys.str[nk],"<sat>"))     sat=keys.ostr[nk]; 
        if (!strcmp(keys.str[nk],"<dat>"))     dat=keys.ostr[nk]; 
        if (!strcmp(keys.str[nk],"<chan>"))    snprintf(xh->chan,8,"%s",keys.ostr[nk]);
        if (!strcmp(keys.str[nk],"<tim_s>"))
        {
          xh->time=timestr2tm(keys.ostr[nk]+1,"yyyyDDDHHMM");
          snprintf(xh->itime,11,"%s",keys.ostr[nk]);
        }
        if (!strcmp(keys.str[nk],"<segnr>"))   xh->segment=atoi(keys.ostr[nk])+1;
      }
    }
    if ((sat) && (dat))
    {
      snprintf(xh->sat,20,"%s_%s",sat,dat);
      if (!strncasecmp(xh->sat,"G",1))
        sprintf(xh->satsrc,"GOES%s",xh->sat+1);
      else
        strcpy(xh->satsrc,xh->sat);
    }
  }
  clear_str_rule(&keys);
  return 1;
}

/*************************************
 * Extract annotation info JMA
 * Examples:
 * 
 * IMG_DK01IR1_200804080330_001 
 *
 * return: normally 1
 *************************************/
#define JMA_RULE "IMG_<satchan>_<time>_<segnr>"
static int extract_anno_jma(XRIT_HDR *xh)
{
  STR rules,keys;
  int nk;
  memset(&rules,0,sizeof(rules));
  memset(&keys,0,sizeof(keys));
  rules.nr=1;
  rules.str[0]=JMA_RULE;
  rule2key(rules.str[0],&keys);

  xh->hl='J';
  if (!(parse_str_rule(xh->anno,rules,&keys)))
  {
    for (nk=0; nk<keys.nr; nk++)
    {
      if (keys.ostr[nk])
      {
        if (!strcmp(keys.str[nk],"<satchan>"))
        {
          strncpy(xh->satsrc,keys.ostr[nk],4); xh->satsrc[4]=0;         // DK01
          strcpy(xh->sat,xh->satsrc); 
          strcpy(xh->src,xh->sat); 
          if (strlen(keys.ostr[nk])>4) strcpy(xh->chan,keys.ostr[nk]+4); // IR1
        } 
        if (!strcmp(keys.str[nk],"<time>"))
        {
          strcpy(xh->itime,keys.ostr[nk]);
          xh->time=timestr2tm(keys.ostr[nk],"yyyymmddHHMM");

          // Roud to 10's of minute (Himawari has not fixed times)
          xh->time.tm_min=(xh->time.tm_min/10)*10;

          // correct itime; used to make grp->id for collecting data; see mk_dbase
          snprintf(xh->itime+strlen(xh->itime)-2,3,"%02d",xh->time.tm_min);
          // mktime(&xh->time); don'tuse becaus of daylight saving issues!
          mday_mon2yday(&xh->time);                /* alternative for mktime */
        } 
        if (!strcmp(keys.str[nk],"<segnr>"))
        {
          xh->segment=atoi(keys.ostr[nk]);
        }
      }
    }
    channame2nr(xh);
  }
  else return 0;
  if (!xh->segment) xh->segment=1;           // new mtsat: no segment info
  clear_str_rule(&keys);
  return 1;
}

/*************************************
 * Extract annotation info Taylor format
 * Examples:
 *
 * 200603291130-msg-ch01.jpg        same for ch02---ch11
 * 200603291130-msg-ch12w.jpg       corrected HRV
 *************************************/
#define TAY_RULE "<time>-<sat>-<chan>.<ext>"
static int extract_anno_tay(XRIT_HDR *xh)
{
  STR rules,keys;
  int nk;
  int ret=0;
  memset(&rules,0,sizeof(rules));
  memset(&keys,0,sizeof(keys));
  rules.nr=1;
  rules.str[0]=TAY_RULE;
  rule2key(rules.str[0],&keys);

  xh->hl='H';
  xh->segment=1; 

  if (!(parse_str_rule(xh->anno,rules,&keys)))
  {
    for (nk=0; nk<keys.nr; nk++)
    {
      if (keys.ostr[nk])
      {
        if (!strcmp(keys.str[nk],"<ext>"))
        {
          if (!strcmp(keys.ostr[nk],"pgm"))
            xh->image_iformat='p';
          else
            xh->image_iformat='j';
        }
        if (!strcmp(keys.str[nk],"<time>"))
        {
          strcpy(xh->itime,keys.ostr[nk]);
          xh->time=timestr2tm(keys.ostr[nk],"yyyymmddHHMM");
          mday_mon2yday(&xh->time);   /* don't use mktime because of daylight saving! */
        }
        if (!strcmp(keys.str[nk],"<sat>"))
        {
          strcpy(xh->sat,keys.ostr[nk]);                     /* copy 1 more to get closing 0 */
          if (!strcmp(xh->sat,"msg")) strcpy(xh->sat,"MSG"); 
          strcpy(xh->src,xh->sat);    
          if (!strncmp(xh->src,"MSG",3))
            strcpy(xh->satsrc,xh->src);
          else if (!strncmp(xh->src,"SERVICE",7))
            strcpy(xh->satsrc,"Srvc");
          else if (!strncmp(xh->src,"MPEF",7))
            strcpy(xh->satsrc,xh->src);
          else if (!strncmp(xh->src,"MET",3))
            strcpy(xh->satsrc,xh->src);
          else if (!strncmp(xh->src,"GOES",4))
            strcpy(xh->satsrc,xh->src);
          else if (!strncmp(xh->src,"MTSAT",5))
            strcpy(xh->satsrc,xh->src);
          else
            strcpy(xh->satsrc,"Frgn");
        }
        if (!strcmp(keys.str[nk],"<chan>"))      // ch01, ch12w
        {
          xh->chan_nr=atoi(keys.ostr[nk]+2);
          // For MDM suffixes: see 'prepare.c'
          if (xh->chan_nr==12) strncpy(xh->mdm_pos_code,keys.ostr[nk]+4,3);
          /* Determine sort-order number: [yyyymmddhhmm][t][c] MUST be always equal length! */
          if ((xh->chan_nr>=1) && (xh->chan_nr <=12))
          {
            strcpy(xh->chan,channellist_msg[xh->chan_nr-1]);
          } 
          if (xh->chan_nr>=0)
            sprintf(xh->sortn,"%s%c%x%02x",xh->itime,xh->hl,xh->chan_nr,xh->segment);
          else
            sprintf(xh->sortn,"%s%c%s%02x",xh->itime,xh->hl,xh->chan,xh->segment);
          sprintf(xh->id,"%-10s %c   ",xh->chan,xh->hl);
          strftime(xh->id+14,20,"%d-%m-%y %H:%M  ",&xh->time);             /* time */
        }
      }
    }
    ret=1;
  }
  else ret=0;
  clear_str_rule(&keys);
  return ret;
}


/* Macros for detecting type */
#define is_meteo(s) ((!strncmp(s,"MSG",3))? 1 : \
                     (!strncmp(s,"MTP",3))? 1 : 0)
#define is_goes(s)  ((!strncmp(s,"gos",3))? 1 : \
                     (!strncmp(s,"met",3))? 1 : \
                     (!strncmp(s,"mts",3))? 1 : 0)
#define is_goeschrt(s)  ((!strncmp(s,"NWSchr",6))? 1 : 0)
#define is_goestext(s)  ((!strncmp(s,"NWSTEXT",7))? 1 : 0)
#define is_goes16(s)  ((!strncmp(s,"OR_ABI-L2",9))? 1 : 0)

#define is_jma(s) ((!strncmp(s,"IMG_DK",6))? 1 : 0)
#define is_jpeg(s)  ((!strncasecmp(s+strlen(s)-3,"jpg",3))? 1 : 0)
#define is_pgm(s)   ((!strncasecmp(s+strlen(s)-3,"pgm",3))? 1 : 0)
#define is_avhrr(s) (((!strncmp(s,"avhrr",5)) && (strstr(s,"hrp")))? 1 : 0)
#define is_avhrrmet(s) ((!strncmp(s,"AVHR_HRP",8)) ? 1 : 0)
#define is_eps(s) ((!strncmp(s,"AVHR_",5))? 1 : 0)
#define is_bufr(s)  ((!strncmp(s+strlen(s)-4,"bufr",4))? 1 : 0)

#define is_dwdsat(s) (((!strncmp(s,"fx301",5)) || (!strncmp(s,"fx401",5)) || \
                       (!strncmp(s,"ps401",5)) || (!strncmp(s,"cgm01",5)) || \
                       ((!strncmp(s,"gts",3)) && (strncmp(s,"gts_",3)))   || \
                       (!strncmp(s,"rad01",5)) || \
                       (!strncmp(s,"sat01",5)) || (!strncmp(s,"sap01",5)) || \
                       (!strncmp(s,"wst11",5)) || (!strncmp(s,"wst12",5)) || \
                       (!strncmp(s,"buf01",5)) || (!strncmp(s,"grb01",5)) || \
                       (strstr(s,".pl3.bz2"))  || (strstr(s,".pl4.bz2"))  || \
                       (strstr(s,".pl5.bz2"))) \
                                        ? 1 : 0)
#define is_message(s)  (((!strncmp(s,"E-UNS_",6)) && (!strcmp(s+strlen(s)-4,".xml")))? 1 : 0)

/* DWDSAT files:
   plotted obs.          : fx301, fx401
   analyses              : fx301, fx401
   forecast              : fx301, fx401, ps401
   diagnoses             : fx301, fx401
   meteograms            : fx301, fx401
   spec. prod            : cgm01, fx401
   sat images            : sat01, sap01, rad01
   prod observ           : fx401
   EGRR                  : fx401
   prod shipping         : fx401
   single bulletins ASCII: gts10, wst11, wst12
   single bulletins BUFR : buf01
   bulletin container    : gts02,gts06,gts05,gts11,gts08
   GRIB container        : grb01
   NIN JO DATA           : grb01, o_gmosw
*/

/*************************************
 * Extract annotation info 
 * Extracted:
 *   type (HRIT/LRIT/AVHRR etc.)
 *   time (needed for sorting etc.)
 * Used first on each file name (fast access), 
 *   then on annotation record
 *************************************/
int extract_anno(XRIT_HDR *xh)
{
  int ret=0;
  /* Do preselection using disseminatingS/C */
  if (is_meteo(xh->anno+6))           /* disseminated by Meteosat */
    xh->xrit_frmt=EXRIT;
  else if (is_goes(xh->anno))         /* disseminated by GOES */
    xh->xrit_frmt=NXRIT;
  else if (is_goes16(xh->anno))       /* disseminated by GOES 16*/
    xh->xrit_frmt=N2XRIT;
  else if (is_goeschrt(xh->anno))     /* disseminated by GOES, charts */
    xh->xrit_frmt=NXRIT;
  else if (is_goestext(xh->anno))     /* disseminated by GOES, text */
    xh->xrit_frmt=NXRIT;
  else if (is_jma(xh->anno))          /* disseminated by JMA: Japan Meteorological Agency */
    xh->xrit_frmt=JMA;
  else if (is_jpeg(xh->anno))         /* jpeg (Taylor-format) */
    xh->xrit_frmt=STDFRMT;
  else if (is_pgm(xh->anno))          /* pgm (Taylor-format) */
    xh->xrit_frmt=STDFRMT;
  else if (is_avhrr(xh->anno))        /* NOAA AVHRR */
    xh->xrit_frmt=NHRPT;
  else if (is_avhrrmet(xh->anno))     /* METOP AVHRR */
    xh->xrit_frmt=MHRPT;
  else if (is_eps(xh->anno))          /* METOP AVHRR */
    xh->xrit_frmt=METOP;
  else if (is_dwdsat(xh->anno))       /* DWDSAT */
    xh->xrit_frmt=DWDSAT;
  else if (is_bufr(xh->anno))         /* BUFR */
    xh->xrit_frmt=BUFR;
  else if (is_message(xh->anno))
    xh->xrit_frmt=MESSAGE;
  else
    xh->xrit_frmt=UNKDTYPE;           /* unknown */
  switch(xh->xrit_frmt)
  {
    case EXRIT:                       /* Meteosat format */
      ret=extract_anno_esat(xh);      /*   format L-000-MSG1__-MSG1___... */
    break;
    case NXRIT:                       /* Goes format */
      ret=extract_anno_gos(xh);       /*   format gos12chnVS01rgnSHseg001... */
    break;
    case N2XRIT:                      /* Goes16 format */
      ret=extract_anno_goes16(xh);    /*  format  OR_ABI-L2-CMI...... */
    break;
    case JMA:                         /* JMA format (Japan) */
      ret=extract_anno_jma(xh);       /*   format IMG_DK01IR1_200804080330_001 */
    break;
    case STDFRMT:                     /* Taylor (std Pic format) */
      ret=extract_anno_tay(xh);       /*   format 200603291130-msg-ch01.jpg */
    break;
    case NHRPT:                       /* AVHRR */
      ret=extract_anno_avh(xh);       /*   format avhrr_20060320_233800_noaa17.hrp.bz2 */
    break;
    case MHRPT:                       /* AVHRR */
      ret=extract_anno_mavh(xh);      /*   format AVHR_HRP_1B_M02_20060528092503Z_20060528092803Z_N_O_20060528093309Z.hrp.bz2 */
    break;
    case METOP:                       /* METOP */
      ret=extract_anno_mtp(xh);       /*   format AVHR_xxx_1B_M02_20060528092503Z_20060528092803Z_N_O_20060528093309Z.bz2avhrr_20060320_233800_noaa17.hrp.bz2 */
    break;
    case BUFR:                        /* BUFR */
      ret=extract_anno_bufr(xh);      /*   format e.g. hirs_20070225_1058_noaa17_24282_lan.l1c_bufr */
    break;
    case DWDSAT:                      /* DWDSAT */
      ret=extract_anno_dwdsat(xh);    /*    */
    break;
    case MESSAGE:
      ret=extract_anno_message(xh);    /*    */
    break;
    default:                          /* unknown: assume Eumetsat */
      ret=extract_anno_esat(xh);      /*   format L-000-MSG1__-MSG1___... */
    break;
  }
  // If sub_lon not defined here (from type2 header) do it now
  if (xh->subsat_lon==NO_SATPOS)
  {
    char satid[20];
    if (xh->hl) sprintf(satid,"%c%s",xh->hl,xh->satsrc); else strcpy(satid,xh->satsrc);
    xh->subsat_lon=get_subsat_lon(globinfo.satlist,satid);
  }
  return ret;
}
