#define HRPTLIN_NW
/*******************************************************************
 * Copyright (C) 2003 R. Alblas 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * void update_segment_xrithdr(SEGMENT *segm)
 * void update_group_xrithdr(GROUP *grp)
 * FPFNTMP open_xritimage_bz(char *fn,XRIT_HDR *xrit_hdr,DATATYPE type,CHANNEL *chan)
 * FILE *open_xritimage(char *fn,XRIT_HDR *xrit_hdr,DATATYPE type,CHANNEL *chan)
 * int fn2xrit(char *fn,XRIT_HDR *xh)
 ********************************************************************/
/*******************************************************************
 * Basic functions to extract info from XRIT files
 ********************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "vcdu.h"
#include "xrit2pic.h"
#include "avhrr.h"

/*************************************
 * Extraction XRIT header info 
 * 
 *************************************/
extern GLOBINFO globinfo;
extern char *channellist_msg[];


static struct tm time2tm(char *p)
{
  struct tm tm;
  memset(&tm,0,sizeof(tm));
  tm.tm_year=atoi(get_strpart(p,0,3))-1900;
  tm.tm_yday=atoi(get_strpart(p,4,6))-1;
  yday2mday_mon(&tm);
  tm.tm_hour=atoi(get_strpart(p,7,8));
  tm.tm_min=atoi(get_strpart(p,9,10));
  return tm;
}

static void catch_primhdr(unsigned char *l, XRIT_HDR *xrit_hdr)
{
  xrit_hdr->file_type=l[3]; /* 0=image,1=GTS mess.,2=text,3=encr. mess.*/
/* Total header length of this file */
  xrit_hdr->hdr_len=(l[4]<<24)+(l[5]<<16)+(l[6]<<8)+l[7];


/* Total content length of this file */
  xrit_hdr->datalen_msb=(l[8]<<24)+(l[9]<<16)+(l[10]<<8)+l[11];
  xrit_hdr->datalen_lsb=(l[12]<<24)+(l[13]<<16)+(l[14]<<8)+l[15];
  xrit_hdr->data_len=(xrit_hdr->datalen_lsb >> 3) +
                     (xrit_hdr->datalen_msb << 5);
}


// Time of frame start = 2007/233/05:30:13;
static void extract_lritgoestime(char *l,char *time)
{
  char *key="Time of frame start = ";
  if (!strncmp(l,key,strlen(key)))
  {
    l+=strlen(key);
    strcpy(time,get_strpart(l,0,3));
    strcat(time,get_strpart(l,5,7));
    strcat(time,get_strpart(l,9,10));
    strcat(time,get_strpart(l,12,13));
    strcat(time,get_strpart(l,15,16));
  }
}



/*************************************
 * extract pix2tekp mapping for foreign sats
 * See: LRIT_HRIT_specific_implementation.pdf
 *        chapter 4.3.2.4: Header Type #3 - Image Data Function
 *      and
 *        A1.1.3 Example of an FSD Image Data Function (Header #3)
 *
 * format (ASCII):
 *   $HALFTONE:=10<CR>>LF>
 *   _NAME:=albedo<CR>>LF>
 *   _UNIT:=percent<CR>>LF>
 *   0:=0.0<CR>>LF>
 *   1023:=100.0<CR>>LF>
 *
 * or
 *   $HALFTONE:=10<CR>>LF>
 *   _NAME:=calibrated infrared<CR>>LF>
 *   _UNIT:=degree Kelvin<CR>>LF>
 *   0:=170.0<CR>>LF>
 *   1023:=340.0<CR>>LF>
 *
 * or:
 *   $HALFTONE:=8<CR>>LF>
 *   _NAME:=calibrated infrared<CR>>LF>
 *   _UNIT:=degree Kelvin<CR>>LF>
 *   0:=320.000<CR>>LF>
 *   ....
 *   255:=213.838<CR>>LF>
 *
 * or:
 *   $HALFTONE:=16<CR>>LF>
 *   _NAME:=INFRARED
 *   _UNIT:=KELVIN
 *   0:=320.000<CR>>LF>
 *   ...
 *   837:=232.95<CR>>LF>
 *   844:=231.42<CR>>LF>
 *
 * GOES16:
 *   $HALFTONE:=8
 *   _NAME:=toa_brightness_temperature
 *   _UNIT:=K
 *   255:=173.1500
 *************************************/
 /*
 13-5-2012: changed \n to \n\r; GOES versus also MTSAT
 */
 
static void catch_irmap(char *l,CHANNEL *chan)
{
  char *w;
  int pv;
  float pt;
  int i;
  if (!chan) return;
  if (chan->cal.caltbl[0]) return; // table already loaded
  if (!(w=strtok(l+3,":=\n")))  return;   // $HALFTONE
  if (strcmp(w,"$HALFTONE"))    return;
  if (!(w=strtok(NULL,"=\n\r")))   return;  // 10
  chan->cal.caltbl_bpp=atoi(w);

  if (!(w=strtok(NULL,":=\n\r"))) return;   // _NAME 
  if (strcmp(w,"_NAME"))        return;
  if (!(w=strtok(NULL,"=\n\r")))   return;  // "calibrated infrared"
//  w[strlen(w)-1]=0;// waarom laatste char weg???
//  if (strstr(w,"infrared")) cal='i';

  if (!(w=strtok(NULL,":=\n\r"))) return;   // _UNIT 
  if (strcmp(w,"_UNIT"))        return;
  if (!(w=strtok(NULL,"=\n\r")))   return;  // "degree Kelvin" or "percent"
//  w[strlen(w)-1]=0; // waarom laatste char weg???
  if (strcasestr(w,"Kelvin"))   chan->cal.caltbl_type='k';
  if (strcasestr(w,"K"))        chan->cal.caltbl_type='k';  // GOES16
  if (strcasestr(w,"percent"))  chan->cal.caltbl_type='p';
  if (strcasestr(w,"albedo"))   chan->cal.caltbl_type='a';

  chan->cal.caltbl[0]=calloc(256,sizeof(*chan->cal.caltbl[0]));
  chan->cal.caltbl[1]=calloc(256,sizeof(*chan->cal.caltbl[1]));

  if (!chan->cal.caltbl[0]) return;
  for (i=0; i<256; i++)
  {
    if (!(w=strtok(NULL,":=\n\r"))) break;
    pv=atoi(w);
    if (!(w=strtok(NULL,":=\n\r"))) break;
    pt=atof(w);
    chan->cal.caltbl[0][i]=(float)pv;
    chan->cal.caltbl[1][i]=pt;
  }
//  if ((i) && (chan->cal.caltbl[0][i-1]==65535)) i--;
  chan->cal.nrcalpoints=i;
}

static struct tm ccsds2tm(int day,int msecs)
{
  struct tm tm;
  memset(&tm,0,sizeof(tm));
  tm.tm_mday=day+1;
  tm.tm_sec=msecs/1000;
  mktime_ntz(&tm);
  tm.tm_year+=58;

  return tm;
}

// This is 'Creation time'! Not to use for grouping!
// ccsds[1:2]=days from 1-1-1958
// ccsds[3:6]=msecs of day
static void xrit_hdr_ccsds2tm(XRIT_HDR *xrit_hdr)
{
  int day, msecs;
  struct tm tm;
  day=((unsigned char)xrit_hdr->ccsds[1]<<8)+(unsigned char)xrit_hdr->ccsds[2];
  msecs=(xrit_hdr->ccsds[3]<<24)+(xrit_hdr->ccsds[4]<<16)+(xrit_hdr->ccsds[5]<<8)+xrit_hdr->ccsds[6];
  tm=ccsds2tm(day,msecs);
//  tm.tm_min=(tm.tm_min-(tm.tm_min%5));
  xrit_hdr->ccsds_time=tm;
}

/*************************************
 * Extract xrit header
 * Return: rest header (if !=0: error!)
 *************************************/
static int catch_xrit_hdr(unsigned char *l,int ln,XRIT_HDR *xrit_hdr,CHANNEL *chan)
{
  unsigned char *p;
  int hdr_len=0;
  int pos=0;
  memset(xrit_hdr,0,sizeof(*xrit_hdr));
  xrit_hdr->subsat_lon=NO_SATPOS;    // local; from header (needs to open file)
  do
  {
/* Get header type and length */
    xrit_hdr->hdr_type=l[0];
    xrit_hdr->hdr_rec_len=(l[1]<<8)+l[2];

/* Test: header length < length l */
    pos+=xrit_hdr->hdr_rec_len;
    if (pos>ln) break;
/* Extract headertype dependent info */
    switch(xrit_hdr->hdr_type)
    {
/* ------------------ Primary header ------------------ */
      case 0:
        catch_primhdr(l,xrit_hdr);
      break;
/* ------------------ Image header ------------------ */
      case 1:
        xrit_hdr->nb=l[3];           /* # bitplanes */
        xrit_hdr->nc=(l[4]<<8)+l[5]; /* # columns (=width) */
        xrit_hdr->nl=(l[6]<<8)+l[7]; /* # lines */
        xrit_hdr->cf=l[8];           /* compr. flag: 0, 1=lossless, 2=lossy */
      break;
/* ------------------ Image navigation ------------------ */
      case 2:
/* Projection name */
// Format: GEOS(<sub_lon>), <sub_lon>='+/-###.#'
        strncpy(xrit_hdr->proj_name,(char *)(l+3),32); xrit_hdr->proj_name[32]=0;
/* Remove leading spaces */
        for (p=(unsigned char *)(xrit_hdr->proj_name+31); 
             ((*p==' ') && ((char *)p > xrit_hdr->proj_name));
             p--);
        p++; *p=0;
        // Extract subsat-pos from header. This will be used always, also if satpos defined
        if (strcasestr(xrit_hdr->proj_name,"GEOS"))
        {
          char *pp;
          if ((pp=strchr(xrit_hdr->proj_name,'('))) xrit_hdr->subsat_lon=atof(pp+1);
        }
        xrit_hdr->cfac=(l[35]<<24)+(l[36]<<16)+(l[37]<<8)+l[38];
        xrit_hdr->lfac=(l[39]<<24)+(l[40]<<16)+(l[41]<<8)+l[42];
        xrit_hdr->coff=(l[43]<<24)+(l[44]<<16)+(l[45]<<8)+l[46];
        xrit_hdr->loff=(l[47]<<24)+(l[48]<<16)+(l[49]<<8)+l[50];
        if (xrit_hdr->lfac > 0) xrit_hdr->scan_dir='n';
        if (xrit_hdr->lfac < 0) xrit_hdr->scan_dir='s';
      break;
/* ------------------ Image data functions ------------------ */
      case 3:
        if (chan)
        {
          catch_irmap((char *)l,chan);
        }
      break;
/* ------------------ Annotation ------------------ */
      case 4:
/* Extract annotation */
        {
          int len=xrit_hdr->hdr_rec_len;
          if (len>ANNOLEN-2) // err: anno too long! Clip.
          {
            len=ANNOLEN-2;
          }
          strncpy(xrit_hdr->anno,(char *)(l+3),len);
          *(xrit_hdr->anno+len-3)=0;
          extract_anno(xrit_hdr);
        }
      break;
/* ------------------ Time stamp ------------------ */
      case 5:
      {
        int i;
        for (i=0; i<7; i++) xrit_hdr->ccsds[i]=l[3+i]; 
        if (xrit_hdr->xrit_frmt==N2XRIT) xrit_hdr_ccsds2tm(xrit_hdr);
      }
      break;
/* ------------------ ancillary text ------------------ */
      case 6:
        if (xrit_hdr->xrit_frmt==NXRIT)
        {
          extract_lritgoestime((char *)(l+3),xrit_hdr->itime);
          xrit_hdr->time=time2tm(xrit_hdr->itime);
        }
      break;
/* ------------------ key header ------------------ */
      case 7:
      break;
/* ------------------ segment identification ------------------ */
      case 128:
        switch(xrit_hdr->xrit_frmt)
        {
          case EXRIT: case UNKDTYPE:
            /* Eumetsat (if unk: for the moment asume Eumetsat) */
            xrit_hdr->gp_sc_id   =(l[3]<<8)+l[4];   /* unique for each sat. source? (MSG, GOES...) */
            xrit_hdr->spec_ch_id =l[5];
            xrit_hdr->seq_no     =(l[6]<<8)+l[7];   /* segment no. */
            xrit_hdr->seq_start  =(l[8]<<8)+l[9];   /* "planned" start segment */
            xrit_hdr->seq_end    =(l[10]<<8)+l[11]; /* "planned" end segment */
            xrit_hdr->dt_f_rep   =l[12];
          break;
          case JMA:
            if (xrit_hdr->hdr_rec_len>10)   // longer hdr for Himawari via GOES
            { // Himawari via GOES16: different from Him. via EUmetcast or Him. via Him.!
              xrit_hdr->seq_no     =l[6];             /* segment no. */
              xrit_hdr->seq_start  =1;                /* "planned" start segment */
              xrit_hdr->seq_end    =l[12];            /* "planned" end segment is this OK? Only checked for =10 */
              xrit_hdr->dt_f_rep   =0;
            }
            else
            {
              xrit_hdr->seq_no     =l[3];             /* segment no. */
              xrit_hdr->seq_start  =1;                /* "planned" start segment */
              xrit_hdr->seq_end    =l[4];             /* "planned" end segment */
              xrit_hdr->dt_f_rep   =0;
              // (l[5]<<8)+l[6]=line number of image segment
            }
          break;
          case NXRIT: case N2XRIT:
            /* NOAA */
            xrit_hdr->gp_sc_id   =(l[3]<<8)+l[4];   /* unique for each picture?? */
            xrit_hdr->seq_no     =(l[5]<<8)+l[6];   /* segment seq. no. */
            /* start column=(l[7]<<8)+l[8] */
            /* start row   =(l[9]<<8)+l[10] */
            xrit_hdr->seq_start  =1;                /* start segment */
            xrit_hdr->seq_end    =(l[11]<<8)+l[12]; /* max segment */
            /* max column=(l[13]<<8)+l[14] */
            /* max row   =(l[15]<<8)+l[16] */
            if (xrit_hdr->xrit_frmt==N2XRIT)
            { 
              // anno GOES16 has NO segmentnr info! (no 'seg<nr>' stringpart)
              xrit_hdr->segment=xrit_hdr->seq_no+1;
            }
          break;
          default:  // not expected
          break;
       }
      break;
/* ------------------ image segment line quality ------------------ */
      case 129:
      break;
/* ------------------ ??? In GOES LRIT ??? ------------------ */
      case 130:
      break;
/* ------------------ JMA:  Image Observation Time Header ------------------ */
      case 131:
        if (xrit_hdr->xrit_frmt==JMA)
        {
          freed(&xrit_hdr->img_obs_time);
          xrit_hdr->img_obs_time=calloc(xrit_hdr->hdr_rec_len,1);
          strncpy(xrit_hdr->img_obs_time,(char *)(l+3),xrit_hdr->hdr_rec_len-3);
          while ((p=(unsigned char*)strchr(xrit_hdr->img_obs_time,'\r'))) *p=' ';
        }
      break;
/* ------------------ ??? In GOES LRIT ??? ------------------ */
      case 132:
      break;

      default:
        printf("Unexpected hdrtype=%d\n",xrit_hdr->hdr_type);
      break;
    }
    if (xrit_hdr->hdr_type==0)
    {
      hdr_len=xrit_hdr->hdr_len;
    }
    l+=xrit_hdr->hdr_rec_len;

    hdr_len-=xrit_hdr->hdr_rec_len;

  } while (hdr_len>0);
  return hdr_len;
}

static void manage_aspect(XRIT_HDR *xh)
{
  if ((xh->cfac) && (xh->lfac) && (xh->cfac != xh->lfac))
  {
    float aspect_ratio=(float)xh->cfac/(float)xh->lfac;
    if (aspect_ratio > 1.)
    { 
      xh->nl_orig=xh->nl;
      xh->nl*=aspect_ratio;
      xh->lfac=xh->cfac;
      xh->loff*=aspect_ratio;
    }
    if (aspect_ratio < 1.)
    { 
      xh->nc_orig=xh->nc;
      xh->nc/=aspect_ratio;
      xh->cfac=xh->lfac;
      xh->coff/=aspect_ratio;
    }
  }
}

/*************************************************************************
 * Read from a extracted file (channel with certain order number)
 * the XRIT header.
 * Remaining file is e.g. JPEG or Wavelet.
 *************************************************************************/
static int read_xrithdr(FILE *fp,XRIT_HDR *xrit_hdr,CHANNEL *chan)
{
  unsigned char l1[20],*l;
  int hdr_len;
  int hdr_rest=0;
  if (!fp) return 0;

/* Read in primary header, just to determine length of all headers  */
  fread(l1,16,1,fp);
  fseek(fp,-16,SEEK_CUR);

/* Test header; expected primary */
  if (l1[0]!=0) return -1;

  if (((l1[1]<<8) + l1[2]) !=16) return -1;

/* Determine total header length */
  hdr_len=(l1[4]<<24)+(l1[5]<<16)+(l1[6]<<8)+l1[7];

  // mtsat may have very big headers (pos. info for lots of x/y positions) pixel
  if ((hdr_len>50000) || (hdr_len<10)) return -1;

/* Allocate and read all headers */
  if (!(l=malloc(hdr_len))) return -1;
  fread(l,hdr_len,1,fp);

/* Extract header info */
  hdr_rest=catch_xrit_hdr(l,hdr_len,xrit_hdr,chan);
// Nu moet fp precies aan begin image staan.
// Lijkt niet altijd het geval te zijn??
/* Determine image type. Used first 2 bytes of actual image. */
  if (xrit_hdr->file_type==0)
  {
    unsigned char c[2];
    fread(c,2,1,fp);
    if ((c[0]==0xff) && (c[1]==0x01))      xrit_hdr->image_iformat='w';
    else if ((c[0]==0xff) && (c[1]==0xd8)) xrit_hdr->image_iformat='j';
    else if (xrit_hdr->image_iformat!='b') xrit_hdr->image_iformat='?'; // also JMA
    fseek(fp,-2,SEEK_CUR);
  }

//  channame2nr(xrit_hdr); // al gedaan in extract_anno

  // matches with correct_for_aspect
  if (globinfo.correct_aspect) manage_aspect(xrit_hdr);
  
  xrit_hdr->done=TRUE;

  free(l);
  return hdr_rest;
}

static void init_noaachars(XRIT_HDR *xrit_hdr)
{
  int nrsecs=60;
  xrit_hdr->nb=10;                 /* # bitplanes */
  xrit_hdr->nc=AVHRR_PICWIDTH*2;   /* # columns (=width) ! Always linearize!*/
#ifdef HRPTLIN_NW
  xrit_hdr->nc=AVHRR_PICWIDTH;
#endif
  xrit_hdr->nl=6*nrsecs;           /* # lines one file */
  xrit_hdr->cf=1;                  /* compr. flag: 0, 1=lossless, 2=lossy */
  xrit_hdr->hdr_rec_len=0;
  xrit_hdr->lines_per_sec=AVHRR_LINESPSEC;
  xrit_hdr->done=TRUE;
}

static void init_metopchars(FILE *fp,XRIT_HDR *xrit_hdr)
{
//  struct tm tm1=xrit_hdr->time;
//  struct tm tm2=xrit_hdr->etime;
//  int nrsecs=mktime(&tm2)-mktime(&tm1);
  int width,height,lps;
  get_eps_picsize(fp,&width,&height,&lps);
  if (!width) width=2048;
  if (!height) height=1080;
  xrit_hdr->nb=14;               /* # bitplanes */
  xrit_hdr->nc=width;            /* # columns (=width) Don't lin, or lin. with expand_l=1! */
  xrit_hdr->nl=height;           /* # lines one file */
  xrit_hdr->cf=1;                /* compr. flag: 0, 1=lossless, 2=lossy */
  xrit_hdr->lines_per_sec=lps;
  xrit_hdr->hdr_rec_len=0;
  xrit_hdr->done=TRUE;
}

static void init_metopavhrrchars(XRIT_HDR *xrit_hdr)
{

  xrit_hdr->nb=10;               /* # bitplanes */
  xrit_hdr->nc=2048;             /* # columns (=width) Don't lin, or lin. with expand_l=1! */
  xrit_hdr->nl=360;              /* # lines one file */
  xrit_hdr->cf=1;                /* compr. flag: 0, 1=lossless, 2=lossy */
  xrit_hdr->lines_per_sec=0;
  xrit_hdr->hdr_rec_len=0;
  xrit_hdr->done=TRUE;
}


/*************************************************************************
 * Extract/define some pic chars.
 *   JPEG or PGM, generated by (e.g.) MDM.
 *************************************************************************/
static int read_stdpichdr(FILE *fp,XRIT_HDR *xrit_hdr)
{
  int width,height;
  int err=0;
  switch (xrit_hdr->image_iformat)
  {
    case 'j':
      err=get_jpegsize_opened(fp,&width,&height);
    break;
    case 'p':
     err=get_pgmsize_opened(fp,&width,&height);
    break;
  }

  xrit_hdr->nb=8;           /* # bitplanes */
  xrit_hdr->nc=width;       /* # columns (=width) */
  xrit_hdr->nl=height;      /* # lines */
  xrit_hdr->cf=2;           /* compr. flag: 0, 1=lossless, 2=lossy */
//  xrit_hdr->gp_sc_id   =(l[3]<<8)+l[4];   /* unique for each sat. source? (MSG, GOES...) */
//  xrit_hdr->spec_ch_id =l[5];
  xrit_hdr->seq_no     =1;   /* segment no. */
  xrit_hdr->seq_start  =1;   /* "planned" start segment */
  xrit_hdr->seq_end    =1;   /* "planned" end segment */
//  xrit_hdr->dt_f_rep   =0;
  xrit_hdr->hdr_rec_len=0;
  xrit_hdr->done=TRUE;

  if (xrit_hdr->nc==3712)                 // Assume MSG channels 1...11
  {
    xrit_hdr->coff=1856;
    xrit_hdr->cfac=13642337;
  }
  else if (xrit_hdr->nc==11136)           // Assume MSG channels 12
  {
    xrit_hdr->coff=5566;
    xrit_hdr->cfac=40927014;
  }
  else if (xrit_hdr->nc > 5566)
  {
    xrit_hdr->coff=xrit_hdr->nc-5566;                  // first segment
    xrit_hdr->cfac=40927014;
  }

  if (xrit_hdr->nl==3712)                 // Assume MSG channels 1...11
  {
    xrit_hdr->loff=1856;                  // first segment
    xrit_hdr->lfac=13642337;
  }
  else if (xrit_hdr->nl==11136)           // Assume MSG channel 12
  {
    xrit_hdr->loff=5566;                  // first segment
    xrit_hdr->lfac=40927014;
  }
  else if (xrit_hdr->nl > 3712)
  {
    xrit_hdr->loff=5566;                  // first segment
    xrit_hdr->lfac=40927014;
  }
  
  else if (xrit_hdr->nl==1392)            // Assume MSG channels 1..11, RSS
  {
    xrit_hdr->loff=-464;                  // first segment
    xrit_hdr->lfac=13642337;
  }
  else if (xrit_hdr->nl==4176)            // Assume MSG channel 12, RSS
  {
    xrit_hdr->loff=-1394;                  // first segment
    xrit_hdr->lfac=40927014;
  }
  return err;
}

static void init_dwdsat(XRIT_HDR *xh)
{
  xh->seq_no     =1;   /* segment no. */
  xh->seq_start  =1;   /* "planned" start segment */
  xh->seq_end    =1;   /* "planned" end segment */
}

static int update_hdr(FILE *fp,DATATYPE type,XRIT_HDR *xrit_hdr,CHANNEL *chan)
{
  int err=0;

  switch(type)
  {
    case STDFRMT:
      if (read_stdpichdr(fp,xrit_hdr))
      {
        err=-1;
//        fclose(fp);
//        fp=NULL;
      }
    break;
    case NHRPT:
      init_noaachars(xrit_hdr);
    break;
    case MHRPT:
      init_metopavhrrchars(xrit_hdr);
    break;
    case METOP:
      init_metopchars(fp,xrit_hdr);
    break;
    case DWDSAT:
      init_dwdsat(xrit_hdr);
    break;
    case BUFR:
    break;
//    case UNKDTYPE:  // unk.: treat as xrit!
//    break;
    default:
      if ((read_xrithdr(fp,xrit_hdr,chan)))
      { // error
        err=-1;
//        if (fp) fclose(fp);
//        fp=NULL;
      }
      // JMA: plain 16 bpp, but actually used 10 bpp(?)
      if ((type==JMA) && (xrit_hdr->nb==16)) xrit_hdr->nb=10;

    break;
  }
  return err;
}

// update one segment; uses content file
void update_segment_xrithdr(SEGMENT *segm)
{
  FILE *fp;
  if (!segm) return;
  if (!segm->pfn) return;
  if ((fp=fopen(segm->pfn,"rb")))
  {
    read_xrithdr(fp,&segm->xh,NULL);
    segm->width=segm->xh.nc;
    segm->height=segm->xh.nl;
    fclose(fp);
  }
  
}

// update one channel; uses content file all segments of channel
static void update_channel_xrithdr(CHANNEL *chan)
{
  SEGMENT *segm;
  for (segm=chan->segm; segm; segm=segm->next)
  {
    if (!segm->xh.done) update_segment_xrithdr(segm);
    segm->width=segm->xh.nc;
    segm->height=segm->xh.nl;
  }
}

// update one group; uses content file all segments of all channels of group
void update_group_xrithdr(GROUP *grp)
{
  CHANNEL *chan;
  for (chan=grp->chan; chan; chan=chan->next) update_channel_xrithdr(chan);
}

// Open a file. If zipped: unzip first. (bz2)
// Read headers from file and extract onfo into xrit_hdr
// type: NHRPT, JMA etc.
// chan: if defined then load ir mapping for temperature calc. (catch_irmap)
FPFNTMP open_xritimage_bz(char *fn,XRIT_HDR *xrit_hdr,DATATYPE type,CHANNEL *chan)
{
  FPFNTMP fpfntmp=clear_fpfntmp();
  XRIT_HDR i_xrit_hdr;
  char *fn1;
  gboolean hdr_intern=FALSE;
  memset(&i_xrit_hdr,0,sizeof(i_xrit_hdr));
  i_xrit_hdr.subsat_lon=NO_SATPOS;

  if ((fn1=strrchr(fn,DIR_SEPARATOR))) fn1++; else fn1=fn;

  fpfntmp=cropen_bunzip(fn);
  if (!fpfntmp.fp) return fpfntmp;

  if (!xrit_hdr)
  {
    xrit_hdr=&i_xrit_hdr;
    hdr_intern=TRUE;
  }
  if ((update_hdr(fpfntmp.fp,type,xrit_hdr,chan)))
  {
    rmclose(&fpfntmp);
  }

  if (hdr_intern) freed(&xrit_hdr->img_obs_time);
  return fpfntmp;
}

// Same, but just return filepointer
FILE *open_xritimage(char *fn,XRIT_HDR *xrit_hdr,DATATYPE type,CHANNEL *chan)
{
  FPFNTMP fpfntmp;

  fpfntmp=open_xritimage_bz(fn,xrit_hdr,type,chan);
  return fpfntmp.fp;
}

/*************************************
 * Translate file name into xrit-info 
 * Return: 1 if OK
 *         <=0 if failed
 * If sat has no Meteosat-formatted xrit headers then return 0
 * Mostly this info is enough to group files into chanels and segments
 * (GOES < 16 not...)
 *************************************/
int fn2xrit(char *fn,XRIT_HDR *xh)
{
  char *p;
  int ret;
  if (!fn) return 0;
  if ((p=strrchr(fn,DIR_SEPARATOR))) p++; else p=fn;
  strncpy(xh->anno,p,100);
  ret=extract_anno(xh);
  if (!ret) return 0;   // no info in filename; no xrit file?

  if (xh->xrit_frmt==UNKDTYPE) return 0;
  if (ret<0) return -1; // some info in filename but not all
  return 1;             // extraction from filename OK
}
